<?php

/**
 * OntoWiki Model Controller Class
 *
 * @author AKSW team
 * @version $Id: ModelController.php 2511 2008-08-23 14:33:51Z p_frischmuth $
 */
class ModelController extends OntoWiki_Controller_Action {
    
	/**
	 * Default action when no action is specified
	 * 
	 * shows a list of models, instances or instance
	 */
	public function indexAction() {
		$this->log->debug('ModelController::indexAction()');
	}
	
	/**
     * Action to change the model.
     * Forwards to an info page describing the model selected. 
     */
    public function selectAction() {
		$this->log->debug('ModelController::selectAction()');
		
		// // load the model and store it in the default session namespace
		// try {
		// 	$session = Zend_Registry::get('session');
		// 	$config = Zend_Registry::get('config');
		// } catch (Exception $e) {
		// 	echo get_class($e) . ': ' . $e->getMessage();
		// }
		// if ($model = $this->erfurtStore->getModel($this->_request->getParam('m'))) {
		// 	$session->activeModelUri = $this->_request->getParam('m');
		// 	$config->activeModel = $model;
		// 	$this->log->debug('Model selected: ' . $model->modelURI);
		// }
		
		// redirect to view model as resource
		$redirectUrl = 'model/info/?m=' . urlencode($this->_request->getParam('m'));
	    $this->ed->trigger('onSelectModel', &$redirectUrl);
		$this->_redirect($redirectUrl, array('code' => 302));
	}
	
	public function infoAction() {
		$this->_helper->viewRenderer->setNoRender();
		$this->_forward('view', 'resource', null, array('r' => $this->_request->getParam('m')));
	}
    
	/**
 	  * Action to change the model
 	  * 
 	  * @deprecated Done by the ModelFetch controller plug-in 
      */
	public function selectModel() {
		$this->log->debug('ModelController::selectModel()');

		// load the model and store it in the default session namespace
		try {
			$session = Zend_Registry::get('session');
		} catch (Exception $e) {
			echo get_class($e) . ': ' . $e->getMessage();
		}
		if ($model = $this->erfurtStore->getModel($this->getOwUrl())) {
			$session->activeModel = $model;
			$this->log->debug('Model selected: ' . $session->activeModel->modelURI);
			return true;
		}
		return false;
	}
	
	public function deleteAction() {
        if ($this->ac->isActionAllowed('ModelManagement')) {
            $model = $this->erfurtStore->getModel($this->_request->getParam('m'));
            if ($model) {
                $modelID = $model->modelID;
                
                $this->erfurtStore->deleteModel($this->_request->getParam('m'));
                
                // remove all entries from the popularity table, that match the removed model (id)
        		$this->erfurtStore->dbConn->Execute('
        		    DELETE FROM popularity WHERE modelID = ' . $modelID);   
            } 
        }
        $this->config->activeModel = null;
        $this->session->activeModelUri = '';
        $this->_redirect('', array('code' => 302));
    }
    
/**
     * action to change the model
     * 
     * shows a list of instances for the current model 
     */
    public function useAction() {
		$this->log->debug('ModelController::userAction()');
		if ($this->selectModel()) {
			require_once('InstanceController.php');
			$session = Zend_Registry::get('session');
			$instContrl = new InstanceController($this->_request, $this->_response);
			$instContrl->overviewAction();
			#$url = getRewriteBase() . 'resource/overview/';
			try {
			#	$this->_redirect($url);
			} catch (Exception $e) {
			#echo get_class($e) . ': ' . $e->getMessage();
		}
			return;
		}
		# FIXX: No MODEL
		echo 'Kein gültiges Modell gewählt!';
    }
    
	/** 
     * Show the 404 file not found error page
     */
    public function norouteAction() {
    	$this->log->info('ModelController::noRouteAction()');
        $this->_redirect('/resource/create');
    }
    
	/**
     * Catch action 
     * 
     * Will be called when an undefined action is called
     */
    public function __call($methodName, $args) {
        $this->log->info('ModelController::__call()');
        $this->_redirect('/resource/noroute');
	}
	
	/**
	  * Model import action
	  */
	public function createAction() {
		
		$this->view->title  = $this->strings->model->create->title;
		
		$errors = array();
		if ($post = $this->_request->getPost()) {
			//print_r($post);
			if (!$post['model']) {
				$errors[] = '<em>Error:</em> Model URI must not be empty.';
			} elseif ($this->erfurtStore->modelExists($post['model'])) {
				$errors[] = 'Error: A model with the same URI already exists.';
			} elseif ($_FILES['source']['error'] == 1) {
				$errors[] = 'Upload Error: File size exceeds PHP&apos;s configuration value <code>upload_max_filesize</code>!';
			} elseif ($_FILES['source']['error'] == 3) {
				$errors[] = 'Upload Error: Only parts of the file could be uploaded. Please try again!';
			} else {
				if ($post['form'] == 'empty') {
					// create new empty model
					$m = $this->erfurtStore->getNewModel($post['model'], $post['baseURI'], $post['type'] ? $post['type'] : 'RDF');

					// update model metadata
					$m->setBaseURI($post['baseURI']);
					// ???
					if ($post['type'] == 'OWL') {
						$m->setType($post['type']);
					}
				} else {
					// load from file/url/paste
					if ($post['form'] == 'paste') {
						// paste source
						$file = tempnam('/tmp', 'ow');
						$temp = fopen($file, 'wb');
						fwrite($temp, $post['paste']);
						fclose($temp);
					} elseif ($post['form'] == 'upload') {
						// upload from file
						$file = $_FILES['source']['tmp_name'];
					} elseif ($post['form'] == 'import') {
						// import from URL
						$file = $post['location'] ? $post['location'] : $post['model'];
					}

					// create new model
					try {
						$m = $this->erfurtStore->loadModel($post['model'], $file, $post['loadImports'], false, $post['filetype']);
					} catch (Exception $e) {
						$errors[]  = 'Can\'t create model: ' . $e->getMessage();
					}

					// remove temp file
					if ($post['form'] == 'paste') {
						unlink($file);
					}
				}

				// no model created
				if (!$m) {
					$this->view->errors = $errors;
					return false;
				}

				// set userModelsEdit 
				try {
					$this->erfurtApp->getAc()->setUserModelRight($m->modelURI, 'edit', 'grant');
				} catch (Erfurt_Exception $e) {
					$errors[] = 'Error on setting model permissions: '. $e->getMessage();
				}

				// redirect to model select
				$this->_redirect('/model/select/?m=' . urlencode($m->modelURI), array('code' => 302));
			}
		}
		
		$this->view->owUriBase = $this->config->uriBase;
        $this->view->filetypes = array('' => 'Autodetect', 'rdf' => 'RDF/XML', 'n3' => 'N3', 'nt' => 'N-Triples', 'grddl' => 'GRDDL');
        
        /* @trigger ontowiki_model_create_filetypes
         * @param array $filetypes
         */
        $this->ed->trigger('ontowiki_model_create_filetypes', &$this->view->filetypes);
		
		$this->view->errors = $errors;
	}
	
	public function addAction() {
		
		$errors = array();
		
		if ($post = $this->_request->getPost()) {
			// print_r($post);
			// exit();
			if ($_FILES['source']['error'] == 1) {
				$errors[] = 'Upload Error: File size exceeds PHP&apos;s configuration value <code>upload_max_filesize</code>!';
			} elseif ($_FILES['source']['error'] == 3) {
				$errors[] = 'Upload Error: Only parts of the file could be uploaded. Please try again!';
			} else {
				// load from file/url/paste
				if ($post['form'] == 'paste') {
					// paste source
					$file = tempnam('/tmp', 'ow');
					$temp = fopen($file, 'wb');
					fwrite($temp, $post['paste']);
					fclose($temp);
				} elseif ($post['form'] == 'upload') {
					// upload from file
					$file = $_FILES['source']['tmp_name'];
				}

				// create new model
				$m = $this->erfurtStore->getModel($this->_request->getParam('m'));
				$tempModel = new MemModel();
				$tempModel->load($file, $post['filetype'], false);
				$m->addModel($tempModel);
				
				// remove temp file
				if ($post['form'] == 'paste') {
					unlink($file);
				}
			}

			// redirect to model select
			$this->_redirect('/model/select/?m=' . urlencode($m->modelURI), array('code' => 302));
		}
	
		
		$this->view->model = $this->_request->getParam('m');
		$this->view->owUriBase = $this->config->uriBase;
        $this->view->filetypes = array('' => 'Autodetect', 'rdf' => 'RDF/XML', 'n3' => 'N3', 'nt' => 'N-Triples', 'grddl' => 'GRDDL');

        /* @trigger ontowiki_model_add_filetypes
         * @param array $filetypes
         */
        $this->ed->trigger('ontowiki_model_add_filetypes',&$this->view->filetypes);
		
		$this->view->title  = $this->strings->model->create->title;
		$this->view->errors = $errors;
	}
	
	public function exportAction() {
	    
		// check whether the required parameters are given... if not 400 bad request
        if (!$this->_request->getParam('output')) {
            header('HTTP/1.0 400 Bad Request');
            echo '400 Bad Request';
            exit;
        }
        
        $type = strtolower($this->_request->getParam('output'));
        $supportedTypes = array('xml', 'rdfxml', 'rdf+xml', 'json', 'rdfjson', 'rdf+json', 'n3', 'nt', 'rdfn3',
                'rdfnt', 'rdf+n3', 'rdf+nt');
        
        // check whether output type is supported... if not 400 bad request
        if (!in_array($type, $supportedTypes, true)) {
            header('HTTP/1.0 400 Bad Request');
            echo '400 Bad Request';
            exit;
        }
        
        // check whether a model uri is given
        if ($this->_request->getParam('m')) {
            $modelUri = $this->_request->getParam('m');
            
            // check whether model exists... if not 404 not found
            if (!$this->erfurtStore->modelExists($modelUri, false)) {
                header('HTTP/1.0 404 Not Found');
                echo '404 Not Found';
                exit;
            }
            
            // try to retrieve the model object with acl activated
            $m = $this->erfurtStore->getModel($this->_request->getParam('m'));

            // check whether $m is not false... if it is the case... 403 forbidden
            if ($m == false) {
                header('HTTP/1.0 403 Forbidden');
                echo '403 Forbidden';
                exit;
            }
        }
        // else use all available models
        else {
            $supportsAllExportTypes = array('xml', 'rdfxml', 'rdf+xml');
            
            if (!in_array($type, $supportsAllExportTypes)) {
                header('HTTP/1.0 400 Bad Request');
                echo '400 Bad Request';
                exit;
            }
            
            $m = $this->erfurtStore->listModels(true);
            
            // if no model is available... 403 Forbidden
            if (count($m) === 0) {
                header('HTTP/1.0 403 Forbidden');
                echo '403 Forbidden';
                exit;
            }
            
            $m = null;
        }
    
        // set a nice filename
        $fileName = 'export' . date('Y-m-d_Hi');
        
        // everything seems to be ok... so handle the different output types
        switch ($type) {
            case 'xml':
            case 'rdfxml':
            case 'rdf+xml':
                $xmlWriter = new Erfurt_Syntax_StringWriterXMLImpl();
                $rdfSerializer = new Erfurt_Syntax_RDFSerializer();
                
                // send rdf/xml content type header
                header('Content-Type: application/rdf+xml');
                header('Content-Disposition: filename="' . $fileName . '.rdf"');
                
                $ad = $this->config->serializer->rdfxml->ad;
                
                echo $rdfSerializer->serializeToString($xmlWriter, $m, $ad);
                exit;
            case 'json':
            case 'rdfjson':
            case 'rdf+json':
// TODO check for large models
                $rdfSerializer = new JsonSerializer();
                $m = $m->getMemModel();
                
                // send rdf/json content type header (no standard yet!!!)
                header('Content-Type: application/rdf+json');
                header('Content-Disposition: filename="' . $fileName . '.json"');
                echo $rdfSerializer->serialize($m);
                exit;
            case 'n3':
            case 'nt':
            case 'rdfn3':
            case 'rdfnt':
            case 'rdf+n3':
            case 'rdf+nt':
// TODO check for large models
                $rdfSerializer = new N3Serializer();
                $m = $m->getMemModel();
                // send rdf/n3 content type header
                header('Content-Type: text/rdf+n3');
                header('Content-Disposition: filename="' . $fileName . '.n3"');
                echo $rdfSerializer->serialize($m);
                exit;
        }
    }
	
	public function emptycacheAction() {
		
		$m = $this->erfurtStore->getModel($this->_request->getParam('m'));
		$c = new stmCache(null, null, $m);
		$c->emptyCache();
		
		$this->_redirect('/model/select/?m=' . urlencode($m->modelURI), array('code' => 302));
	}

	/**
	 * Shows filemanager for specific model
	 */
	public function filemanagerAction() {
		$sysontModel = Zend_Registry::get('sysontModel');
		$fMime = $this -> config -> filemanager -> file -> mimeproperty;
		$fClass = $this -> config -> filemanager -> file -> ofclass;
		$fModel = $this -> config -> filemanager -> file -> ofmodel;

		if (!$this->model) {
			$errors[] = $this->strings->default->nomodel;
		} else {
			if (array_key_exists('action',$_POST) && sizeof($_POST) > 1) {
				foreach ($_POST as $k => $v) {
					if ($k != 'action') {
						$k = rawurldecode($k);
						// Remove statements signaling resource is file from ConfigModel
						$res = $sysontModel -> resourceF($k);
						$res -> remove();
						// Remove statements from selected model too?
						$res = $this -> model -> resourceF($k);
						$res -> remove();
						$pathHashed = REAL_BASE . $this -> config -> filemanager -> path . md5($k);
						unlink($pathHashed);
					} else {
						;
					}

						$pathHashed = REAL_BASE . $this -> config -> filemanager -> path . md5($k);
						unlink($pathHashed);
				}
				$this -> _redirect('/model/filemanager');
			}
			$modeluri =  $this -> model -> getModelUri();

			$query = 'SELECT DISTINCT ?mime ?uri WHERE {?uri <' . $fModel . '> <' . $modeluri . '> . ' .
			'?uri <' . $fMime . '> ?mime . ?uri a <' . $fClass . '> . }';
			
			$filesresult = $sysontModel->sparqlQuery($query);
			
			if (sizeof($filesresult) == 0) {
				$messages[] = $this -> strings -> filemanager -> nofiles;
			} else {			
				foreach ( $filesresult as $result ) {
					$filelist[] = array('uri' => $result['uri'] -> getUri(), 
					'mime' => $result['mime'] -> getUri());
				}
			}
			$this -> view -> filelist = $filelist;
		}
		$this -> view -> title = $this -> strings -> filemanager -> title;
		$this -> view -> messages = $messages;
		$this -> view -> errors = $errors;
	}
	
}

?>
