<?php

/**
 * OntoWiki resource controller class which handles all resource-associated requests.
 *
 * @license GPL
 * @author AKSW team
 * @version $Id: ResourceController.php 2677 2009-01-26 16:39:38Z fusel2k $
 */
class ResourceController extends OntoWiki_Controller_Action {

	private $_filters = null;

	private $_filtersCombined = false;

	private $_maxErgs = 10;

	/**
	 * Initialisation method.
	 */
	public function init() {
		parent::init();

		// redirect if no model has been selected
		if (!($this->model instanceof RDFSModel)) {
			$this->_redirect('');
			return;
		}

		if ($this->model instanceof RDFSModel) {
			try {
				if ($config->activeResource instanceof Erfurt_Rdfs_Resource ||
				isset($this->session->activeResourceUri) ) {
					$this->resource = $this->model->instanceF($this->session->activeResourceUri);
				} else if (isset($this->_request->r)) {
					$this->resource = $this->model->instanceF($this->_request->r);
				}

				// check if resource is a class
				// and forward to list view
				if ($this->resource->isClass()) {
					$this->class = $this->model->classF($this->resource);
					$this->session->activeClassUri = $this->class->getURI();
				}

				#if (($this->model->findNode(null, EF_RDF_TYPE, $this->resource) ||
				#		$this->model->findStatement($this->resource, EF_RDF_TYPE, EF_OWL_CLASS) ||
				#		$this->model->findNode($this->resource, EF_RDFS_SUBCLASSOF, null) ||
				#		$this->model->findNode(null, EF_RDFS_SUBCLASSOF, $this->resource) ||
				#		$this->model->findStatement($this->resource, EF_RDF_TYPE, EF_RDFS_CLASS))) {
				#
				#	$this->class = $this->model->classF($this->resource);
				#	$this->session->activeClassUri = $this->class->getURI();
				#}
			} catch (Exception $e) {
				$this->log->debug('Uncatched Exception: ' . __CLASS__ . ':' .$e->message . PHP_EOL . $e->file . ':' . $e->line);
			}
		}
	}

	/**
	 * Default action when no action is specified
	 *
	 * shows a list of models, instances or instance
	 */
	public function indexAction() {
		$this->log->debug('InstanceController::indexAction()');

		$this->view->assign('content', $this->view->render($this->config->themes->path . 'resource/index.php'));
	}

	/**
	 * Wiki-like creation of not found resources.
	 */
	public function norouteAction() {
		$this->log->debug('InstanceController::noRouteAction()');
		$this->_redirect('/resource/create');
	}

	public function deleteAction() {
		// TODO: check if allowed

		if (isset($this->_request->r) && ($this->_request->r !== '')) {

			$r = $this->model->resourceF($this->_request->r);

			$r->remove();
			$this->resource = null;
			$this->_redirect('/model/info/?m=' . urlencode($this->model->modelURI), array('code' => 302));
		} else {
			throw new Erfurt_Exception('Resource parameter not given.');
		}
	}

	/**
	 * Action that ist executed when a resource which has no instances was selected.
	 */
	public function viewAction() {
		$this->log->debug('InstanceController::viewAction()');

		$session = Zend_Registry::get('session');

		// store popularities
		$this->erfurtStore->dbConn->Execute('
			INSERT INTO popularity (modelID, uri) 
			VALUES (' . $this->model->modelID . ', "' . $this->resource->getURI() . '")'
			);

			// if form has been submitted
			if ($this->_request->getPost()) {
				// forward to save action
				$this->saveAction();
			}

			// trigger resource view event
			$this->ed->trigger('resource.view.start', $this->resource);

			$this->view->actionUrl    = $this->_request->getRequestUri();
			$this->view->oldUri       = $this->resource->getURI();

			$this->view->erfurtUriBase     = $this->config->erfurtUriBase;
			$this->view->strings           = $this->strings;
			$this->view->tableId           = uniqid();
			$this->view->editingAllowed    = $this->model->isEditable();

			// $this->subView->cssClasses = array('has-contextmenus-block');
			$title   = $this->getSuperClassPath();
			$tabs    = $this->getTabNavigation(0);

			// resource sub menu
			if ($this->model->isEditable()) {
				$resourceMenu[] = '<a href="' . $this->config->uriBase . 'resource/create/">Create New</a>';
				$resourceMenu[] = '<a href="' . $this->config->uriBase . 'resource/delete/?r=' . urlencode($this->resource->getUri()) . '">Delete</a>';
			}
			$resourceMenu[] = '<a href="' . $this->config->uriBase . 'resource/export/?r=' .
			urlencode($this->resource->getUri()) . '&m=' . urlencode($this->model->getModelURI()) .
		 		'&output=xml">Export as RDF/XML</a>';
			$resourceMenu[] = '<a href="' . $this->config->uriBase . 'resource/export/?r=' .
			urlencode($this->resource->getUri()) . '&m=' . urlencode($this->model->getModelURI()) .
				'&output=json">Export as RDF/JSON (Draft)</a>';
			$resourceMenu[] = '<a href="' . $this->config->uriBase . 'resource/export/?r=' .
			urlencode($this->resource->getUri()) . '&m=' . urlencode($this->model->getModelURI()) .
				'&output=n3">Export as RDF/N3</a>';

			// edit menu
			if ($this->model->isEditable()) {
				// TODO: check if enabled
				$editMenu[] = '<a id="menu-inline-toggle"></a>';
			}
				
			if (!empty($editMenu)) {
				$editSub = 'Edit' . $this->view->makeList($editMenu, 'ul');
			}

			if (!empty($resourceMenu)) {
				$resourceSub = 'Resource' . $this->view->makeList($resourceMenu, 'ul');
			}

			$viewSub = '<span class="window-view-menu">View</span>';

			$menu = array($resourceSub, $editSub, $viewSub);

			$this->view->menu  = $menu;
			$this->view->title = $title;
			$this->view->tabs  = $tabs;

			/* new */
			$this->view->resourceLocal = $this->resource->getURI();
			$this->view->resourceTitle = $this->resource->getTitle($session->modelLanguage);
			$this->view->resourceUrl   = OntoWiki_Util::getURL($this->resource);

			$pageModel = new MemModel();
			
			// get sorting properties for the current instance
			$prefixString = 'PREFIX SysOnt: <' . $this->config->sysont->schema . '> ';
			$queryString = 'SELECT * WHERE {?p SysOnt:order ?num . }';
			$ResultOrderProps = $this->model->sparqlQuery($prefixString . $queryString);
			$ResultOrder = array();
			foreach ($ResultOrderProps as $value) {
				$ResultOrder[$value['p']->getURI()] = $value['num']->getLabel();
			}
			asort($ResultOrder,SORT_NUMERIC);
			$ResultOrder = array_keys($ResultOrder);
			
			// construct data model
			
			// TODO: do this with SPARQL in O(n)!!!
			$predicates = array();
			
			foreach ($this->resource->getModel()->findPredicates($instance, null) as $predicate) {

				$stmArray = array();

				if ($predicate->getQualifiedName() != 'rdf:type') {
					$pred = array();
					$pred['title'] = $predicate->getTitle($session->modelLanguage);
					$pred['url']   = OntoWiki_Util::getURL($predicate);
					$pred['uri']   = $predicate->getURI();

					// find all statements for the predicate
					foreach ($this->resource->getModel()->findStatements($this->resource, $predicate, null, 0, $this->_maxErgs, &$erg) as $statement) {
						$object = $statement->obj;
							
						if (!($object instanceof BlankNode)) {
							if ($object instanceof Resource) {
								$stmArray['localName'] = $object->getQualifiedName();
								$stmArray['label']     = $object->getTitle($session->modelLanguage);
								$stmArray['uri']       = $object->getURI();
								$stmArray['url']       = OntoWiki_Util::getURL($object);
								$pred['type']          = 'resource';

								if ($object->hasPropertyValue(null, null)) {
									$stmArray['classes'] = 'class="expandable"';
								}
							} else {
								// literal
								$stmArray['label']     = $object->getLabel();
								$pred['type']          = 'literal';
							}

							if ($erg > $this->_maxErgs) {
								$pred['moreStatements'] = '<a href="' . $this->config->uriBase . 'resource/statements/' .
								urlencode($this->resource->getQualifiedName()) . '?p=' .
								urlencode($predicate->getQualifiedName()) . '">Show all ' . $erg . ' &gt;&gt;</a>' . PHP_EOL;
							}

							$stms[] = $stmArray;

							$pageModel->add($statement);
						}
					}

					if (!empty($stmArray)) {
						// set special resource types (also used fo literals)
						$predUri = $predicate->getURI();
						if (in_array($predUri, $this->config->imgLinkProperties->toArray())) {
							$pred['type'] = 'image';
						} elseif (in_array($predUri, $this->config->webLinkProperties->toArray())) {
							$pred['type'] = 'web';
						} elseif (in_array($predUri, $this->config->mailtoProperties->toArray())) {
							$pred['type'] = 'mail';
						} elseif (in_array($predUri, $this->config->widget->htmlEditProperties->toArray())) {
							$pred['type'] = 'htmledit';
						}
							
						// prepare view arrays
						$pred['statements'] = $stms;
						$stms               = array();
						// Sorting the predicates with as defined with SysOnt:order properties 
						// or lexically by uri
						if (!empty($ResultOrder) && in_array($predicate->getURI(),$ResultOrder) ) {
							$predicates_sorted[array_search($predicate->getURI(),$ResultOrder)] = $pred;
						} else {
							// get lowercase titles for correct sorting later
							$title_lc[] = strtolower($pred['title']);
							$predicates[]       = $pred;
						}
					}
				}
			}
			
			// Do the sortation for all the predicates, then merge the arrays
			// They are sorted as defined by your SysOnt:order properties
			// followed by the remaining ones sorted by title (label)
			if (!empty($title_lc)) {
				array_multisort($title_lc,SORT_ASC,$predicates);
			}
			if (!empty($predicates_sorted)) {
				ksort($predicates_sorted);
				$predicates = array_merge($predicates_sorted, $predicates);
			}

			// no edit button for Ajax
			if ($this->_request->isXmlHttpRequest()) {
				$this->view->editingAllowed = false;
			}

			// test
			// $this->_helper->layout()->disableLayout();

			// print_r($predicates);

			// trigger resource view event
			$this->ed->trigger('resource.view.end', $predicates);

			$this->view->predicates = $predicates;
	}

	/**
	 * Default action when no action is specified
	 *
	 * shows a list of models, instances or instance
	 */
	public function listAction() {
		$this->log->debug('InstanceController::listAction()');

		// if form has been submitted
		if ($this->_request->getPost()) {
			// forward to save action
			$this->saveAction();
		}

		// offset for the request
		if (isset($this->_request->start)) {
			$start = $this->_request->start;
		} else {
			$start = 0;
		}

		// number of columns to show
		// TODO: store cols in session?
		if (isset($this->_request->cols)) {
			$cols = $this->_request->cols;
		} else {
			$cols = 1;
		}

		// additional properties to be shown
		$localName = $this->resource->getQualifiedName();
		if (isset($this->session->show)) {
			$show = $this->session->show;
		} else {
			$show = array();
		}

		if (isset($this->_request->show)) {
			if (is_array($show[$localName])) {
				$show[$localName] = array_unique(array_merge($show[$localName], array_filter($this->_request->show)));
			} else {
				$show[$localName] = $this->_request->show;
			}
		}
		if (isset($this->_request->showRemove)) {
			$show[$localName] = array_diff($show[$localName], $this->_request->showRemove);
		}
		$this->session->show = $show;

		// number of instances to be shown on one page
		if (isset($this->_request->count)) {
			$this->session->count = $this->_request->count;
		}

		// fetch filters
		$this->fetchFiltersAction();

		if ($this->class || $this->resource->getQualifiedName() == 'owl:Thing') {
			if ($this->resource->getQualifiedName() != 'owl:Thing') {
				$instances = $this->class->findInstancesRecursive($this->_filters, 'exact', $start, $this->session->count, &$end);
			} else {
				$instances = $this->model->findInstances($this->_filters, 'exact', $start, $this->session->count, &$end);
			}
		}

		// foreach ($instances as $inst) {
		//     var_dump($inst->getURI());
		// }
		//         exit;

		$this->view->language		= $this->session->modelLanguage;
		$this->view->strings        = $this->strings;
		$this->view->instances      = $instances;
		$this->view->editingAllowed = $this->model->isEditable();
		$this->view->actionUrl      = $this->config->uriBase . '/resource/list/' . urlencode($this->resource->getQualifiedName());
		$this->view->start          = $start;
		$this->view->count          = $this->session->count;
		$this->view->cols           = $cols;
		$this->view->show           = $show[$localName];
		$this->view->classTitle     = $this->resource->getTitle($this->session->modelLanguage);
		$this->view->model          = $this->model;
		$this->view->themeBase      = $this->config->themeBase;

		// no caption when filtering as it needed to be more complex
		if (!$this->_filters) {
			$this->view->tableCaption = 'Instances of ' . $this->view->classTitle;
		}

		// $windowView->cssClasses = array('has-contextmenus-block');
		$this->view->title   = $this->getSuperClassPath();
		$this->view->tabs    = $this->getTabNavigation(0);

		// edit menu
		if ($this->model->isEditable()) {
			// TODO: check if enabled
			$editMenu[] = '<a id="menu-inline-toggle"></a>';
		}
			
		// view sub menu
		$viewMenu[] = '<a class="menu-box-display-toggle" id="menuitem-predicates"></a>';
		$viewMenu[] = '<a class="menu-box-display-toggle" id="menuitem-properties"></a>';
		$viewMenu[] = '<a class="menu-box-display-toggle" id="menuitem-filter"></a>';

		if (!empty($editMenu)) {
			$editSub = 'Edit' . $this->view->makeList($editMenu, 'ul');
		}

		if (!empty($viewMenu)) {
			$viewSub = 'View' . $this->view->makeList($viewMenu, 'ul');
		}

		$this->view->menu = array($editSub, $viewSub);

		$countScript = '<a href="javascript:setCount(' . $this->session->count . ')">' . $this->session->count . '</a>';

		if ($end > $this->session->count) {
			$statusTools[] = 'Columns:&nbsp;' . $this->view->makeList(OntoWiki_Util::getColLinks($cols, $end), 'ul', 'minibutton');
			$statusTools[] = 'Pages:&nbsp;' . $this->view->makeList(OntoWiki_Util::getPagerHtml($start, $this->session->count, $end), 'ul', 'minibutton');
				
			$statusLines[] = 'Search returned ' . $end . ' instances (showing ' . $countScript . '  per page).';
		} else {
			if ($end > 1) {
				$statusLines[] = 'Search returned ' . $end . ' instances (showing ' . $countScript . '  per page).';
				$statusTools[] = 'Columns:&nbsp;' . $this->view->makeList(OntoWiki_Util::getColLinks($cols, $end), 'ul', 'minibutton');
			} elseif ($end == 1) {
				$statusLines[] = 'Search returned 1 instance.';
				$statusTools[] = 'Columns:&nbsp;' . $this->view->makeList(OntoWiki_Util::getColLinks($cols, $end), 'ul', 'minibutton');
			} else {
				$statusLines[] = 'No instances found.';
			}
		}

		$this->view->statusTools = $statusTools;
		$this->view->statusLines = $statusLines;
	}

	/**
	 * Renders an instance's Google map page.
	 */
	public function mapAction() {
		
		// Array to store messages
		$messages = null;
		
		// find a suitable map key
		foreach ($this->config->map->keys->toArray() as $key) {
			if ($key['url'] === $this->_request->getServer('HTTP_HOST')) {
				$keyToUse = $key['key'];
				break;
			}
		}

		// or use the default one
		if (!$keyToUse) {
			$keyToUse = $this->config->defaultMapKey;
		}

		$geoProperties = $this->config->map->geoProperties->toArray();

		// get transitive relations from config
		$transitiveProperties = $this->config->map->transitiveGeoRelations->toArray();

		// overwrite config props with those received via post
		$post = $this->_request->getPost();
		if (isset($post['latProp']) && isset($post['lonProp'])) {
			$transitiveProperties['geo']['latitude'] = $post['latProp'];
			$transitiveProperties['geo']['longitude'] = $post['lonProp'];
		}

		// fetch filters
		$this->fetchFiltersAction();

		// if we have a class, get all instances
		if ($this->class) {
			if ($this->resource->getQualifiedName() != 'owl:Thing') {
				$instances = $this->class->findInstancesRecursive($this->_filters, 'exact', $start, $count, &$end);
			} else {
				$instances = $this->model->findInstances($this->_filters, 'exact', $start, $count, &$end);
			}
		} else {
			$instances = array($this->resource);
		}

		foreach ($instances as $inst) {
			// the instances for which to chek geo data
			// contain first of all the instance itself
			$checkGeoInsts = array($inst);

			foreach ($transitiveProperties as $transProp) {
				$transValue = $inst->getPropertyValue($transProp);
				if ($transValue) {
					$checkGeoInsts[] = $this->model->instanceF($transValue);
				}
			}

			foreach ($checkGeoInsts as $checkInst) {
				if (!$lat = $checkInst->getPropertyValue($geoProperties['geo']['latitude']))
				continue;
				$lat = $lat->getLabel();
					
				if (!$lon = $checkInst->getPropertyValue($geoProperties['geo']['longitude']))
				continue;
				$lon = $lon->getLabel();
			
				// check if geodata values are numeric else there is an undefined behavior
				// of jscript (dangerous)
				if ( preg_match('/^\s*-{0,1}[0-9]+\.{0,1}[0-9]*\s*$/',$lat) && preg_match('/^\s*-{0,1}[0-9]+\.{0,1}[0-9]*\s*$/',$lon) ) {
					$geoinsts++;

					$id = uniqid();

					$markers[] = array('lat' => $lat, 'lon' => $lon, 'html' => '\'<a class="expandable" about="' . $inst->getURI() . '" href="' . OntoWiki_Util::getURL($inst) . '">' .
					$inst->getTitle($this->session->modelLanguage) . '</a><div id="' . $id . '" style="width:auto;height:auto;"></div>\'');
					// echo ($lat . ', ' . $lon);
					break;
				} else {
					$messages[] = 'Invalid geodata (non-numeric): <a href="'. OntoWiki_Util::getURL($inst) . '">' . 
					    $inst->getTitle($this->session->modelLanguage) . '</a>';
				}
			}
		}

		// if we have a class, get its properties
		if ($this->class) {
			$props = /*pwlArrayCopyValues2Indexes(array_keys(*/$this->class->listPropertiesUsed();
		} elseif ($type = $this->resource->getClass()) {
			$props = /*pwlArrayCopyValues2Indexes(array_keys(*/$type->listPropertiesUsed();
		} else {
			$props = array();
		}

		foreach ($props as $key => $prop) {
			$properties[$prop->getURI()] = $key;
		}

		$formStr = '
		<form method="post" action="' . $this->_request->getServer('REQUEST_URI') . '">' . PHP_EOL;

		$formStr .= '
			<div class="no-min-width"><label for="lat-prop-select">Latitude:</label>' . new SelectNew('latProp', 
		$geoProperties['geo']['latitude'],
		$properties, array('cardinality' => 1,
			                                           'multi' => false, 'cssId' => 'lat-prop-select', 'class' => 'width99')) . '</div>
			<div class="no-min-width"><label for="lon-prop-select">Longitude:</label>' . new SelectNew('lonPop', 
		$geoProperties['geo']['longitude'],
		$properties,
		array(
																					'cardinality' => 1, 
			                                            							'multi' => false, 
																					'cssId' => 'lon-prop-select', 
																					'class' => 'width99')) . '</div>
		<hr />
		<div><button type="submit" name="submit">Submit</button></div>
		</form>
		';

		$this->onLoadRegistry[] = 'loadGMap2();';

		$this->view->statusLines     = $messages;
		$this->view->markers         = $markers;
		$this->view->apiKey          = $keyToUse;
		$this->view->centerLat       = $this->config->map->defaultPositionLatitude;
		$this->view->centerLon       = $this->config->map->defaultPositionLongitude;
		$this->view->zoom            = $this->config->map->defaultZoomLevel;
		$this->view->latPropSel      = $post['latProp'];
		$this->view->lonPropSel      = $post['lonProp'];
		$this->view->title           = $this->getSuperClassPath();
		$this->view->tabs            = $this->getTabNavigation(1);
		$this->view->floatSubWindows = true;
		
		$this->view->innerWindows = array(
		array(
				'cssId' => 'geoprops', 
				'cssClasses' => ($this->session->geopropsBoxMinimized ? 'is-minimized' : ''), 
				'title' => 'Geo Properties', 
				'content' => $formStr
		)
		);

		// if ($this->class) {
		// 	$windowView->innerWindows = array_filter(array(
		// 		$subView->render('micro/window.php'),
		// 		$this->getFilterBox($subView)
		// 	));
		// } else {
		// 	$windowView->innerWindows = array($subView->render('micro/window.php'));
		// }

		// $this->view->mainWindows = array($windowView->render('micro/window.php'));
	}

	/**
	 *
	 */
	public function calendarAction() {
		$params = array();
		if (isset($this->_request->start)) {
			$params['start'] = $this->_request->getParam('start');
			$start = strtotime($params['start']);
		} else {
			$start = time();
		}
		if (isset($this->_request->weeksToShow)) {
			$weeksToShow = $this->_request->getParam('weeksToShow');
			$params['weeksToShow'] = $weeksToShow;
		} else {
			$weeksToShow = 5;
		}

		$monday = mktime(0, 0, 0, date('n', $start), date('j', $start) - date('w', $start) + 1, date('Y', $start));

		// fetch filters
		$this->fetchFiltersAction();

		// if we have a class, get all instances
		if ($this->class) {
			if ($this->resource->getQualifiedName() != 'owl:Thing') {
				$instances = $this->class->findInstancesRecursive($this->_filters, 'exact', $start, $count, &$end);
			} else {
				$instances = $this->model->findInstances($this->_filters, 'exact', $start, $count, &$end);
			}
		} else {
			$instances = array($this->resource);
		}

		foreach ($instances as $inst) {
			$sql = '
				FROM statements
				WHERE l_datatype IN (\'http://www.w3.org/2001/XMLSchema#dateTime\',\'http://www.w3.org/2001/XMLSchema#date\')
					' . ($inst ? "AND subject='{$inst->getURI()}'" : '') . '
				AND modelID IN (' . $this->model->getModelIds() . ')
				AND object > \'' . strftime('%Y-%m-%d', $monday) . '\' 
				AND object < \'' . strftime('%Y-%m-%d', $monday + ($weeksToShow * 7 * 24 * 3600)) . '\'';
				
			// TODO: what is $np?
			$res = $this->erfurtStore->dbConn->getAll('
				SELECT subject, predicate, object, l_language, l_datatype, subject_is, object_is, modelID, id ' . $sql . ($np ?' 
				AND predicate NOT IN (\'' . join('\',\'', $np) . '\')' : ''));

			foreach($res as $row) {
				$cal[$row[2]][] = $this->model->_convertRowToStatement($row);
			}
		}

		$this->view->days        = $this->strings->calendar->days->toArray();
		// $subView->start       = $start;
		$this->view->weeksToShow = $weeksToShow;
		$this->view->monday      = $monday;
		$prevDate   = strftime('%Y-%m-%d', $monday - ($weeksToShow * 7 * 24 * 3600));
		$nextDate   = strftime('%Y-%m-%d', $monday + ($weeksToShow * 7 * 24 * 3600));
		$this->view->cal_data    = $cal;
		// $this->view->params      = $params;

		$this->view->title   = $this->getSuperClassPath();
		$this->view->tabs    = $this->getTabNavigation(2);

		$pager[] = '<li><a href="' . Erfurt_Util::replaceUrlParam('start', $prevDate) . '">&laquo;&nbsp;Previous</a></li>';
		$pager[] = '<li><a href="' . Erfurt_Util::replaceUrlParam('start', $nextDate) . '">Next&nbsp;&raquo;</a></li>';

		$weekLinks[] = '<li><a href="' . Erfurt_Util::replaceUrlParam('weeksToShow', '5') . '"' . (($weeksToShow == 5) ? ' class="selected"' : '') . '>5</a></li>';
		$weekLinks[] = '<li><a href="' . Erfurt_Util::replaceUrlParam('weeksToShow', '10') . '"' . (($weeksToShow == 10) ? ' class="selected"' : '') . '>10</a></li>';
		$weekLinks[] = '<li><a href="' . Erfurt_Util::replaceUrlParam('weeksToShow', '20') . '"' . (($weeksToShow == 20) ? ' class="selected"' : '') . '>20</a></li>';
		$this->view->statusTools = array('Page:' . $this->view->makeList($pager, 'ul', 'minibutton'), 'Weeks per Page:' . $this->view->makeList($weekLinks, 'ul', 'minibutton'));
	}

	/**
	 *
	 */
	public function historyAction() {
		$versioning = new Erfurt_Versioning($this->model);

		if ($sel = $this->_request->getPost('selected')) {
			$versioning->multipleRollback($sel);
				
			$this->_redirect(OntoWiki_Util::getURL($this->resource, null, 'history'), array('code' => 302, 'prependBase' => false));
		}

		$restrict = $this->_request->getParam('restrict');
		if (is_array($restrict)) {
			$filter = array_merge($restrict, array('resource' => $this->resource->getURI()));
		} else {
			$filter = array('resource' => $this->resource->getURI());
		}

		$start = 0;
		$count = 20;

		$res = $versioning->listEntries($filter, $start, $count, $erg);

		$this->view->th             = $this->strings->history->th->toArray();
		$this->view->res            = $res;
		$this->view->versioning     = $versioning;
		$this->view->tab_navigation = $this->getTabNavigation(3);
		$this->view->title          = $this->getSuperClassPath();
		$this->view->tabs           = $this->getTabNavigation(3);
	}

	/**
	 *
	 */
	public function discussionAction() {
		$this->view->title          = $this->strings->discussion->title . ' ' . $this->resource->getQualifiedName();
		$this->view->tabs           = $this->getTabNavigation(3);
		// $this->view->assign('content', $this->view->render('resource/discussion.php'));
		// $this->_helper->viewRenderer->setNoRender(true);
	}

	/**
	 * Action to save edited property values.
	 * Used in  inline and edit mode and to
	 * create new instances.
	 */
	public function saveAction() {
		
		function stripslashes_deep($value) {
			if (is_array($value)) {
				if (count($value)>0) {
					$return = array_combine(array_map('stripslashes_deep', array_keys($value)),array_map('stripslashes_deep', array_values($value)));
				} else {
					$return = array_map('stripslashes_deep', $value);
				}
				return $return;
			} else {
				$return = stripslashes($value);
				return $return ;
			}
		}



		if ( get_magic_quotes_gpc() ) {
			$oldUriPost  = stripslashes( $this->_request->getPost('olduri') );
			$uriPost     = stripslashes( $this->_request->getPost('uri') );
			$labelPost   = stripslashes( $this->_request->getPost('label') );
			$instPost    = stripslashes_deep( $this->_request->getPost('inst') );
			$propPost    = stripslashes_deep( $this->_request->getPost('prop') );
			$propaddPost = stripslashes_deep( $this->_request->getPost('propadd') );
		} else {
			$oldUriPost  = $this->_request->getPost('olduri');
			$uriPost     = $this->_request->getPost('uri');
			$labelPost   = $this->_request->getPost('label');
			$instPost    = $this->_request->getPost('inst');
			$propPost    = $this->_request->getPost('prop');
			$propaddPost = $this->_request->getPost('propadd');
		}

		if ($instPost) {
			
			foreach ($instPost as $instance => $properties) {
				$i = $this->model->instanceF($instance);
				foreach ($properties as $prop => $vals) {
					foreach ($vals as $mult => $p) {
						if (is_array($p)) {
							foreach ($p as $k => $v) {
								// TODO: won't work in table edit mode
								if (is_string($v) && OntoWiki_Util::isValidEmail($v)) {
									$vals[$k] = 'mailto:' . $v;
								}
							}
						} else {
							if (is_string($p) && OntoWiki_Util::isValidEmail($p)) {
								$vals[$mult] = 'mailto:' . $p;
							}
						}
					}
					$i->setPropertyValues($prop, $vals);
				}
			}

		}
		// if uri has changed, rename
		if ($oldUriPost) {
			$resource = $this->model->resourceF($oldUriPost);
			if ($uriPost && $uriPost != $oldUriPost) {
				$resource->rename($uriPost);
				$this->resource = $resource;
			}
			// create new resource from create action
		} else {
			$resource = $this->model->resourceF($uriPost);
		}

		// save properties
		if ($propPost) {
			// var_dump($propPost);
			// exit();
			// remove all old properties that are not type or label
			// caused some problems so deactivate for now
			// foreach ($resource->listPropertyValues() as $pv) {
			// 	if ($pv->getQualifiedName() != 'rdf:type' && $pv->getQualifiedName() != 'rdfs:label') {
			// 		$resource->removePropertyValues($pv);
			// 	}
			// }

			foreach ($propPost as $prop => $value) {

				$p = $this->model->propertyF($prop);

				if ($p instanceof Erfurt_Owl_Property && $p->isObjectProperty()) {
					if (is_array($value)) {
						foreach ($value as $key => $val) {
							// check for an e-mail address entered							
							if (is_string($value[$key]) && OntoWiki_Util::isValidEmail($value[$key])) {
								// we now have an email adress
								$value[$key] = 'mailto:' . $value[$key];
							}
						}
						// rewrite property values
						$resource->setPropertyValues($prop, $value);
					}
						
					// set all values class to be the property's range
					if ($range = $p->getRange()) {
						foreach ($resource->listPropertyValues($p) as $v) {
							if (!$v->getClass()) {
								$v->setClass($range);
							}
						}
					}
				} else {
					$resource->setPropertyValues($prop, $value);
				}

				if ($p->getNamespace() != EF_RDF_NS &&
				$p->getNamespace() != EF_RDFS_NS &&
				$p->getNamespace() != EF_OWL_NS &&
				!$this->model->findStatement($prop, EF_RDF_TYPE, NULL)) {
					$undef[] = urlencode($prop);
				}
			}
			// TODO: handle undefined properties
		}

		// save label
		if ($labelPost) {
			$resource->setLabel(new Literal($labelPost));
		}

		// redirect to the just created resource
		if ($this->_request->getActionName() != 'list') {
			$this->_redirect(OntoWiki_Util::getURL($resource), array('code' => 302, 'prependBase' => false));
		}
	}


	/**
	 * Instance edit action
	 */
	public function editAction() {
		// if form has been submitted
		if (isset($this->_request->submit)) {
			// forward to save action
			$this->saveAction();
		}

		$prop = array();
		foreach ($this->resource->getModel()->findPredicates($this->resource, null) as $p) {
			if ($p->getQualifiedName() != 'rdf:type') {
				$prop[$p->getQualifiedName()] = $this->resource->listPropertyValues($p);
			}
		}

		$tableId = uniqid();

		if (is_null(array_shift($this->resource->listLabels()))) {
			// Find out if given resource is implicit class, then  set type to current model class type
			// otherwise the edit class description for an implicit class won't work
			$implicitClasses = $this->model->listTopClasses(false,false,true,false);
			foreach ($implicitClasses as $iclass) {
				$icArray[] = $iclass->getURI();
				$icArray[] = $iclass->getClassType();
			}
			if ($icPos = array_search($this->resource->getURI(),$icArray)) {
				$prop['rdf:type'] = $this->model->resourceF($icArray[$icPos+1])->getURI();
			}
			$label = $this->resource->getQualifiedName();
		} else {
			$label = array_shift($this->resource->listLabels())->getLabel();
		}

		$this->view->actionUrl     = $this->_request->getRequestUri();
		$this->view->strings       = $this->strings;
		$this->view->instanceLabel = $label;
		$this->view->oldUri        = $this->resource->getUri();
		$this->view->instanceUri   = $this->resource->getURI();
		$this->view->type          = $this->class ? $this->class : $this->resource->getClass();
		$this->view->properties    = $prop;
		$this->view->tableId       = $tableId;
		$this->view->title         = $this->getSuperClassPath();
		$this->view->tabs          = $this->getTabNavigation(4);
	}

	/**
	 * Action to create Resource; standard use case is
	 * -instance of an class
	 * -new class
	 *
	 */
	public function createAction() {
		$this->log->debug('InstanceController::createAction()');

		// if form has been submitted
		if (isset($this->_request->submit)) {
			// forward to save action
			$this->saveAction();
		}

		// get predefined values if any
		$prop = isset($this->_request->prop) ? array_filter($this->_request->prop) : array();

		// set rdf:type property
		if (isset($prop['rdf:type'])) {
			$type = $this->model->resourceF(urldecode($prop['rdf:type']));
		} elseif ($this->class) {
			$type = $this->class;
		} else {
			$type = $this->model->resourceF($this->resource->getClass());
		}

		if ($c = $this->model->getClass($type)) {
			foreach ($c->listPropertiesUsed() as $key) {
				$prop[$key->getQualifiedName()] = $prop[$key->getQualifiedName()] ? $prop[$key->getQualifiedName()] : '';
			}
		}

		if ( isset($this->_request->r) ) {
			$instanceUri = $this->_request->r;
		} else {
			$instanceUri = $this->model->getUniqueResourceURI(str_replace(':', '', $type->getQualifiedName()));
		}

		$this->view->actionUrl     = $this->config->uriBase . 'resource/create/';
		$this->view->strings       = $this->strings;
		$this->view->instanceUri   = $instanceUri;
		$this->view->modelBase     = $this->model->baseURI;
		$this->view->type          = $this->class ? $this->class : $this->resource->getClass();
		$this->view->properties    = $prop;
		$this->view->title         = $this->getSuperClassPath();
		$this->view->tabs          = $this->getTabNavigation(4);

		// we render this action with the edit template
		$this->_helper->viewRenderer->setScriptAction('edit');
	}

	public function statementsAction() {
		$this->log->debug('ResourceController::statementAction()');

		// offset for the request
		if (isset($this->_request->start)) {
			$start = $this->_request->start;
		} else {
			$start = 0;
		}

		// number of instances to be shown on one page
		if (isset($this->_request->count)) {
			$this->session->count = $this->_request->count;
		}

		// number of columns to show
		// TODO: store cols in session?
		if (isset($this->_request->cols)) {
			$cols = $this->_request->cols;
		} else {
			$cols = 1;
		}

		$property = $this->model->propertyF($this->_request->getParam('p'));
		$statements = $this->model->findStatements($this->resource, $property, null, $start, $this->session->count, &$end);

		$data = array();
		foreach ($statements as $s) {
			$data[] = array(OntoWiki_Util::displayObject($s));
		}

		$this->view->start   = $start;
		$this->view->count   = $this->session->count;
		$this->view->end     = $end;
		$this->view->cols    = $cols;
		$this->view->data    = $data;

		$this->view->title   = OntoWiki_Util::getTitle($this->resource) . ' : ' . OntoWiki_Util::getTitle($property);

		// TODO: export to function
		$countScript = '<a href="javascript:setCount(' . $this->session->count . ')">' . $this->session->count . '</a>';

		$statusTools = array();
		$statusLines = array();
		if ($end > $this->session->count) {
			$statusTools[] = $this->strings->pager->columns
			. ':&nbsp;'
			. $this->view->makeList(OntoWiki_Util::getColLinks($cols, $end), 'ul', 'minibutton');
			$statusTools[] = $this->strings->pager->pages
			. ':&nbsp;'
			. $this->view->makeList(OntoWiki_Util::getPagerHtml($start, $this->session->count, $end), 'ul', 'minibutton');
			$statusLines[] = 'Showing ' . $countScript . ' of ' . $end . ' statements per page.';
		} else {
			if ($end > 1) {
				$statusLines[] = 'Search returned ' . $end . ' statements (showing ' . $countScript . '  per page).';
				$statusTools[] = $this->strings->pager->columns
				. ':&nbsp;'
				. $this->view->makeList(OntoWiki_Util::getColLinks($cols, $end), 'ul', 'minibutton');
			} elseif ($end == 1) {
				$statusLines[] = 'Search returned 1 statement.';
				$statusTools[] = $this->strings->pager->pages
				. ':&nbsp;'
				. $this->view->makeList(OntoWiki_Util::getColLinks($cols, $end), 'ul', 'minibutton');
			} else {
				$statusLines[] = 'No statements found.';
			}
		}

		$this->view->statusTools = $statusTools;
		$this->view->statusLines = $statusLines;
		$this->_helper->viewRenderer->setScriptAction('table');
	}

	public function exportAction() {

		$modelURI = $this->_request->getParam('m');
		if ($modelURI == false) {
			$m = new MemModel();
				
			$stmList = $this->erfurtStore->executeFind(null, new Resource($this->_request->getParam('r')), null, null,
			true, true, true, true,	false, true);
				
			foreach ($stmList as $stm) {
				$m->add($stm);
			}
		} else {
			$model = $this->erfurtStore->getModel($modelURI);
			$r = $model->resourceF($this->_request->getParam('r'));
				
			$m = $r->getDefiningModel();
		}

		$type = $this->_request->getParam('output');

		if ($type === 'xml') {
			$xmlWriter = new Erfurt_Syntax_StringWriterXMLImpl();
			$rdfSerializer = new Erfurt_Syntax_RDFSerializer();
				
			// send rdf/xml content type header
			header('Content-Type: application/rdf+xml');
			// header('Content-Disposition: attachment; filename="export.rdf"');

			echo $rdfSerializer->serializeToString($xmlWriter, $m);
			exit();
		} else if ($type === 'json') {
			$rdfSerializer = new JsonSerializer();
				
			// send rdf/json content type header (no standard yet!!!)
			header('Content-Type: application/rdf+json');
			echo $rdfSerializer->serialize($m);
			exit();
		} else if ($type === 'n3') {
			$rdfSerializer = new N3Serializer();
				
			// send rdf/n3 content type header
			header('Content-Type: text/rdf+n3');
			echo $rdfSerializer->serialize($m);
			exit();
		}
	}

	/**
	 * tab navigation
	 */
	function getTabNavigation($current = 0, $class = null) {
		if ($class || $GLOBALS['class']) {
			$class = $class ? $class : $GLOBALS['class'];
		} else {
			$pref = '#';
		}

		if (/*$this->_request->getActionName() == 'list'*/ isset($this->class)) {
			$label = $this->strings->instance->tabs->instances;
			$viewAction = 'list';
		} else {
			$label = $this->strings->instance->tabs->properties;
			$viewAction = 'view';
		}

		if ($this->_filters) {
			if ($this->_filtersCombined) {
				$query = http_build_query(array('select' => $this->_filters, 'combine' => 'on'));
			} else {
				$query = http_build_query(array('select' => $this->_filters));
			}
		} else {
			$query = '';
		}

		$tabs = array(
		OntoWiki_Util::getURL($this->resource->getQualifiedName(), null, $viewAction, $query)   => $label,
		OntoWiki_Util::getURL($this->resource->getQualifiedName(), null, 'map', $query)		    => $this->strings->instance->tabs->map,
		OntoWiki_Util::getURL($this->resource->getQualifiedName(), null, 'calendar', $query)	=> $this->strings->instance->tabs->calendar
		// discussion temporarily disabled
		// OntoWiki_Util::getURL($this->resource->getQualifiedName(), null, 'disussion')			=> $this->strings->instance->tabs->discussion,
		);

		/*
		 if ($this->resource instanceof Erfurt_Rdfs_Resource_Default) {
		 $url = OntoWiki_Util::getURL($this->resource->getQualifiedName(), null, 'discussion');
		 $tabs[$url] = $this->strings->instance->tabs->discussion;
		 }
		 */

		# auth
		$identity = Zend_Auth::getInstance()->getIdentity();
		if (/* $identity['anonymous'] === false && */$this->model && $this->model->isEditable()) {
			$tabs[OntoWiki_Util::getURL($this->resource->getQualifiedName(), null, 'history')] = $this->strings->instance->tabs->history;
			$tabs[OntoWiki_Util::getURL($this->resource->getQualifiedName(), null, 'edit')] = $this->strings->instance->tabs->edit;
		}
		// add commentstab (needs to be the last tab due to to javascript restrictions)
		if ( $this->_request->getActionName() != 'list' ) {
		    $tabs['javascript:showOntowikiComments(\''.$this->resource->getUri() . '\');'] = $this->strings->comments->title;
		}
		
		$ret = $this->getTabs($tabs, $current);
		return $ret;
	}

	/**
	 * Fetches filters from URI
	 */
	public function fetchFiltersAction() {
		// only old-school OntoWiki filters supported by now
		if (isset($this->_request->select)) {
			$this->_filters = array_filter($this->_request->select);
		} else {
			$this->_filters = array();
		}

		if (isset($this->_request->combine) && $this->_request->combine == 'on') {
			$this->_filtersCombined = true;
		} else {
			$this->_filtersCombined = false;
		}
	}

	/**
	 *	Action to enter, edit and execute SPARQL Queries
	 */
	public function queryAction() {

		$resultFormat = $this->_request->getPost('result_format');
		$query        = trim(stripslashes($this->_request->getPost('query')));

		$this->view->q         = $query;
		$this->view->owUriBase = $this->config->uriBase;
		$this->view->model     = $this->model;
		$this->view->strings   = $this->strings;

		if (!empty($query)) {
			if ($this->model instanceof RDFSModel) {
				// build query base
				$prefixedQuery = 'PREFIX : <' . $this->model->getBaseURI() . '>' . PHP_EOL;
				// prefixes
				foreach ($this->model->getParsedNamespaces() as $uri => $prefix) {
					if ($uri != $baseUri) {
						$prefixedQuery .= 'PREFIX ' . $prefix . ': <' . $uri . '>' . PHP_EOL;
					}
				}
			}
				
			// and actual query
			$prefixedQuery .= $query;
				
			//echo htmlentities($prefixedQuery);exit();
				
			// query model
			try {
				if ($resultFormat == 'xml') {
					$renderer = 'xml';
				} else {
					// TODO: use Erfurt result renderer
					// $renderer = new Erfurt_Sparql_ResultRenderer_Default($this->model, 'resource');
					$renderer = 'Default';
				}

				$endpoint = new Erfurt_Sparql_Endpoint_Default($this->config);

				$endpoint -> setQuery($prefixedQuery);
				$endpoint -> addModel($this->model->modelURI);
				$endpoint -> setRenderer($renderer);

				$result = $endpoint->query();

				if (count($result) == 0) {
					$this->view->assign('message', 'Your query returned no results.');
				}
			} catch (SparqlParserException $e) {
				$this->view->errors = array('Your query contains the following error: ' . $e->getMessage());
			} catch (Exception $e) {
				$this->view->errors = array('Exception on query execution: ' . $e->getMessage());
			}
				
			// number of columns to show
			// TODO: store cols in session?
			if (isset($this->_request->cols)) {
				$cols = $this->_request->cols;
			} else {
				$cols = 1;
			}
				
			$subView->cols  = $cols;
			$subView->end   = count($result);

			if ($resultFormat == 'xml') {
				$this->view->results = '<p><textarea class="query" style="width:100%" rows="36" readonly="readonly">' . htmlentities($result) . '</textarea></p>';
			} else {
				$this->view->results = $this->view->partial('resource/table.php',array(
					'data' => $result,
					'cols' => $cols
				));
			}
				

		} else {
			; // do nothing
		}

		$menuElements[] = '<span onclick="comment()">Comment Region</span>';
		$menuElements[] = '<li><span onclick="unComment()">Uncomment Region</span>';
		$menuElements[] = '<span onclick="optional()">Make Optional</span>';
		$menuElements[] = '<span onclick="indent()">Indent Region</span>';
		$menuElements[] = '<span onclick="insert(\'BASE <http://example.org/base>\')">BASE</span>';
		$menuElements[] = '<span onclick="insert(\'FROM <http://example.org/from>\')">FROM</span>';
		$menuElements[] = '<span onclick="insert(\'FROM NAMED <http://example.org/named>\')">FROM NAMED</span>';
		$menuElements[] = '<span onclick="insert(\'UNION\')">UNION</span>';
		$menuElements[] = '<span onclick="insert(\'GRAPH\')">GRAPH</span>';
		$menuElements[] = '<span onclick="insert(\'ORDER BY\')">ORDER BY</span>';
		$menuElements[] = '<span onclick="insert(\'ORDER BY ASC(?x)\')">ORDER BY ASC()</span>';
		$menuElements[] = '<span onclick="insert(\'ORDER BY DESC(?x)\')">ORDER BY DESC()</span>';
		$menuElements[] = '<span onclick="insert(\'LIMIT 10\')">LIMIT</span>';
		$menuElements[] = '<span onclick="insert(\'OFFSET 10\')">OFFSET</span>';
		$menuElements[] = '<span onclick="insert(\'FILTER ( ?x < 3 ) .\')">Simple Filter</span>';
		$menuElements[] = '<span onclick="insert(\'FILTER regex( str(?name), \"Jane\", \"i\" ) .\')">Regex Filter</span>';
		$menuElements[] = '<span onclick="insert(\'FILTER ( bound(?x) ) .\')">Bound Filter</span>';
		$menuElements[] = '<span onclick="insert(\'FILTER ( ?date > \"2005-01-01T00:00:00Z\"^^xsd:dateTime ) .\')">Date Filter</span>';
		$ElementsSub = 'Elements' . $this->view->makeList($menuElements, 'ul');
		$menuHelp[] = '<a href="http://www.w3.org/TR/rdf-sparql-query/">Specification</a>';
		$menuHelp[] = '<a href="http://www.dajobe.org/2005/04-sparql/">Reference Card</a>';
		$menuHelp[] = '<a href="http://platon.escet.urjc.es/%7Eaxel/sparqltutorial/">Tutorial</a>';$ElementsSub = 'Elements' . $this->view->makeList($menuElements, 'ul');
		$HelpSub = 'Help' . $this->view->makeList($menuHelp, 'ul');

		$this -> view -> menu = array ($ElementsSub, $HelpSub);
		$this -> view -> title = $this -> strings -> query -> title;
	}

	/**
	 * For upload of files as resources
	 */
	public function fileAction() {

		$this -> view -> owUriBase = $this -> config -> uriBase;
		$this -> view -> title = $this -> strings -> filemanager -> title . ' - Upload';

		// Generate default URI for file and set it in view
		$size = sizeof(scandir(REAL_BASE . $this -> config ->filemanager -> path));
		$name = 'file_' . ($size - 2 );
		$defaultUri .= str_replace($this -> config -> erfurtUriBase, '', $this -> config -> erfurtUrlBase);
		$defaultUri .= $this -> config -> uriBase;
		$defaultUri .= $this -> config -> filemanager -> path . $name;
		$this -> view -> defaultUri = $defaultUri;

		$sysontModel = Zend_Registry::get('sysontModel');

		if (! is_writable(REAL_BASE . $this -> config -> filemanager -> path)) {
			throw new Exception('Directory ' . $this -> config -> filemanager -> path . ' not writeable ');
		}

		// check flag for execution as Service
		if ($_POST['asService']) {
			// service  needs no view renderer
			$this->_helper->viewRenderer->setNoRender();
			// service needs no layout template
			$this->_helper->layout->disableLayout();
			// Set service var
			$asService = true;
		} else {
			$asService = false;
		}

		$source = $_FILES['source'];
		$uri = $_POST['uri'];
		$pathHashed = REAL_BASE . $this -> config -> filemanager -> path . md5($uri);

		if (!$source['error'] && $uri != '') {
				
			// Set label to $_POST['label'] or extract from $_POST['uri']
			if ($_POST['label']) {
				$label = $_POST['label'];
			} else {
				$label = str_replace($this -> config -> uriBase . $this -> config -> filemanager -> path,
				'',$uri);
			}
				
			if ( !file_exists($pathHashed) && copy( $source['tmp_name'], $pathHashed)) {
				$fMime = $this -> config -> filemanager -> file -> mimeproperty;
				$fClass = $this -> config -> filemanager -> file -> ofclass;
				$fModel = $this -> config -> filemanager -> file -> ofmodel;

				// Adding statements to the system configuration ontology
				$sysontModel -> add($uri, EF_RDF_TYPE, $fClass);
				$sysontModel -> add($uri, $fMime, $source['type']);
				$sysontModel -> add($uri, $fModel , $this -> model -> getModelURI());

				// Adding statements to the model (only label by now)
				$this -> model -> add ($uri, EF_RDFS_LABEL, new Erfurt_Rdfs_Literal_Default($label));
				$this -> model -> add ($uri, EF_RDF_TYPE,   $fClass);

				// Add defined link (used by SoftWiki plugin only ATM)
				if ($_POST['linkAs']) {
					$linkprop = $_POST['linkAs'];
					$this -> model -> add ($this -> resource -> getURI(), $linkprop, $uri);
				}

				// Success message
				$this -> view -> messages[] = $this->strings->filemanager->upload->success;

				// Controll output if no service is used redirect to edit
				if (!$asService) {
					// Redirect to edit metadata for created instance.
					$this -> _redirect('/resource/view/?r='.rawurlencode($uri));
				} else {
					// Output of messagebox only
					echo '<div class="messagebox success">'.
					$this->strings->filemanager->upload->success . '</div>' . PHP_EOL;
				}
			} else {
				$this -> view -> errors[] =$this->strings->filemanager->upload->error;
				// Controll output
				if (!$asService) {
					// do nothing
				} else {
					// Output of messagebox only
					echo '<div class="messagebox error">' .
					$this->strings->filemanager->upload->error . '</div>' . PHP_EOL;
				}
			}
		} else {
			// do nothing
		}
	}

}

?>
