<?php

/**
  * OntoWiki Service Controller
  *
  * @author Norman Heino
  * @version $Id: ServiceController.php 2685 2009-02-06 11:04:08Z sknappe $
  */
class ServiceController extends Zend_Controller_Action {
	
	protected $session;
	protected $model;
	protected $resource;
	protected $subView;
	protected $config;
	
	public function init() {
		parent::init();
		
		// service controller needs no view renderer
		$this->_helper->viewRenderer->setNoRender();
		
		// service controller needs no layout template
		$this->_helper->layout->disableLayout();
		
		$this->config = Zend_Registry::get('config');
		
		try {
			$this->session = Zend_Registry::get('session');
			$this->model = Zend_Registry::get('config')->activeModel;
			if ($this->model) {
				$this->resource = $this->model->resourceF($this->_request->getParam('r'));
			} else {
				// try resource uri as model also
				// TODO: 
				// $this->resource = new Erfurt_Rdfs_Resource_Default($this->_request->getParam('r'), $powl->getModel($this->_request->getParam('r')));
			}
			
		} catch (Exception $e) {
			// $this->_log->debug('Uncatched Exception: ' . __CLASS__ . ':' . $this->message . PHP_EOL . $this->file . ':' . $this->line);
		}
	}
	
	/**
     * Catch action 
     * 
     * Will be called when an undefined action is called
     */
    public function __call($methodName, $args) {
        Zend_Registry::get('owLog')->debug('ServiceController::__call()');
        exit;
    }
	
	public function isactivemodeleditableAction() {
		$this->_response->setHeader('Content-Type', 'text/html; charset=utf-8');
		print 'selected';
	}
	
	/**
	  * Stores a variable passed as parameters <code>name</code> and <code>value</code> in the current session.
	  */
	public function sessionstoreAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::sessionStoreAction()');
		
		$name  = $this->_request->name;
		$value = $this->_request->value;
		
		if (is_array($this->_request->{$name})) {
			// we have an array of $name
			if (isset($this->session->{$name}) && is_array($this->session->{$name})) {
				$this->session->{$name} = array_merge($this->session->{$name}, $this->_request->{$name});
			} else {
				$this->session->{$name} = $this->_request->{$name};
			}
		} elseif (is_string($name)) {
			$this->session->{$name} = $value;
		} else {
			throw new Exception('Illegal parameter type sent.');
		}
	}
	
	/**
	  * Sets a rating for the current instance by the currently logged in user.
	  */
	public function setratingAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::setRatingAction()');
		
		if ($rating = $this->_request->getParam('rating')) {
			$identity = Zend_Auth::getInstance()->getIdentity();
			if ($identity['anonymous'] !== false) {
				return false;
			}
			$sql = "REPLACE ratings 
			            (modelID, ef_user, ef_resource, rating) 
			        VALUES 
			            ('{$this->model->modelID}', '{$identity['uri']}', '{$this->resource->getURI()}', '$rating')";
			$this->model->dbConn->execute($sql);
			
			// echo $sql;
		}
	}
	
	/**
	  * Responds with the average rating image for the currently selected instance.
	  */
	public function getratingAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::getRatingAction()');
		
		echo OntoWiki_Util::getRating($this->resource);
	}
	
	/**
	  * Responds with the instance rendering for the currently selected resource done by 
	  * <code>OntoWiki_Util::renderInstance()</code>.
	  */
	public function getinstanceAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::getInstanceAction()');
		
		echo OntoWiki_Util::renderInstance($this->_request->r, $this->_request->allowEdit, $this->_request->maxLength, null,  $this->_request->noCaption);
	}
	
	/**
	  * Responds with an edit widget for the property passed as param 
	  * <code>property</code> for currently selected resource.
	  */
	public function editinstanceAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::editInstanceAction() -- ' . $this->_request->property);
		
		if (!$this->model && isset($this->_request->m)) {
			$model = Zend_Registry::get('erfurt')->getModel($this->_request->m);
		}
		
		echo OntoWiki_Util::editInstance(urldecode($this->_request->r), $this->_request->property);
	}
	
	/**
	  * Responds with an edit widget preset with a specific value.
	  */
	public function editvaluesAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::editValuesAction()');
		
		echo OntoWiki_Util::editValues($this->resource, $this->_request->property, $this->_request->value);
	}
	
	// public function editValuesEmptyAction() {
	// 	Zend_Log::log('ServiceController::getValuesEmptyAction()', Zend_Log::LEVEL_INFO, 'Controller');
	// 	
	// 	echo new $this->_request->class() call_user_func(array(, 'getSingleValueHtml'), '', $this->_request->count + 1);
	// }
	
	public function getwidgetsAction() {
		$subjectUri  = $this->_param->s;
		$propertyUri = $this->_param->p;
		
				
	}
	
	/**
	  *
	  */
	public function getpropertyselectorAction() {
		$property = $this->_param->property;
		$id = uniqid();
		$config = Zend_Registry::get('config');
		?>
		<input type="text" id="autosuggest<?php echo $id?>" class="autosuggest text" value="<?php echo $property ?>"
		onchange="updateTable(this, '<?php echo $id?>')" />
		<div id="autosuggest_choices<?php echo $id?>" class="autosuggest"></div>
		<img class="button clickicon" style="visibility:hidden" onclick="this.parentNode.parentNode.parentNode.removeChild(this.parentNode.parentNode)" src="<?php echo $config->themeBase . 'images/icon-delete.png'?>" alt="delete" />
		<script type="text/javascript">
		//<![CDATA[
		new Ajax.Autocompleter('autosuggest<?php echo $id?>', 
		                       'autosuggest_choices<?php echo $id?>', 
		                       owUriBase + 'service/searchProperties/', 
		                       {paramName: 'searchText', 
		                        minChars: 3, 
		                        afterUpdateElement: updTable = function(input, li) {
			new Ajax.Updater(input.parentNode.parentNode.getElementsByTagName('td')[1], owUriBase + 'service/editValues/?property=' + input.value, {evalScripts: true});
		}});
		//]]>
		</script>
		<?php
	}
	
	/**
	  *
	  */
	public function deletestatementAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::deleteStatementAction()');
		
		OntoWiki_Util::deleteStatement($this->_request->stId);
	}
	

	/**
	  * Responds with an unordered list of properties that can be used
	  * with Scriptaculous' suggest control.
	  */
	public function searchpropertiesAction() {
		if (isset($this->_request->searchText)) {
			$searchText = $this->_request->searchText;
		} else {
			$searchText = '';
		}
		
		$search = new InstanceSearch($this->model, $searchText);
		// $search->allModels = true;
		$search->propertiesOnly = true;
		$results = $search->search();
		$encoding = Zend_Registry::get('config')->ow->encoding;
		
		$ret = '<ul>' . PHP_EOL;
		foreach ($results->getRows() as $row) {
			// TODO: use better property names
			if ($res = $this->model->getResource($row[0])) {
				$localName = $res->getQualifiedName();
				$title = OntoWiki_Util::getTitle($res, false, 28);
			}
			if ($localName == '') {
				$localName = $row[0];
			}
			if ($title == '') {
				$title = $row[0];
			}
			
			$ret .= '<li><span class="formal">' . $localName . '</span><span class="informal" title="' . 
					$localName . '">' . htmlentities($title, ENT_NOQUOTES, $encoding) . '</span></li>' . PHP_EOL;
			
			$localName = '';
			$title = '';
		}
		$ret .= '</ul>' . PHP_EOL;
		
		echo $ret;
	}
	
	/**
	  * Responds with the rendered html template.
	  */
	public function gettemplateAction() {
		Zend_Registry::get('owLog')->debug('ServiceController::getTemplateAction()');
		
		// fetch folder parameter
		if (isset($this->_request->f)) {
			$folder = $this->_request->getParam('f');
		} else {
			throw new Exception('Missing parameter f!');
			exit();
		}
		
		// fetch template parameter
		if (isset($this->_request->t)) {
			$template = $this->_request->getParam('t');
		} else {
			throw new Exception('Missing parameter t!');
			exit();
		}
		
		if (!ereg('^[a-z_]+$', $folder) || !ereg('^[a-z_]+$', $template)) {
			throw new Exception('Illegal characters in folder or template name!');
			exit();
		}
		
		$path = REAL_BASE . $this->config->theme->path . 'templates/' . $folder . '/';
		$file =  $template . '.php';
		
		if (!is_readable($path . $file)) {
			$this->log('Template file not readable: ' . $path .  $file, Zend_Log::ERR);
			throw new Exception('Template file not readable. ' . $path .  $file);
			exit();
		}
		
		$this->view->strings = Zend_Registry::get('strings');
		$this->view->owUriBase = $this->config->uriBase;
		$this->view->model = $this->model;
		$this->view->config = $this->config;
		$this->view->q = $this->_request->getParam('q');
		
		$this->view->setScriptPath($path);
		$this->_response->setBody($this->view->render($file));
	}
	
	/**
	  * Attempts an authentication to the underlying Erfurt framework via 
	  * HTTP GET/POST parameters.
	  */
	public function authAction() {
		if (!$this->config->rest->allowGetAuth) {
			// disallow get
			if (!$this->_request->isPost()) {
				header('HTTP/1.0 405 Method Not Allowed');
				header('Allow: POST');
				exit();
			}
		}
	
		// fetch params
		if (isset($this->_request->logout)) {
			$logout = $this->_request->logout == 'true' ? true : false;
		} elseif (isset($this->_request->u)) {
			$username = $this->_request->u;
			$password = $this->_request->getParam('p', '');
		} else {
			header('HTTP/1.0 400 Bad Request');
			// header('');
			exit();
		}
	  
		if ($logout) {
			// logout
			Zend_Auth::getInstance()->clearIdentity();
			session_destroy();
			header('HTTP/1.0 200 OK');
			exit();
		} else {
	  		// authenticate
	  		$result = Zend_Registry::get('erfurt')->authenticate($username, $password);
		}
	  
		// return HTTP result
		if ($result->isValid()) {
	  		// return success (200)
	  		header('HTTP/1.0 200 OK');
	  		exit();
		} else {
	  		// return fail (401)
	  		header('HTTP/1.0 401 Unauthorized');
	  		exit();
		}
	}
	
	/**
	  * Executes a sparql query according to the SPARQL protocol.
	  * Uses Ontowiki-configuration if available in session.
	  */
	public function sparqlAction() {
		//extract the attribut 'query' from the request if available
		if (isset($this->_request->query)) {
			$sparql = $this->_request->query;
		}
		
		try {	
			// use ontowiki settings if available
			$session = new Zend_Session_Namespace('ERFURT' . md5($this->config->session->ns_identifier));
			$endpoint = new Erfurt_Sparql_Endpoint_Default($this->config);
			
			$params = $this->_request->getParams();
			
			if (!is_null($params['default-graph-uri'])) {
				$endpoint->addModel($params['default-graph-uri']);
			}
			
			$endpoint->setQuery($sparql);
			echo $endpoint->query();
		} catch (Exception $e) {
			echo $e->getMessage();
			exit();
		}
	}
	
	/**
	  * Updates the model supplied by model with a diff of old and new.
	  * The parameters old and new must be in one of the following formats:
	  * <ul>
	  *   <li>RDF/JSON</li>
	  *   <!--<li>RDF/XML</li>-->
	  *   <!--<li>RDF/N-Triples</li>-->
	  *   <!--<li>RDF/N3</li>-->
	  * </ul>
	  */
	public function updateAction() {
		// check params

#header('HTTP/1.0 401 Unauthorized');
#header('WWW-Authenticate: Basic realm="Secure Area"');
#exit;

		if (!isset($this->_request->model)) {
			header('HTTP/1.0 400 Bad Request');
			throw new Exception('Missing parameter model!');
			exit();
		}

		if (!isset($this->_request->old) && !isset($this->_request->new)) {
			header('HTTP/1.0 400 Bad Request');
			throw new Exception('At least one of these parameters needed: old or new!');
			exit();
		}

		if (isset($this->_request->old)) {
			$oldModelString = $this->_request->old;
		} else {
			$oldModelString = '{}';
		}
		if (isset($this->_request->new)) {
			$newModelString = $this->_request->new;
		} else {
			$newModelString = '{}';
		}
		if (get_magic_quotes_gpc() == TRUE) {
			$oldModelString = stripslashes($oldModelString);
			$newModelString = stripslashes($newModelString);
		}
		
		// instantiate model to operate on
		try {
			$model = Zend_Registry::get('erfurt')->getStore()->getModel($this->_request->model);
		} catch (Erfurt_Exception $ee) {
			header('HTTP/1.0 400 Bad Request');
			exit();
		}
				
		if (!$model) {
		    if (Zend_Registry::get('erfurt')->getStore()->modelExists($this->_request->model, false)) {
		        header('HTTP/1.0 401 Unauthorized');
                header('WWW-Authenticate: Basic realm="Update Service"');
    			exit;
		    } else {
		        header('HTTP/1.0 400 Bad Request');
    			throw new Exception('Could not instantiate model ' . $this->_request->model);
    			exit();
		    }				
		}
				
		// parse old model
		// TODO: support formats other than JSON
		$oldModel = new MemModel();
		try {
		    $oldModel->loadFromString($oldModelString, 'json');
		} catch (Exception $e) {
		    header('HTTP/1.0 400 Bad Request');
		    throw new Exception('Could not parse json model (old).');
			exit();
		}
				
		// parse new model
		// TODO: support formats other than JSON
		$newModel = new MemModel();
		try {
		    $newModel->loadFromString($newModelString, 'json');
		} catch (Exception $e) {
		    header('HTTP/1.0 400 Bad Request');
		    throw new Exception('Could not parse json model (new).');
			exit();
		}
		
		try {
			$model->update($oldModel, $newModel);
		} catch (Erfurt_Exception $ee) {
			header('HTTP/1.0 400 Bad Request');
			exit();
		}
		
		// return success
		header('HTTP/1.0 200 OK');
		exit();
	}
	
	/**
	 * WebDAV for OntoWiki
	 * basing up on Erfurt WebDAV Extension
	 */
	public function webdavAction() {
		try {	
			// use ontowiki settings if available
			$session = new Zend_Session_Namespace('ERFURT');
		
			if (isset($session->config)) {
				$config = $session->config;
			}
			
			$baseURI = $this -> _request -> getBaseUrl() . '/' . $this -> _request ->getControllerName() .
			'/' . $this -> _request -> getActionName();
			
			$WebDAV = new Erfurt_WebDAV_Server_Default($config,$baseURI);

			$WebDAV -> ServeRequest();
			
		} catch (Exception $e) {
			echo $e->getMessage();
			exit();
		}
	}
	
	public function gettreenodecontentAction() {
		$optionalParams = array();
		
		// complete tree or top elements only
		if (isset($this->_request->complete)) {
			$completeTree = true;
		} else {
			$completeTree = false;
		}
		
		// entry node
		if (isset($this->_request->r)) {
			$optionalParams['entryPoint'] = $this->_request->r;
		}
		
		// search string
		if (isset($this->_request->s)) {
			$optionalParams['search'] = $this->_request->s;
		}
		
		// node type
		if (isset($this->_request->nt)) {
			$nodeType = $this->_request->nt;
		} else {
			header('HTTP/1.0 400 Bad Request');
			throw new Exception('Missing parameter nt (node type)!');
			exit();
		}
		
		// subtype relation
		if (isset($this->_request->sr)) {
			$subRel = $this->_request->sr;
		} else {
			header('HTTP/1.0 400 Bad Request');
			throw new Exception('Missing parameter sr (sub relation)!');
			exit();
		}
		
		// instance relation
		if (isset($this->_request->ir)) {
			$instRel = $this->_request->ir;
		} else {
			header('HTTP/1.0 400 Bad Request');
			throw new Exception('Missing parameter ir (instance relation)!');
			exit();
		}
		
		// instance relation
		if (isset($this->_request->javascript)) {
			$jsMode = $this->_request->javascript;
		} else {
			$jsMode = false;
		}
		
		// get model language from session
		$lang = Zend_Registry::get('session')->modelLanguage;
		
		// check whether store can count
		if ($this->model->getStore() instanceof Erfurt_Store_CountableInterface) {
			$optionalParams['doCount'] = true;
		} else {
			$optionalParams['doCount'] = false;
		}
		
		$resTree = $this->model->resourceTree($nodeType, $subRel, $instRel, $optionalParams);
		
		if ($completeTree) {
			$content = $this->_completeHierarchy($resTree, $lang, $jsMode, $nodeType, $subRel, $instRel);
		} else {
			$content = $this->_showHierarchy($resTree, $lang, $jsMode);
		}
		
		echo $content;
	}
	
	private function _completeHierarchy($resTree, $lang, $jsMode = false, $nodeType, $subRel, $instRel) {
		foreach ($resTree as $row) {
			$r = $this->model->resourceF($row['iri'], false);
			if (isset($row['count'])) {
				$count = $row['count'];
			}
			
			if ($row['hasChildren'] === true) {
				$subTree = $this->model->resourceTree($nodeType, $subRel, $instRel, array('entryPoint' => $row['iri']));
				$sub = PHP_EOL . $this->_completeHierarchy($subTree, $lang, $jsMode, $nodeType, $subRel, $instRel);
			}
			
			$ret .= '<li>' . PHP_EOL;
			
			if ($jsMode) {
				$ret .= '<a style="display:block" class="tree-item' . ($row['hasChildren'] ? '' : ' leaf') . '" about="' . $r->getUri() . '">' 
				     . htmlentities($r->getTitle($r, $lang), ENT_NOQUOTES, 'UTF-8') . ($count ? ' (' . $count . ')' : '') . '</a>';
			} else {
				$ret .= '<a ' . ($row['hasChildren'] ? '' : 'class="leaf"') . ' about="' . $r->getUri() . '" href="' . OntoWiki_Util::getURL($r, null, 'list') . '">' 
				     . htmlentities($r->getTitle($r, $lang), ENT_NOQUOTES, 'UTF-8') . ($count ? ' (' . $count . ')' : '') . '</a>';
			}
			
			$ret .= $sub . PHP_EOL . '</li>' . PHP_EOL;
			
			$subTree = null;
			$sub     = null;
		}
		
		return '<ul class="bullets-none">' . PHP_EOL . $ret . '</ul>';
	}
	
	private function _showHierarchy($resTree, $lang, $jsMode = false) {
		foreach ($resTree as $row) {
			$r = $this->model->resourceF($row['iri'], false);
			if (isset($row['count'])) {
				$count = $row['count'];
			}
			
			if ($row['hasChildren'] === true) {
				if (isset($this->session->nodeState[$row['iri']]) &&
				    $this->session->nodeState[$row['iri']] === 'open') {

					$cssClass = 'tree-open';
					$sub = PHP_EOL .'<ol class="bullets-none"></ol>';
				} else {
					$cssClass = 'tree-closed';
					$sub = PHP_EOL . '<ol class="bullets-none"></ol>';
				}
			} else {
				$cssClass = 'tree-none';
				$sub = '';
			}
			
			if ($cssClass === 'tree-none') {
				$ret .= '<li><span class="' . $cssClass . '"></span>';
			} else {
				$ret .= '<li><a id="hierarchytree-' . uniqid() . '" class="tree-node-toggle ' . $cssClass . '"></a>';
			}
			
			if ($jsMode) {
				$ret .= '<a class="resource-select" about="' . $r->getUri() . '">' . htmlentities($r->getTitle($r, $lang), ENT_NOQUOTES, 'UTF-8') . ($count ? ' (' . $count . ')' : '') . '</a>';
			} else {
				$ret .= '<a about="' . $r->getUri() . '" href="' . OntoWiki_Util::getURL($r, null, 'list') . '">' 
				      . htmlentities($r->getTitle($r, $lang), ENT_NOQUOTES, 'UTF-8') . ($count ? ' (' . $count . ')' : '') . '</a>';
			}
			
			$ret .= $sub . '</li>' . PHP_EOL;
		}
		
		return $ret;
	}
	
	public function tagsuggestionAction() {
	    
		// get all needed parameters from GET and strip slashes if needed
		// stripslashes for all values
		if (get_magic_quotes_gpc()) {
			foreach ($_GET as $key => $value) {
				$params[$key] = stripslashes($value);
			}
		} else {
		    $params = $_GET;
	    }
	    
	    if (isset($params['search'])) {
	        $sql = 'SELECT DISTINCT s.object FROM statements s
    	            WHERE modelID in (' . $this->model->getModelIds() . ') AND
    	            s.predicate = "http://www.holygoat.co.uk/owl/redwood/0.1/tags/name" AND 
    	            s.object LIKE "' . $row[0] . '%" 
    	            ORDER BY s.object ASC';
	    } else {
	        $sql = 'SELECT DISTINCT s.object FROM statements s
    	            WHERE modelID in (' . $this->model->getModelIds() . ') AND
    	            s.predicate = "http://www.holygoat.co.uk/owl/redwood/0.1/tags/name" 
    	            ORDER BY s.object ASC';
	    }
	    
	    
	    
	    $result = $this->model->getStore()->sqlQuery($sql);
	    
	    if (count($result) > 0) {
	        $content = array();
	        
	        foreach ($result as $i=>$row) {
	            $content[] = $row[0];
	        }
	        
	        echo json_encode($content);
	    } else {
	        echo json_encode(array());
	    }
	}
	
	/**
	 * Renders comments for a given resource
	 */
	public function commentsAction() {
		$comments = $this->config->comments;
		$resource = isset($this->_request->r) ? $this->_request->r : $this->resource->getURI();
		$showValuations = isset($this->_request->val) ? $this->_request->val : 'true';
		$showValuations = $showValuations=='true' ? true : false;
		$showComments = isset($this->_request->com) ? $this->_request->com : 'true';
		$showComments = $showComments=='true' ? true : false;
		
		$user    = Zend_Auth::getInstance()->getIdentity();
		$userUri = $user['uri'];
			
		if (isset($this->_request->add)) {
			// new comment -> save it
			if (get_magic_quotes_gpc()) {
				$text = stripslashes($this->_request->add);
			} else {
				$text = $this->_request->add;
			}
			
			$name    = $user['username'];
			$date    = date('Y-m-d\TH:i:s'); // xsd:datetime
			
			if (!$this->model->isEditable()) {
				throw new Erfurt_Exception('Model not writable.');
			}
			
			if (!$resource) {
				throw new Erfurt_Exception('Nothing to comment on.');
			}
			
			/* all data fetched, now let's create resources */
			$commentUri      = $this->model->getUniqueResourceURI('comment');
			$commentResource = $this->model->instanceF($commentUri, false);
			$userResource    = $this->model->instanceF($userUri, false);
			
			// 1. comment sioc:about resource
			// avoid unnecessary instantiation
			$resourceObj = isset($this->_request->r) ? $this->model->resourceF($resource) : $this->resource;
			$commentResource->setPropertyValue($comments->about, $resourceObj);
			
			// 2. comment content $text
			$commentResource->setPropertyValue($comments->content, $text);
			
			// 3. comment rdf:type sioc:post
			$commentResource->setPropertyValue(EF_RDF_TYPE, $this->model->resourceF($comments->type, false));
			
			// 4. comment sioc:created_at $date
			$commentResource->setPropertyValue($comments->date, $date);
			
			// 5. $userUri sioc:creator_of comment
			$userResource->addPropertyValue($comments->creator, $commentResource);
			
			// 6. $userUri rdfs:label $name
			$userResource->setPropertyValue(EF_RDFS_LABEL, $name);
			
			// $this->view->content = '<p class="messagebox info">Your comment has been added.</p>';
			// $this->view->title = 'Comments';
			// $this->_response->setBody($this->view->render('micro/window.php'));
			
			// hack ;)
			$this->_request->setParam('list', 'list');
		}
		if (isset($this->_request->addValuation)) {
			// new or edited valuation -> save it
			if (get_magic_quotes_gpc()) {
				$text = stripslashes($this->_request->addValuation);
			} else {
				$text = $this->_request->addValuation;
			}
			$valuation = $this->_request->valuation;
						
			$name    = $user['username'];
			$date    = date('Y-m-d\TH:i:s'); // xsd:datetime
			
			if (!$this->model->isEditable()) {
				throw new Erfurt_Exception('Model not writable.');
			}
			
			if (!$resource) {
				throw new Erfurt_Exception('Nothing to comment on.');
			}
			
			//if valuation was edited delete old one
			if (isset($this->_request->edit)){
				$removeValuationSql = 'DELETE FROM statements where subject ="' . $this->_request->edit . '" OR object = "' . $this->_request->edit . '"';
				$store = $this->model->getStore();
				$result = $store->sqlQuery($removeValuationSql);
			}
			
			/* all data fetched, now let's create resources */
			$valuationUri      = $this->model->getUniqueResourceURI('valuation');
			$valuationResource = $this->model->instanceF($valuationUri, false);
			$userResource    = $this->model->instanceF($userUri, false);
			
			// 1. comment sioc:about resource
			// avoid unnecessary instantiation
			$resourceObj = isset($this->_request->r) ? $this->model->resourceF($resource) : $this->resource;
			$valuationResource->setPropertyValue($comments->about, $resourceObj);
			
			// 2. valuation number content $valuation
			$valuationResource->setPropertyValue($comments->content, $text);
			
			// 7. valuation content $text
			$valuationResource->setPropertyValue($comments->note, $valuation);
						
			// 3. comment rdf:type sioc:post
			$valuationResource->setPropertyValue(EF_RDF_TYPE, $this->model->resourceF($comments->valuationsType, false));
			
			// 4. comment sioc:created_at $date
			$valuationResource->setPropertyValue($comments->date, $date);
			
			// 5. $userUri sioc:creator_of comment
			$userResource->addPropertyValue($comments->creator, $valuationResource);
			

			// 6. $userUri rdfs:label $name
			$userResource->setPropertyValue(EF_RDFS_LABEL, $name);
			
			// $this->view->content = '<p class="messagebox info">Your comment has been added.</p>';
			// $this->view->title = 'Comments';
			// $this->_response->setBody($this->view->render('micro/window.php'));
			
			// hack ;)
			
			$this->_request->setParam('list', 'list');
		}
		
		// Setting prefix for css Classes and for id's
		if (isset($this->_request->prefix)) {
			$this->view->prefix = $this->_request->prefix;
		} else {
			$this->view->prefix = 'ontowiki';	
		}
		
		if (isset($this->_request->list)) {
			if (!$resource) {
				return;
			}
			// return a list of comments for the current resource
			$commentSparql = '
				SELECT DISTINCT ?comment, ?creator, ?text, ?timestamp, ?name, ?note
				WHERE {
					?comment <' . $comments->about . '> <' . $resource . '>.
					?comment <' . $comments->content . '> ?text.
					?comment <' . $comments->date . '> ?timestamp.
					?creator <' . $comments->creator . '> ?comment.
					?creator <' . EF_RDFS_LABEL . '> ?name.
					OPTIONAL {?comment <' . $comments->note . '> ?note.}
				}
				ORDER BY DESC(?timestamp) ASC(?text)
			';
						
			// echo $commentSparql;
			// $comments = $this->model->sparqlQuery($commentSparql);
			$this->view->config = Zend_Registry::get('config');
			$this->view->strings = Zend_Registry::get('strings');
			$comments=$this->model->sparqlQuery($commentSparql, new Erfurt_Sparql_ResultRenderer_Plain);
			$this->view->comments = $comments;
			$numberOfComments=0;
			$numberOfValuations=0;
			$averageValuation=0;
			$userComment='';
			$editCommentURI='';
			$userValuation=0;
			$userHasValuation=false;
			foreach ($comments as $comment){
				if (strlen($comment['note'])>0){
					$numberOfValuations++;
					$averageValuation+=$comment['note'];
				}
				else $numberOfComments++;
				if ($userUri==$comment['creator']&&strlen($comment['note'])>0){
					$userComment=$comment['text'];
					$userValuation=$comment['note'];
					$userHasValuation=true;
					$editCommentURI=$comment['comment'];
				}
			}
			$this->view->numberOfComments=$numberOfComments;
			$this->view->numberOfValuations=$numberOfValuations;
			$this->view->averageValuation= $numberOfValuations>0 ? round($averageValuation/$numberOfValuations) : 0;
			$this->view->showValuations = $showValuations;
			$this->view->showComments = $showComments;
			$this->view->userComment=$userComment;
			$this->view->userValuation=$userValuation;
			$this->view->userHasValuation=$userHasValuation;
			$this->view->editCommentURI=$editCommentURI;
			$this->view->resourceuri=$resource;
			$this->view->content  = $this->view->render('partials/comments.php');
			$this->view->title = $this->view->strings->comments->title;
			$this->view->cssClasses = 'softwiki-comments';
			$this->_response->setBody($this->view->render('micro/window.php'));
		}
	}
	
	public function tagsAction() {
		//mandatory
		$tagProperty      = $this->_request->tagProperty;
		$tagLabelProperty = $this->_request->labelProperty;
		
		// optional
		$resourceUri      = isset($this->_request->r) ? $this->_request->r : $this->resource->getURI();
		$resource         = isset($this->_request->r) ? $this->model->resourceF($this->_request->r) : $this->resource;
		$min              = $this->_request->minSize ? $this->_request->minSize : 0.6;
		$max              = $this->_request->maxSize ? $this->_request->maxSize : 1.25;
		$nonLinScale      = $this->_request->nonLinearScale ? $this->_request->nonLinearScale : 0.5;
		$jsMode           = true;
		
		$tags             = isset($this->_request->tags) ? $this->_request->tags : null;
		
		$selectedTagUri   = isset($this->_request->tagUri) ? $this->_request->tagUri : null;
		
		if ($tags) {
			if (get_magic_quotes_gpc()) {
				$tags = stripslashes($tags);
			}
			$tags = json_decode($tags);
		}
		
		if (!($tagProperty && $tagLabelProperty)) {
			return;
		}
		
		if (isset($this->_request->addTagDescription)) {
			// new tag description -> save it
			if (get_magic_quotes_gpc()) {
				$text = stripslashes($this->_request->addTagDescription);
			} else {
				$text = $this->_request->addTagDescription;
			}
			$addTagUri=$this->_request->addTagUri;
									
			if (!$this->model->isEditable()) {
				throw new Erfurt_Exception('Model not writable.');
			}
			
			if (!$addTagUri) {
				throw new Erfurt_Exception('Nothing to make description on.');
			}
			
			/* all data fetched, now let's create resources */
			$tagResource = $this->model->instanceF($addTagUri, false);
						
			// avoid unnecessary instantiation
			$resourceObj = isset($this->_request->r) ? $this->model->resourceF($resource) : $this->resource;
			//add comment to tag
			$tagResource->setPropertyValue(EF_RDFS_COMMENT, $text);
		}
		else if (isset($this->_request->editTagDescription)) {
			// new tag description -> save it
			if (get_magic_quotes_gpc()) {
				$text = stripslashes($this->_request->editTagDescription);
			} else {
				$text = $this->_request->editTagDescription;
			}
			$addTagUri=$this->_request->addTagUri;
									
			if (!$this->model->isEditable()) {
				throw new Erfurt_Exception('Model not writable.');
			}
			
			if (!$addTagUri) {
				throw new Erfurt_Exception('Nothing to make description on.');
			}
			
			//if tag-description was edited delete old one
			if (isset($this->_request->edit)){
				$removeTagDescriptionSql = 'DELETE FROM statements where subject ="' . $addTagUri . '" AND predicate = "' . EF_RDFS_COMMENT . '"';
				$store = $this->model->getStore();
				$result = $store->sqlQuery($removeTagDescriptionSql);
			}
			
			/* all data fetched, now let's create resources */
			$tagResource = $this->model->instanceF($addTagUri, false);
						
			// avoid unnecessary instantiation
			$resourceObj = isset($this->_request->r) ? $this->model->resourceF($resource) : $this->resource;
			//add comment to tag
			$tagResource->setPropertyValue(EF_RDFS_COMMENT, $text);
		}
		
		if ($resourceUri == SWORE_REQUIREMENT) {
			$sql = '
				SELECT s1.object as tagUri, COUNT(s1.object) as tagCount, s2.object as tagName
					FROM statements s1, statements s2';
			for ($i=0;$i<count($tags);$i++)
				$sql.=', statements s'.($i+3);
			$sql.='	WHERE s1.predicate = "' . $tagProperty . '"
					AND s2.predicate = "' . $tagLabelProperty . '"
					AND s2.object_is = "l"
					AND s1.object = s2.subject
					AND s1.modelID IN (' . $this->model->getModelIds() . ')';
			for ($i=0;$i<count($tags);$i++){
				$sql.='AND s1.subject=s'.($i+3).'.subject
					   AND s'.($i+3).'.predicate="'. $tagProperty .'"
					   AND s'.($i+3).'.object="'.$tags[$i].'"';
			}
			$sql.='	GROUP BY s1.object
					ORDER BY tagCount DESC
					LIMIT 50';
		} else {
			$subcategories=array();
			//get direct subcategories
			$sql='SELECT subject
					FROM statements 
					WHERE object = "' . $resourceUri . '"
					AND predicate = "http://www.w3.org/2004/02/skos/core#broader"';
			$sqlResult = $this->model->dbConn->Execute($sql);
			while ($tagData = $sqlResult->FetchRow()){
				$subcategories[]=$tagData[0];
			}
			$i=0;
			//get all subsub...categories
			while (isset($subcategories[$i])){
				$sql='SELECT subject
						FROM statements 
						WHERE object = "' . $subcategories[$i] . '"
						AND predicate = "http://www.w3.org/2004/02/skos/core#broader"';
				$sqlResult = $this->model->dbConn->Execute($sql);
				while ($tagData = $sqlResult->FetchRow()){
					$subcategories[]=$tagData[0];					
				}
				$i++;
			}
						
			$sql = '
				SELECT s1.object as tagUri, COUNT(s1.object) as tagCount, s2.object as tagName
				FROM statements s1, statements s2';
			for ($i=0;$i<count($tags);$i++)
				$sql.=', statements s'.($i+3);
				
			$sql.='	WHERE s1.predicate = "' . $tagProperty . '"
				AND s1.object = s2.subject
				AND s2.predicate = "' . $tagLabelProperty . '"
				AND s2.object_is = "l"
				AND s1.subject IN (
					SELECT subject 
					FROM statements 
					WHERE object IN ("' . $resourceUri . '"';
			foreach ($subcategories as $sc)
				$sql.=',"'.$sc.'"';		
			$sql.='	)

					AND predicate = "http://www.w3.org/2004/02/skos/core#primarySubject"
				)
				AND s1.modelID IN (' . $this->model->getModelIds() . ')';
			for ($i=0;$i<count($tags);$i++){
				$sql.='AND s1.subject=s'.($i+3).'.subject
					   AND s'.($i+3).'.predicate="'. $tagProperty .'"
					   AND s'.($i+3).'.object="'.$tags[$i].'"';
			}	
			$sql.='
				GROUP BY s1.object
				ORDER BY tagCount DESC
				LIMIT 50
			';
		}
				
		// get ado result set to be used with cursor
		$sqlResult = $this->model->dbConn->Execute($sql);
		
		$tagName   = '';
		$tags      = array();
		$firstRow  = $sqlResult->getRowAssoc();
		$maxCount  = (int) $firstRow['TAGCOUNT'];
		$uriBase   = $this->config->uriBase;
		
		if ($resource->isClass()) {
			$className = $this->model->classF($resource)->getQualifiedName();
		}
		
		$selectedTagDescription='';
		
		// define variables for ordering
		$tagsToOrder = array();
		$orderField = array();
		
		// make strings available
		$strings = Zend_Registry::get('strings');
		
		while ($tagData = $sqlResult->FetchRow()) { // 0 => uri, 1 => count, 2 => name
			if (is_array($_REQUEST['select']) and in_array($tagData[0], $_REQUEST['select'])) {
				$classes = 'selected';
				$selected = true;
			} else {
				$classes = '';
				$selected = false;
			}
			
			// replace missing names by URIs
			if (isset($tagData[2])) {
				$tagName = htmlentities($tagData[2], ENT_NOQUOTES, 'UTF-8');
			} else {
				$tagName = $tagData[0];
			}
			
			//get name of selected tag
			if ($tagData[0]==$selectedTagUri){
				$selectedTag=$tagName;
			}
			
			// calculate scale
			$plainScale   = pow((int) $tagData[1] / $maxCount, $nonLinScale);
			$opacityScale = $plainScale * (1 - $min) + $min;
			$sizeScale    = ($plainScale * ($max - $min) + $min) * 100;
			
			// highlight selected tag
			if ($selected) {
				// remove filter
				$filter = '';
			} else {
				// set filter
				$filter = sprintf('?select[%s]=%s', urlencode($tagProperty), urlencode($tagData[0]));
			}
			
			//look, if there is a description for the tag available
			$sql='
				SELECT s1.object as tagDescription
					FROM statements s1
					WHERE s1.predicate = "' . EF_RDFS_COMMENT . '"
					AND s1.subject = "' . $tagData[0] . '"
					LIMIT 1;
			';
			$sqlResult2 = $this->model->dbConn->Execute($sql);
			if ($tagDescriptionData=$sqlResult2->FetchRow()){
				if ($tagData[0]==$selectedTagUri) {
					$selectedTagDescription=$tagDescriptionData[0];
				}
				$tagDescriptionData = $tagDescriptionData[0] . ' - (' . $strings->tags->numberOf . ' ' . $tagData[1] . ')';
				$color='tagGreen';
			}
			else{
				$tagDescriptionData = '(' . $strings->tags->numberOf . ' ' . $tagData[1] . ')';
				$color='tagBlue';
			}
			
			// create order and data arrays for multisort
			$tagsToOrder[] = array($classes, $color, $tagData[0], $sizeScale, $opacityScale, $tagDescriptionData, $tagName, $tagData[1]);
			$orderField[] = $tagName;

		}
		// order tags lexicograhpically
		array_multisort($orderField, SORT_ASC, SORT_STRING, $tagsToOrder);
		
		foreach ($tagsToOrder as $sTagData) {
			// construct tag link
			$tags[] = sprintf('<a class="tag javascript-on %s %s" about="%s" style="font-size:%d%%;opacity:%f" title="%s">%s<span class="onlyAural">(%d)</span></a> ', 
			                  $sTagData[0],
			                  $sTagData[1], 
			                  $sTagData[2], 
			                  $sTagData[3], 
			                  $sTagData[4], 
			                  $sTagData[5],
			                  $sTagData[6], 
			                  $sTagData[7]);
		}

		if (empty($tags)) {
			// empty resultset
			echo '<div class="messagebox info">No Matches.</div>';
		} else {
			$this->view->config = Zend_Registry::get('config');
			$this->view->strings = Zend_Registry::get('strings');
			$this->view->tagList=$this->view->makeList($tags, 'ol', 'bullets-none inline');
			$this->view->tagName=$selectedTag;
			$this->view->tagUri=$selectedTagUri;
			$this->view->tagDescription=$selectedTagDescription;
			echo $this->view->render('partials/tags.php');
		}
	}
}
