<?php
/**
 * @package auth
 */
class Erfurt_Auth_Adapter_RDF implements Zend_Auth_Adapter_Interface
{

	/**
	 * Digest authentication user
	 *
	 * @var string
	 */
	protected $_username;

	/**
	 * Password for the user of the realm
	 *
	 * @var string
	 */
	protected $_password;

	/**
	 * ac model
	 *
	 * @var object
	 */
	protected $_acModel;

	private $_users = array();

	private $userFetched = false;

	private $_anonymousUserUri = 'http://ns.ontowiki.net/SysOnt/Anonymous';

	private $_userClass = 'http://xmlns.com/foaf/0.1/Agent';
	private $_userNamePredicate = 'http://xmlns.com/foaf/0.1/nick';
	private $_userPasswordPredicate = 'http://ns.ontowiki.net/SysOnt/userPassword';
	private $_userMailPredicate = 'http://xmlns.com/foaf/0.1/mbox';
	private $_userDenyActionPredicate = 'http://ns.ontowiki.net/SysOnt/denyAccess';
	private $_userDenyLoginAction = 'http://ns.ontowiki.net/SysOnt/denyAccess';
	private $_userPasswordResetHash = 'http://ns.ontowiki.net/SysOnt/resetHash';
	private $_userPasswordResetTime = 'http://ns.ontowiki.net/SysOnt/resetValidTime';

	/**
	 * super admin user uri
	 */
	private $_defaultSuperUserUri = 'http://ns.ontowiki.net/SysOnt/SuperAdmin';

	private $_dbUsername = '';
	private $_dbPassword = '';

	/**
	 * Sets username and password for authentication
	 *
	 * @return void
	 */
	public function __construct($acModel, $username, $password) {
		if (Zend_Registry::isRegistered('erfurtLog')) {
			Zend_Registry::get('erfurtLog')->debug('Erfurt_Auth_Adapter_RDF::_constuctor()');
		}

		$this->_acModel = $acModel;

		$this->_username = $username;
		$this->_password = $password;

		$this->_userClass = Zend_Registry::get('config')->ac->user->class;
		$this->_userNamePredicate = Zend_Registry::get('config')->ac->user->name;
		$this->_userPasswordPredicate = Zend_Registry::get('config')->ac->user->pass;
		$this->_userMailPredicate = Zend_Registry::get('config')->ac->user->mail;

		$this->_anonymousUserUri = Zend_Registry::get('config')->ac->user->anonymousUser;
		$this->_defaultSuperUserUri = Zend_Registry::get('config')->ac->user->superAdmin;

		$this->_dbUsername = Zend_Registry::get('config')->database->params->username;
		$this->_dbPassword = Zend_Registry::get('config')->database->params->password;


	}

	/**
	 * Performs an authentication attempt
	 *
	 * @throws Zend_Auth_Adapter_Exception If authentication cannot be performed
	 * @return Zend_Auth_Result
	 */
	public function authenticate() {
		Zend_Registry::get('erfurtLog')->debug('Erfurt_Auth_Adapter_RDF::authenticate()');

		# returnset
		$result = array(
	           'isValid'  => false,
	           'identity' => array(
	               'username' => $this->_username,
	               'uri'	=>	'',
	               'dbuser'	=>	false,
	               'anonymous' => false
		),
	           'messages' => array()
		);
		
		if ($this->_username == 'Anonymous') {
			$result['isValid'] = true;
			$result['identity'] = $this->_getAnonymous();
		}
		# database user
		else if ($this->_username == $this->_dbUsername and $this->_password == $this->_dbPassword) {
				
			$result['isValid'] = true;
			$result['identity'] = $this->_getSuperAdmin();
		}
		# normal user
		else if (($userUri = $this->fetchData($this->_username, $this->_password)) === false) {
			// TODO bug #1881292 (authentification problem) strings are ow specific... not available in erfurt; just a quick hack
			#$result['messages'][] = Zend_Registry::get('strings')->auth->login->msg->incorrect;
			$result['messages'][] = 'Incorrect login data!';
		} else {
			# valid user from sysont
			$result['isValid'] = true;
			$result['identity']['uri'] = $userUri;
			$result['identity']['email'] = $this->_users[$userUri]['userEmail'];
				
			# TODO: POWL HACK
			//$_SESSION['PWL']['user']= $this->_username;
		}

		if (Zend_Registry::isRegistered('erfurtLog')) {
			Zend_Registry::get('erfurtLog')->debug('User authenticated: ' . $result['identity']['uri']);
		}

		return new Zend_Auth_Result($result['isValid'], $result['identity'], $result['messages']);
	}


	/**
	 * fetch the user data from rdf-store
	 */
	function fetchUserData() {
		Zend_Registry::get('erfurtLog')->debug('Erfurt_Auth_Adapter_RDF::fetchUserData()');
		$this->userFetched = true;

		# no system-ontology
		if (empty($this->_acModel->modelURI)) {
			return false;
		}
		 
		## direct user rights
		$sparqlQuery = 'select ?s ?p ?o
														where { 
														?s ?p ?o								
														?s rdf:type <'.$this->_userClass.'>.
														}';
			
		if (!$result = $this->_sparql($sparqlQuery)) {
			return false;
		}

		foreach($result as $e) {
				
			if ($e['p'] == $this->_userNamePredicate)
			$this->_users[$e['s']]['userName'] = $e['o'];
			else if ($e['p'] == $this->_userPasswordPredicate)
			$this->_users[$e['s']]['userPassword'] = $e['o'];
			else if ($e['p'] == $this->_userMailPredicate)
			$this->_users[$e['s']]['userEmail'] = $e['o'];
			else if ($e['p'] == $this->_userMailPredicate and $e['o'] == $this->_userDenyLoginAction)
			$this->_users[$e['s']]['loginForbidden'] = true;
		}
	}

	/**
	 * Fetch data from rdf store
	 *
	 * @access public
	 */
	public function fetchData($username, $password, $isChallengeResponse=false) {
		Zend_Registry::get('erfurtLog')->debug('Erfurt_Auth_Adapter_RDF::fetchData()');
		# no system-ontology
		if (empty($this->_acModel->modelURI)) {
			return false;
		}
		 
		## direct user rights
		$sparqlQuery = 'select ?s ?p ?o
														where { 
														?s ?p ?o.								
														?s rdf:type <'.$this->_userClass.'>.
														?s <'.$this->_userNamePredicate.'> "'.$username.'"^^<' . EF_XSD_NS . 'string>
														}';
		if (!$result = $this->_sparql($sparqlQuery)) {
			return false;
		}
			

		$uri = '';
		$checkedPassword = false;
		foreach($result as $e) {
				
			## user check
			# wront username => not possible :)
			#if ($e['p'] == $this->_userNamePredicate and $e['s'] != $username)
			#	return false;
			# wrong password
			if ($e['p'] == $this->_userPasswordPredicate ) {
				$checkedPassword = true;
				if (	!$this->verifyPassword($password, $e['o'], 'sha1') and
				!$this->verifyPassword($password, $e['o'], 'md5') and
				!$this->verifyPassword($password, $e['o'], 'crypt') and
				!$this->verifyPassword($password, $e['o'], ''))
				return false;
			}
	   
			# set params
			if ($uri == '')
			$uri = $e['s'];
			if ($e['p'] == $this->_userNamePredicate) {
				$this->_users[$e['s']]['userName'] = $e['o'];
			}
			else if ($e['p'] == $this->_userPasswordPredicate)
			$this->_users[$e['s']]['userPassword'] = $e['o'];
			else if ($e['p'] == $this->_userMailPredicate)
			$this->_users[$e['s']]['userEmail'] = $e['o'];
			# user denied to login
			else if ($e['p'] == $this->_userMailPredicate and $e['o'] == $this->_userDenyLoginAction)
			return false;
		}

		# no password
		if (!$checkedPassword and $password != '')
		return false;
		return  $uri;
	}

	 

	/**
	 * Returns a list of users available within the container
	 *
	 * @return array
	 */
	public function listUsers() {
		Zend_Registry::get('erfurtLog')->debug('Erfurt_Auth_Adapter_RDF::listUsers()');
		if (!$this->userFetched) {
			$this->fetchUserData();
		}
	  
		$ret = array();
		return $this->_users;
	}
	/**
	 * generate a random password
	 *
	 * @param int length, default is 8 characters
	 * @return random string
	 */
	public function generateRandomPassword($length = 8) {
		Zend_Registry::get('erfurtLog')->info('Erfurt_Auth_Adapter_RDF::generateRandomPassword()');

		// Allowed chars in random password
		$allowedChars = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
		$randPasswd = '';
		$numberAllowedChars = (strlen($allowedChars))-1;
		for($i=0;$i<=$length;$i++) {
			$randPasswd.= substr($allowedChars,mt_rand(0,$numberAllowedChars),1);
		}
		return $randPasswd;
	}


	/**
	 * Crypt and verfiy the entered password
	 *
	 * @param  string Entered password
	 * @param  string Password from the data container (usually this password
	 *                is already encrypted.
	 * @param  string Type of algorithm with which the password from
	 *                the container has been crypted. (md5, crypt etc.)
	 *                Defaults to "md5".
	 * @return bool   True, if the passwords match
	 */
	private function verifyPassword($password1, $password2, $cryptType = "md5") {
		Zend_Registry::get('erfurtLog')->debug('Erfurt_Auth_Adapter_RDF::verifyPassword()');

		switch ($cryptType) {
			case "crypt" :
				return ((string)crypt($password1, $password2) === (string)$password2);
				break;
			case "none" :
			case "" :
				return ((string)$password1 === (string)$password2);
				break;
			case "md5" :
				return ((string)md5($password1) === (string)$password2);
				break;
			case "sha1" :
				return ((string)sha1($password1) === (string)$password2);
				break;
			default :
				if (function_exists($cryptType)) {
					return ((string)$cryptType($password1) === (string)$password2);
				} elseif (method_exists($this,$cryptType)) {
					return ((string)$this->$cryptType($password1) === (string)$password2);
				} else {
					return false;
				}
				break;
		}
	}

	/**
	 * parse sparql query
	 */
	private function _sparql($sparqlQuery) {
		static $prefixed_query;

		Zend_Registry::get('erfurtLog')->debug('Erfurt_Auth_Adapter_RDF::_sparql()');

		# get all
		if ($prefixed_query == '') {
			$prefixed_query = '';
			foreach ($this->_acModel->getParsedNamespaces() as $uri => $prefix) {
				$prefixed_query .= 'PREFIX ' . $prefix . ': <' . $uri . '>' . PHP_EOL;
			}
		}


		# query model
		try {
			$renderer = new Erfurt_Sparql_ResultRenderer_Plain();
			$result = $this->_acModel->sparqlQuery($prefixed_query.$sparqlQuery, $renderer, false);

		} catch (SparqlParserException $e) {
			Zend_Registry::get('erfurtLog')->info('Erfurt_Auth_Adapter_RDF::_sparql() - query contains the following error: '.$e->getMessage());
			return false;
		} catch (Exception $e) {
			Zend_Registry::get('erfurtLog')->info('Erfurt_Auth_Adapter_RDF::_sparql() - There was a problem with your query, most likely due to a syntax error.: '.$e->getMessage());
			return false;
		}

		return $result;
	}


	/**
	 * delievers the anonymous user details
	 */
	private function _getAnonymous() {
		$user['username'] = 'Anonymous';
		$user['uri'] = $this->_anonymousUserUri;
		$user['dbuser'] = false;
		$user['email'] = '';
		$user['anonymous'] = true;
		return $user;
	}

	/**
	 * delievers the anonymous user details
	 */
	private function _getSuperAdmin() {
		$user['username'] = 'SuperAdmin';
		$user['uri'] = $this->_defaultSuperUserUri;
		$user['dbuser'] = true;
		$user['email'] = '';
		$user['anonymous'] = false;
		return $user;
	}

	/**
	 *  Method to send a reset link to a given Username/Email
	 *
	 *  @param string username
	 *  @param boolean send info via mail (unused by now)
	 *  @return boolean true if the information has been successfully sent, false otherwise
	 *
	 */
	public function sendResetLink($username = "", $viaMail = true) {

		$list = $this->listUsers();
		$zvE = new Zend_Validate_EmailAddress();
		$forbidden = array('anonymous' , 'admin' , 'superadmin');
		if ( in_array( strtolower($username) , $forbidden) ) {
			return false;
		} else if ( $zvE->isValid($username) ) {
			// check if it is an email address
			foreach ($list as $value) {
				if ($value['userEmail'] == 'mailto:' . $username) {
					$email = $username;
					$username = $value['userName'];
					break;
				}
			}
		} else if ( array_key_exists($this->_acModel->getModelURI() . $username , $list) ) {
			// check if it is an valid username
			$email = substr($list[$this->_acModel->getModelURI() . $username]['userEmail'],7);
		} else {
			return false;
		}
		$userUri = $this->_acModel->getModelURI() . $username;

		$idhash =  md5(uniqid());
		$time = time() + (24 * 60 * 60);

		$userRes = $this->_acModel->resourceF($userUri);
		$userRes->removePropertyValues($this->_userPasswordResetHash);
		$userRes->removePropertyValues($this->_userPasswordResetTime);

		$this -> _acModel -> add(
		$userUri,
		$this->_userPasswordResetHash,
		new  Erfurt_Rdfs_Literal_Default($idhash)
		);
		$this->_acModel->add(
		$userUri,
		$this->_userPasswordResetTime,
		new  Erfurt_Rdfs_Literal_Default ($time)
		);

		$config = Zend_Registry::get('config');

		if ($config->ServerPort) {
			$port = ':' . $config->ServerPort;
		} else {
			$port = '';
		}
		$url = $config->ServerProtocol . $_SERVER['HTTP_HOST'] . $port
		. $config->uriBase . 'wiki/recoverpassword?step=2&hash=' . $idhash;
		// TODO templatize mail things (in Erfurt or OW?)
		// $view = new Zend_View();
		// $view->setScriptPath(ERFURT_BASE);
		// var_dump(ERFURT_BASE);exit();
		// echo $view->render('templates/mail/mail_en.php');exit();

		// echo 'Generated Hash: ' . $idhash . ' with time to live: ' . $time . ' <a href="'
		//	. $url . '" > link to next step </a>';
		try {
			$mail = new Zend_Mail();
			$mail->setBodyHtml('<p>You requested a link to reset your ontowiki password on '
			. $_SERVER['HTTP_HOST'] . '.</br> Follow this <a href="'
			. $url . '">link</a> to set a new password</p><p></p>'
			. '<p> Do <b> NOT </b> answer on this message, this is an automatic mail</p>');
			$mail->setFrom('noreply@' . $config->mail->param->hostname, 'OntoWiki Password Recovery');
			$mail->addTo($email, $username);
			$mail->setSubject('Requested OntoWiki Password Recovery');
			$mail->send();
			return true;
		} catch (Exception $e) {
			throw new Erfurt_Exception('failed to send email. mail feature for php may not be configured.',2002);
			return false;
		}
	}

	/**
	 *  Function that checks whether an password reset link hash exists inside system ontology.
	 *
	 *  @param string hashed password
	 *  @return boolean true if this hash is valid false else
	 */
	public function checkResetLink($hash = "0") {
		$valid = $this -> _sparql ('SELECT * WHERE { ?s <' . $this->_userPasswordResetHash . '> "' . $hash . '" . '
		. '?s <' . $this->_userPasswordResetTime . '> ?o . }');
		if (!empty($valid) && $valid[0]['o'] > time() ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 *  Method to sent a username to a given (registered) email.
	 *
	 *  @param string email to send username to
	 *  @param boolean send info via mail (unused by now)
	 *  @return boolean true if the email is registered false otherwise
	 */
	public function sendUsername($email, $viaMail = true) {
		$list = $this->listUsers();
		$zvE = new Zend_Validate_EmailAddress();
		if ($zvE->isValid($email) ) {
			foreach ($list as $value) {
				if ($value['userEmail'] == 'mailto:' . $email) {
					$username = $value['userName'];
					break;
				}
			}
			if ( !empty($username) ) {
				// TODO templatize mail things (in Erfurt or OW?)
				try {
					$mail = new Zend_Mail();
					$mail->setBodyHtml('<p>You requested your Ontowiki username on ' . $_SERVER['HTTP_HOST']
					. '.</br> Your username is: '. $username . ' .</p>'
					. '<p> Do <b> NOT </b> answer on this message, this is an automatic mail</p>');
					$mail->setFrom('noreply@' . $config->mail->param->hostname, 'OntoWiki Username Recovery');
					$mail->addTo($email, $username);
					$mail->setSubject('Requested OntoWiki Username Recovery');
					$mail->send();
					return true;
				} catch (Exception $e) {
					throw new Erfurt_Exception('failed to send email. mail feature for php may not be configured.',2002);
					return false;
				}
			} else {
				return false;
			}
		} else {
			return false;
		}

	}

	/**
	 *  Method to set a new password to a given password reset hash
	 *
	 *  @param string password1
	 *  @param string password2 (to match password1)
	 *  @param string pwd-recovery-hash to use for transactions (defining user indirectly)
	 *  @return boolean true if all actions were exectuted successful false otherwise
	 */
	public function setPasswordOnHash ($password1 , $password2, $hash) {
		if ($password1 != $password2 || strlen($password1) < 5) {
			return false;
		} else {
			$hashquery = 'SELECT ?s WHERE {?s <' . $this->_userPasswordResetHash . '> "' . $hash . '". }';
			$hashresult = $this->_sparql($hashquery);
			if ( !empty($hashresult) ) {
				$userRes = $this->_acModel->resourceF($hashresult[0]['s']);
				$userRes->removePropertyValues($this->_userPasswordResetHash);
				$userRes->removePropertyValues($this->_userPasswordResetTime);
				$userRes->removePropertyValues($this->_userPasswordPredicate);
				$userRes->setPropertyValues($this->_userPasswordPredicate,
				new Erfurt_Rdfs_Literal_Default(sha1($password1)) );
				return true;
			} else {
				return false;
			}
		}
	}

	/**
	 *  Add a new user to the System-Ontology
	 */
	public function addUser($username, $email, $password1, $password2) {
		$list = $this->listUsers();
		$erfurtApp = Zend_Registry::get('erfurt');
		$config = Zend_Registry::get('config');
		$zvE = new Zend_Validate_EmailAddress();

		if (!$zvE->isValid($email) ) {
			$exceptionString = 'invalid email address (validated by Zend_Validate_EmailAddress)';
			$code = 1302;
		} elseif ( !$erfurtApp->getAc()->isActionAllowed('RegisterNewUser')
		|| !($actionConfig = $erfurtApp->getAc()->getActionConfig('RegisterNewUser'))) {
			$exceptionString = 'register action not allowed for user';
			$code = 1303;
		} elseif (!empty($actionConfig['uidregexp']) && !preg_match($actionConfig['uidregexp'], $username)) {
			$exceptionString = 'invalid characters in username';
			$code = 1304;
		} elseif (strlen($password1) < 5 || $password1 != $password2) {
			$exceptionString = 'password not matching and/or too short (min. 5 characters)';
			$code = 1305;
		} elseif (!empty($actionConfig['passregexp']) && !preg_match($actionConfig['passregexp'], $password1)) {
			$exceptionString = 'invalid characters in password';
			$code = 1306;
		}
		if (empty($code)) {

			foreach ($list as $useruri => $value) {
				if ($value['userName'] == $username) {
					$exceptionString = 'user already registered';
					$code = 1300;
				}
					
				if (substr($value['userEmail'],7) == $email) {
					$exceptionString = 'email address already registered';
					$code = 1301;
				}
			}
				
		}

		if (empty($code) && empty($exceptionString)) {

			$userUri = $this->_acModel->modelURI.$username;

			$emailLiteral = new Erfurt_Rdfs_Literal_Default('mailto:'.$email);
			$usernameLiteral = new Erfurt_Rdfs_Literal_Default($username);
			$pwHashLiteral = new Erfurt_Rdfs_Literal_Default(sha1($password1));

			$this->_acModel->add($userUri, EF_RDF_TYPE, $config->ac->user->class);
			$this->_acModel->add($userUri, $config->ac->user->name, $usernameLiteral);
			$this->_acModel->add($userUri, $config->ac->user->mail, $emailLiteral);
			$this->_acModel->add($userUri, $config->ac->user->pass, $pwHashLiteral);

			if (isset($actionConfig['defaultGroup'])) {
				$this->_acModel->add($actionConfig['defaultGroup'], $config->ac->group->membership, $userUri);
			}
				
			// Some very (!!) old powl stuff. what does it exactly do? is it still needed?
			//$activationCode=md5(uniqid(rand(),true));
			//$powl->dbConn->execute("INSERT users VALUES ('{$_POST[username]}','{$_POST[email]}','".md5($_POST['password'])."','$activationCode')");
			//mail($_POST[email],'OntoWiki activation',"Please follow the following link to activate your account:\n\n{$_OWIKI['urlBase']}{$_OWIKI['uriBase']}/register.php?username={$_POST[username]}&amp;activationCode=$activationCode");
			//echo('An activation key was sent to you by email!');

			return true;
		} else {
			throw new Erfurt_Exception($exceptionString,$code);
			return false;
		}
	}

	/**
	 *  Edit existing user on System-Ontology
	 */
	public function editUser($username, $email, $password1, $password2) {

	}
}
?>
