<?php

/**
  * Tries to fetch a HTTP authentication header from the current 
  * request and authenticates with credentials when found.
  *
  * @author Norman Heino <norman.heino@googlemail.com>
  * @version $Id$
  * @package OntoWiki
  * @subpackage	Controller
  */
class OntoWiki_Controller_Plugin_HttpAuth extends Zend_Controller_Plugin_Abstract {
	
	/**
	  * Overwrite preDispatch to get informed before any dispatching occurs.
	  */
	public function preDispatch(Zend_Controller_Request_Abstract $request) {
		// fetch credentials
		if ($creds = $this->_getAuthHeaderCredentials($this->getRequest())) {
			// authenticate
			$result = Zend_Registry::get('erfurt')->authenticate($creds[0], $creds[1]);
			
			if ($result->isValid()) {
				Zend_Registry::get('owLog')->info('User authenticated via HTTP: ' . $creds[0] . ', ' . $creds[1]);
			}
		}
	}
	
	/**
	  * Fetches auth credentials from the request header and returns them in an array.
	  *
	  * @return array
	  */
	private function _getAuthHeaderCredentials(Zend_Controller_Request_Http $request) {
		// TODO: exception codes
		$authHeader = $request->getHeader('Authorization');
		
		if (empty($authHeader)) {
			// throw new Erfurt_Exception('Empty auth header');
		}
		
		$auth  = substr($authHeader, strlen('Basic '));
		$auth  = base64_decode($auth);
		
		$creds = array_filter(explode(':', $auth));
		if (count($creds) != 2) {
			// throw new Erfurt_Exception('Empty auth header');
		}
		
		return $creds;
	}
}

?>
