<?php

/**
  * OntoWiki box view helper
  *
  * @author Norman Heino <norman.heino@gmail.com>
  * @version $Id$
  */

class OntoWiki_Helper_OntoWikiBox {

	protected $store = null;
	protected $model = null;
	protected $resource = null;
	protected $class = null;
	protected $session = null;
	protected $config = null;
	protected $ac = null;
	protected $eventDispatcher = null;
	protected $view = null;
	protected $strings = null;
	protected $request = null;

	private $_filters = null;
	private $_filtersCombined = false;

	public function __construct() {
		$this->store    = Zend_Registry::get('erfurt')->getStore();
		$this->session  = Zend_Registry::get('session');

		$this->model    = $this->store->getModel($this->session->activeModelUri);

		if (is_object($this->model)) {
		    if ($this->session->activeResourceUri === null) {
		        $this->resource = $this->model->resourceF($this->model->getModelURI());
		        //$this->resource = null;
		    } else {
		        $this->resource = $this->model->resourceF($this->session->activeResourceUri);
		    }
		    
		    if ($this->session->activeClassUri === null) {
		        $this->class = $this->model->classF($this->model->getModelURI());
		    } else {
		        $this->class = $this->model->classF($this->session->activeClassUri);
		    }
		}

		$this->ac       = Zend_Registry::get('ac');
		$this->strings  = Zend_Registry::get('strings');
		$this->config   = Zend_Registry::get('config');
		$this->request  = Zend_Controller_Front::getInstance()->getRequest();

		$this->eventDispatcher = Zend_Registry::get('ed');
	}

	public function setView(Zend_View_Interface $view) {
		$this->view = clone $view;
	}

	public function ontoWikiBox($type) {
		$args = func_get_args();
		array_shift($args);

		if (!is_string($type)) {
			throw new Exception('Unsupported box type.');
		}

		// function name
		$fnString = 'get' . ucfirst($type) . 'Box';

		/** Reflection (@see http://www.php.net/manual/en/language.oop5.reflection.php) */
		try {
			$method = new ReflectionMethod('OntoWiki_Helper_OntoWikiBox', $fnString);
		} catch (Exception $e) {
			throw new Exception('OntoWiki box ' . ucfirst($type) . ' not found.');
		}

		// get content from box
		$boxContent = $method->invokeArgs($this, $args);

		// pass it the event dispatcher with triggers
		$this->eventDispatcher->trigger('create_sidecontent', &$boxContent);
		$this->eventDispatcher->trigger('create_sidecontent_' . ucfirst($type) . 'Box', &$boxContent);

		// return content
		return $boxContent;
	}

	/**
	  * Renders and returns the OntoWiki application box.
	  *
	  * @return string
	  */
	public function getApplicationBox() {
		$viewableModels = $this->store->countAvailableModels();

		if ($viewableModels) {
			$content  = '<form method="get" action="' . $this->config->uriBase . 'wiki/search/">' . PHP_EOL;
			if (null !== $this->model) {
				$content .= '<p><input type="hidden" name="m" value="' . $this->model->modelURI . '" /></p>';
			}
			$content .= '<p>
			             <label class="display-block" for="searchtext-input">' . $this->strings->wiki->search->box->label . '</label> ' . PHP_EOL .
			            '<input class="text width95 inner-label" type="text" id="searchtext-input" name="s" value="' . $_REQUEST['s'] . '" />
			             </p>' . PHP_EOL;

			if (null === $this->model) {
				$content .= '<p><input type="hidden" name="allModels" id="allModels" value="on" /></p> ' .PHP_EOL;
			} else {
				$content .= '<p><label class="display-block"><input class="checkbox" type="checkbox" name="allModels" id="allModels"
' .
				            ( ( isset($this->request->allModels) || !$this->config->activeModel ) ? 'checked' : '') . '/> ' . $this->strings->wiki->search->box->allbases . '</label>' . PHP_EOL;
			}

			$content .= '<button type="submit">' . $this->strings->wiki->search->box->submit . '</button></p>'
. PHP_EOL;
			$content .= '</form>' . PHP_EOL;

			$this->eventDispatcher->trigger('exampletrigger',&$content);
		}

		// user sub menu
		if ($this->ac->isActionAllowed('RegisterNewUser')) {
			$userMenu[] = '<a href="' . $this->config->uriBase . 'wiki/registernew/">Register New User</a>';
		}

		// get user
		$auth = Zend_Auth::getInstance();
		if ($auth->hasIdentity()) {
			$usr = $auth->getIdentity();
		}
		// show reg edit and logot links not for anonymous users
		if ($auth->hasIdentity() && !$usr['anonymous'] ) {
			$userMenu[] = '<a href="' . $this->config->uriBase . 'wiki/registeredit/">Preferences</a>';
			$userMenu[] = '<a href="' . $this->config->uriBase . 'wiki/logout/">Logout</a>';

			$viewSub = '<span class="window-view-menu">View</span>';
		}

		if (!empty($userMenu)) {
			$userSub = 'User' . $this->view->makeList($userMenu, 'ul');
		}

		//extras sub menu
		if ($viewableModels > 0) {
			$extras = array(
				'<a href="' . $this->config->uriBase . 'resource/query">SPARQL Query Editor</a>',
				'<a class="inline" href="' . $this->config->uriBase . 'service/getTemplate/?f=micro&amp;t=news">News</a>',
				'<a href="' . $this->config->uriBase . 'model/filemanager">File Manager</a>'
				// '<a class="inline" href="' . $this->config->uriBase . 'service/getTemplate/?f=micro&amp;t=query">SPARQL Query Editor</a>'
			);
			
			// trigger extras event
			$this->eventDispatcher->trigger('onCreateExtrasMenu', &$extras);
    		$extrasSub = 'Extras' . $this->view->makeList($extras, 'ul');
		}
		
		// help sub menu
		$helpSub = 'Help' . $this->view->makeList(array(
			'<a href="http://ontowiki.net/Projects/OntoWiki/Features">Documentation</a>',
			'<a title="SF.net Bugreport page" href="http://sourceforge.net/tracker/?func=add&amp;group_id=99425&amp;atid=624170">Found a Bug?</a>',
			'<a href="http://ontowiki.net/Projects/OntoWiki/ChangeLog#' . $this->config->version->number . '">Version Info ' . $this->config->version->number . '</a>',
			'<a href="http://ontowiki.net/">About</a>'
		), 'ul');

		// get username
		$auth = Zend_Auth::getInstance();
		if ($auth->hasIdentity()) {
			$usr = $auth->getIdentity();
			if (!$usr['anonymous']) {
				$usrString = '(' . $usr['username'] . ')';
			}
		}

		// construct return array with template variables
		$retArray = array(
			'cssId'      => 'search',
			'cssClasses' => ($this->session->searchBoxMinimized ? 'is-minimized' : '') . ' no-disable',
			'title'      => $this->strings->wiki->search->box->title . ' ' . $usrString,
			'content'    => $content,
			'menu'       => array($userSub, $viewSub, $extrasSub, $helpSub)
		);

		return $retArray;
	}

	public function getLanguagesBox() {
		$languages = array();
		$query     = $this->request->getQuery();
		$noLang    = 'none';

		if (!$this->model) {
			return array();
		}

		// get literal languages used from model
		$langs = $this->model->listLanguages();

		// we dont want an empty language box!
		if (count($langs) == 0) {
			return;
		}

		// add empty for no language
		$langs[] = $noLang;

		foreach ($langs as $lang) {
			$languages[] = '<a ' . ($this->session->modelLanguage == $lang ? 'class="selected" ' : '') . 'href="' .
			               Erfurt_Util::replaceUrlParam('lang', urlencode($lang)) . '">' .
			               ($lang != $noLang ? $lang : $this->strings->default->languages->nolang) . '</a>';
		}

		$returnArray = array(
			'cssId'      => 'languages',
			'cssClasses' => ($this->session->languagesBoxMinimized ? ' is-minimized' : '') .
			                ($this->session->languagesBoxDisabled ? ' is-disabled' : ''),
			'title'      => $this->strings->default->languages->title,
			'content'    => $this->view->makeList($languages, 'ol', 'bullets-none separated-horizontal')
		);

		return $returnArray;
	}

	public function getModelsBox() {
		if ($this->model) {
			$activeModelUri = $this->model->modelURI;
		}

		// fill array with available models
		$models    = array();
		$modelList = $this->store->listModels(false);
		$modelManagementAllowed = $this->ac->isActionAllowed('ModelManagement');

		// no permitted model
		if (count($modelList) == 0 && !$modelManagementAllowed) {
			return false;
		}

		// set model list
		foreach ($modelList as $uri=>$m) {
			if ($activeModelUri == $uri) {
				$cssClasses = ' class="Model selected"';
			} else {
				$cssClasses = ' class="Model"';
			}
			//$models[] = '<a href="' . $this->config->uriBase . 'model/select/?m=' .
			//		urlencode($m['modelURI']) . '"' . $cssClasses . ' about="' . $m['modelURI'] . '">' .
			//		(isset($m['label']) ? OntoWiki_Util::shorten($m['label'], 34) : OntoWiki_Util::shorten($m['modelURI'], 34)) . '</a>';

			$models[] = '<a href="' . $this->config->uriBase . 'model/select/?m=' .
					urlencode($uri) . '"' . $cssClasses . ' about="' . $uri . '">' .
					$m->getAsResource()->getTitle($this->session->modelLanguage, true, 34) . '</a>';
		}

		if ($this->ac->isActionAllowed('ModelManagement')) {
			$modelMenu[] = '<a href="' . $this->config->uriBase . 'model/create/">Create Knowledge Base</a>';
		}

		$modelMenu[] = '<a href="#"><del>Show Hidden Knowledge Bases</del></a>';

		$contextMenu = array(
			$this->view->makeList($modelMenu, 'ul'),
			'<hr />',
			$this->view->makeList(array(
				'<a href="http://aksw.org/Projects/OntoWiki/Help/Models">Help</a>'
			), 'ul')
		);

		$returnArray = array(
			'cssId'       => 'models',
			'cssClasses'  => 'has-contextmenus-block' . ($this->session->modelsBoxMinimized ? ' is-minimized' : '') .
			                                            ($this->session->modelsBoxDisabled ? ' is-disabled' : ''),
			'title'       => $this->strings->default->models->title,
			'content'     => $this->view->makeList($models, 'ol', 'bullets-none separated-vertical'),
			'contextMenu' => $contextMenu
		);

		return $returnArray;
	}

	public function getClassesBox() {

		if (!$this->session->classesBoxDisabled) {
			if ($this->request->hC && $this->request->hC == 'true') {
			  	$this->session->hiddenClasses = true;
			} elseif (($this->request->hC && $this->request->hC == 'false') || !isset($this->session->hiddenClasses)) {
				$this->session->hiddenClasses = false;
			} else {
				// leave it alone
			}

			if ($this->request->eC && $this->request->eC == 'true') {

				$this->session->emptyClasses = true;
			} elseif (($this->request->eC && $this->request->eC == 'false') || !isset($this->session->emptyClasses)) {
				$this->session->emptyClasses = false;
			} else {
				// leave it alone
			}

			if ($this->request->sC && $this->request->sC == 'true') {
				$this->session->systemClasses = true;
			} elseif (($this->request->sC && $this->request->sC == 'false') || !isset($this->session->systemClasses)) {
				$this->session->systemClasses = false;
			} else {
				// leave it alone
			}

			if (!$this->model) return array();

			$c = new stmCache(
				'showClasses',
				array(
					$this->session->hiddenClasses,
					$this->session->emptyClasses,
					$this->session->systemClasses,
					$this->config->completeClassTree,
					$this->session->modelLanguage),
				$this->model
			);

			$cVal = $c->get();

			if (null === $cVal) {
				 //echo 'cache was empty ';
				$cVal = $this->_showClasses(
					$this->model->listTopClasses(
					$this->session->systemClasses, $this->session->emptyClasses, true, $this->session->hiddenClasses),
					$this->session->modelLanguage);
				$c->set($cVal);
			}

			$classes .= $cVal;

			if ($this->model instanceof Erfurt_Owl_Model) {
				$clsType = 'owl%3AClass';
			} elseif ($this->model instanceof RDFSModel) {
				$clsType = 'rdfs%3AClass';
			}
		}

		$contextMenu = array(
			$this->view->makeList(
				array(
					'<a href="' . $this->config->uriBase . 'resource/create/?prop%5Brdf%3Atype%5D=' . $clsType . '">Create Class</a>',
					'<a href="' . Erfurt_Util::replaceUrlParam('hC', $this->session->hiddenClasses ? 'false' : 'true') . '">' .
						($this->session->hiddenClasses ? $this->strings->default->classes->hidehidden : $this->strings->default->classes->showhidden) . '</a>',
					'<a href="' . Erfurt_Util::replaceUrlParam('eC', $this->session->emptyClasses ? 'false' : 'true') . '">' .
						($this->session->emptyClasses ? $this->strings->default->classes->hideempty : $this->strings->default->classes->showempty) . '</a>',
					'<a href="'. Erfurt_Util::replaceUrlParam('sC', $this->session->systemClasses ? 'false' : 'true') . '">' .
						($this->session->systemClasses ? $this->strings->default->classes->hidesystem : $this->strings->default->classes->showsystem) . '</a>'
				), 'ul'),
			$this->view->makeList(array(
				'<a href="http://aksw.org/Projects/OntoWiki/Help/Classes">Help</a>'),
				'ul'
			));

		$cssId       =
		$cssClasses  = 'has-contextmenus-block' . ($this->session->classesBoxMinimized ? ' is-minimized' : '') .
		                           ($this->session->classesBoxDisabled ? ' is-disabled' : '');
		$title       = $this->strings->default->classes->title;

		// show classes box even if empty
		// if ($classes) {
			$content     = '<ol class="bullets-none separated-vertical">' . PHP_EOL . $classes . PHP_EOL . '</ol>' . PHP_EOL;
		// }
		$contextMenu = $contextMenu;

		$returnArray = array(
			'cssId'       => 'classes',
			'cssClasses'  => 'has-contextmenus-block' .
			                ($this->session->classesBoxMinimized ? ' is-minimized' : '') .
			                ($this->session->classesBoxDisabled ? ' is-disabled' : ''),
			'title'       => $this->strings->default->classes->title,
			'content'     => $content,
			'contextMenu' => $contextMenu
		);

		return $returnArray;
	}

	/**
	  * Returns a string with classes in the current model.
	  *
	  * @param $classes An array of classes to be shown.
	  */
	public function _showClasses($classes, $lang) {
		foreach ($classes as $row) {
			if ($row instanceof Resource) {
				$count = $row->countInstancesRecursive();

/*				if ($this->class instanceof RDFSClass && $this->resource->getUri() == $row->getUri()) {
					$classes = 'Class selected';
				} else {
					$classes = 'Class';
				}
*/
				$classes = 'Class';

				if ($row->isHidden()) {
					$classes .= ' hidden';
				}

				if ($row->isImplicit()) {
					$classes .= ' implicit';
				}

				if ($row->isSystem()) {
					$classes .= ' system';
				}

				if ($row->isEmpty()) {
					$classes .= ' empty';
				}

				if ($row->getClassType() === 'rdfs:Class') {
					$classes .= ' rdfs:Class';
				} else if ($row->getClassType() === 'owl:Class') {
					$classes .= ' owl:Class';
				} else if ($row->getClassType() === 'owl:DeprecatedClass') {
					$classes .= ' owl:DeprecatedClass';
				}

				$ret .= "\t" . '<li><a class="' .  $classes . '" about="' . $row->getUri() . '" href="' . OntoWiki_Util::getURL($row, null, 'list') . '">' .
						$row->getTitle($lang) . ($count ? ' (' . $count . ')' : '') . '</a>';

				$sub = $this->_showClasses($row->listDirectSubClasses($this->session->emptyClasses, $this->session->hiddenClasses), $lang);

				if ($sub && $this->config->completeClassTree) {
					$ret .= PHP_EOL . "\t" . '<ol class="bullets-none">' . PHP_EOL . $sub . '</ol>';
				}

				$ret .= '</li>' . PHP_EOL;
			}
		}
		return $ret;
	}

	/**
	 * Returns a box with tags found the current resource's class' instances.
	 *
	 * @param array an array of configuration parameters. The following parameters are possible:
	 *              - tag_property:     the property that associates resources with tags
	 *              - label_property:   the property for the tag's label
	 *              - min_size:         minimum text size (factor, default = 0.75)
	 *              - max_size:         size of the tag with highest frequency (factor, default = 1.25)
	 *              - non_linear_scale: exponent for the tag scaling function (default = 0.5 -- root)
	 *
	 * @return string
	 */
	public function getTagsBox($args = null) {
		// no conig given
		if (!is_array($args) || !isset($args['tag_property']) || !isset($args['label_property'])) {
			$returnArray = array(
				'cssId'       => 'tags',
				'cssClasses'  => 'tags',
				'title'       => 'Tags',
				'content'     => '<div class="messagebox error" style="height:2em">Wrong Configuration</div>',
			);

			return $returnArray;
		}

		// fetch parameters
		$tagProperty      = $args['tag_property'];
		$tagLabelProperty = $args['label_property'];
		$min              = $args['min_size'] ? $args['min_size'] : 0.6;
		$max              = $args['max_size'] ? $args['max_size'] : 1.25;
		$nonLinScale      = $args['non_linear_scale'] ? $args['non_linear_scale'] : 0.5;
		$jsMode           = $args['javascript_mode'] ? $args['javascript_mode'] : false;

		// find tags on instances
		// $sparql = '
		// 	SELECT DISTINCT ?tagUri, ?tagName WHERE {
		// 		?instance <' . EF_RDF_TYPE . '> <' . $this->class->getURI() . '>.
		// 		?instance <' . $tagProperty . '> ?tagUri.
		// 		OPTIONAL {
		// 			?tagUri <' . $tagLabelProperty . '> ?tagName.
		// 		}
		// 	}';

		// get data
		// $data = $this->model->sparqlQuery($sparql, $renderer = new Erfurt_Sparql_ResultRenderer_Plain());

		// sql needed since we cannot efficiently count in sparql atm :(
		$sql = '
			SELECT s1.object as tagUri, COUNT(s1.object) as tagCount, s2.object as tagName
				FROM statements s1, statements s2
				WHERE s1.predicate = "' . $tagProperty . '"
				AND s2.predicate = "' . $tagLabelProperty . '"
				AND s2.object_is = "l"
				AND s1.object = s2.subject
				AND s1.modelID IN (' . $this->model->getModelIds() . ')
				GROUP BY s1.object
				ORDER BY tagCount DESC
				LIMIT 50;
		';

		// get ade result set to be used with cursor
		$sqlResult = $this->model->dbConn->Execute($sql);

		if (!$sqlResult) {
			// empty resultset
			$returnArray = array(
				'cssId'      => 'tags',
				'title'      => 'Tags',
				'content'    => '<div class="messagebox info">'.$this->strings->default->noresults.'</div>',
			);

			return $returnArray;

		} else {
			
			$tags      = array();
			$firstRow  = $sqlResult->getRowAssoc();
			$maxCount  = (int) $firstRow['TAGCOUNT'];
			$className = $this->class->getQualifiedName();
			$uriBase   = $this->config->uriBase;

			while ($tagData = $sqlResult->FetchRow()) { // 0 => uri, 1 => count, 2 => name
				if (is_array($_REQUEST['select']) and in_array($tagData[0], $_REQUEST['select'])) {
					$classes = 'selected';
					$selected = true;
				} else {
					$classes = '';
					$selected = false;
				}

				// replace missing names by URIs
				if (isset($tagData[2])) {
					$tagName = $tagData[2];
				} else {
					$tagName = $tagData[0];
				}

				// calculate scale
				$plainScale   = pow((int) $tagData[1] / $maxCount, $nonLinScale);
				$opacityScale = $plainScale * (1 - $min) + $min;
				$sizeScale    = ($plainScale * ($max - $min) + $min) * 100;

				// highlight selected tag
				if ($selected) {
					// remove filter
					$filter = '';
				} else {
					// set filter
					$filter = sprintf('?select[%s]=%s', urlencode($tagProperty), urlencode($tagData[0]));
				}

				if ($jsMode) {
					// construct tag link
					$tags[] = sprintf('<a class="tag javascript-on %s" about="%s" style="font-size:%d%%;opacity:%f" title="%d">%s<span class="onlyAural">(%d)</span></a>',
					                  $classes,
					                  $tagData[0],
					                  $sizeScale,
					                  $opacityScale,
					                  $tagData[1],
					                  $tagName,
					                  $tagData[1]);
				} else {
					// construct tag link
					$tags[] = sprintf('<a class="tag %s" style="font-size:%d%%;opacity:%f" href="%sresource/list/%s%s" title="%d">%s<span class="onlyAural">(%d)</span></a>',
					                  $classes,
					                  $sizeScale,
					                  $opacityScale,
					                  $uriBase,
					                  urlencode($className),
					                  $filter,
					                  $tagData[1],
					                  $tagName,
					                  $tagData[1]);
				}
			}
		}

		$content .= '<script type="text/javascript">//<!--
			var tagParams = {
				tagProperty:   \'' . $tagProperty . '\',
				labelProperty: \'' . $tagLabelProperty . '\'
			};
		//--></script>' . PHP_EOL;

		if (!$jsMode) {
			$content .= $this->view->makeList($tags, 'ul', 'bullets-none inline');
		}
		
		$strings = Zend_Registry::get('strings');
		
		$returnArray = array(
			'cssId'   => 'tags',
			'title'   => $strings->sw->tags/*'Tags'*/,
			'content' => $content
		);

		return $returnArray;
	}


	public function getHierarchyBox($args = null) {
		if (!is_array($args) || !isset($args['object_type']) || !isset($args['sub_relation'])) {
		$returnArray = array(
			'cssId'       => 'hierarchy-' . $id,
			'cssClasses'  => 'hierarchy',
			'title'       => 'Error',
			'content'     => '<div class="messagebox error">Wrong Parameter</div>',
			'contextMenu' => 'ttt'
		);
		return $returnArray;
		}

		if (!$this->model) {
			return false;
		}

		// fetch configuration
		$objectType       = $args['object_type'];
		$subRelation      = $args['sub_relation'];
		$instanceRelation = isset($args['instance_type'])   ? $args['instance_type']   : null;
		$jsMode           = isset($args['javascript_mode']) ? $args['javascript_mode'] : false;
		$withRoot         = isset($args['with_root']) ? $args['with_root'] : false;
        $rootElement      = isset($args['root_element']) ? $args['root_element'] : $this->resource->getUri();

		// check whether counting is supported by store
		if ($this->model->getStore() instanceof Erfurt_Store_CountableInterface) {
			$doCount = true;
		} else {
			$doCount = false;
		}

		$id = uniqid();

		$content .= '<script type="text/javascript">//<!--
			var hierarchyParams = {
				nt: \'' . $objectType . '\',
				sr: \'' . $subRelation . '\',
				ir: \'' . $instanceRelation . '\'
			};
			var selectedResource = \'' . $this->resource->getUri() . '\';
		//--></script>' . PHP_EOL;

		$resTree = $this->model->resourceTree($objectType, $subRelation, $instanceRelation, array('doCount' => $doCount));

		// var_dump($resTree);
		// exit;



		$content .= '<div class="display-block">
					<label class="display-block" for="hierarchy-' . $id . '-search-input">'.$this->strings->default->hierarchy->search .' ...</label>
					<input type="text" class="width90 text inner-label hierarchy-search-input" id="hierarchy-' . $id . '-search-input" />	    
					</div>';
		
		if ($withRoot) {
		    $content .= '<a class="resource-select root" about="' 
		              . $rootElement . '">' 
		              . $this->strings->default->hierarchy->all . '</a>';
		}
		$content .= '<ol class="bullets-none separated-vertical"></ol>' . PHP_EOL;
		// $content .= '<ol class="bullets-none separated-vertical">' . PHP_EOL . $this->_showHierarchy($resTree, $lang) . PHP_EOL . '</ol>' . PHP_EOL;

		$returnArray = array(
			'cssId'       => 'hierarchy-' . $id,
			'cssClasses'  => 'hierarchy' . ($jsMode ? ' javascript-on' : ''),
			'title'       => $this->strings->default->hierarchy->title,
			'content'     => $content,
		);

		return $returnArray;
	}

	/**
	 * Returns a list of most oftenly viewed resource.
	 *
	 * @param array an array of configuration parameters. The following parameters are possible:
	 *              - tag_property:     the property that associates resources with tags
	 *              - label_property:   the property for the tag's label
	 *              - min_size:         minimum text size (factor, default = 0.75)
	 *              - max_size:         size of the tag with highest frequency (factor, default = 1.25)
	 *              - non_linear_scale: exponent for the tag scaling function (default = 0.5 -- root)
	 *
	 * @return string
	 */
	public function getPopularsBox() {
		$populars = array();

		// model ac
		$modelWhere = '';
		$modelFrom = '';
		if ($this->model) {
			 $modelWhere = ' WHERE modelID = "' . $this->model->modelID . '" ';
		} else {
			if ($this->ac->isAnyModelAllowed('view') and count($this->ac->getDeniedModels('view')) > 0) {
				$modelFrom = ', models as m';
				$modelWhere = ' WHERE m.modelID = popularity.modelID AND modelID NOT IN (\''.implode("','", $this->ac->getDeniedModels('view')).'\')';
			} else if ($this->ac->getAllowedModels('view') !== false) {
				$modelFrom = ', models as m';
				$modelWhere = ' WHERE m.modelID = popularity.modelID AND modelID IN (\''.implode("','", $this->ac->getAllowedModels('view')).'\')';
			}
		}

		$pop = $this->store->dbConn->getAll(
			'SELECT count(*) c, uri, modelID
			 FROM popularity ' . $modelFrom . $modelWhere . '
		     GROUP BY uri
		     ORDER BY c DESC LIMIT 10');

		// no entries
		if (!$pop or count($pop) == 0) {
			return array();
		}

		if ($pop) {
			foreach($pop as $row) {
				$m = $this->model ? $this->model : $this->store->getModel($row[2]);
					$populars[] = '<a about="' . $row[1] . '" href="' . OntoWiki_Util::getURL($m->resourceF($row[1])) . '">' .
												$row[0] . ' &ndash; ' .
												$m->resourceF($row[1], false)->getTitle($this->session->modelLanguage,
													 true, 30) . '</a>';
			}
		}

		$returnArray = array(
			'cssId'      => 'popularity',
			'cssClasses' => 'has-contextmenus-block' .
			                ($this->session->popularityBoxMinimized ? ' is-minimized' : '') .
			                ($this->session->popularityBoxDisabled ? ' is-disabled' : ''),
			'title'      => $this->strings->default->populars->title,
			'content'    => $this->view->makeList($populars, 'ol', 'bullets-none separated-vertical')
		);

		return $returnArray;
	}

	public function getLoginBox() {
		$content     = '';
		$returnArray = array();

		try {
			// $result = Zend_Registry::get('authResult');
			if (isset($this->session->authResult)) {
				foreach($this->session->authResult as $message) {
     		   		$content .= '<p class="messagebox error">' . $message . '</p>' . PHP_EOL;
				}
				unset($this->session->authResult);
			}
			/* check, if cookies are enabled. session_trans_sid, doesn't work, because php misses 
			to add PHPSESSID to the hidden redirect variable, in the hidden field of the login box,
			So I decided to make cookies mandatory for ontowiki. this fixes ISSUE 35. 
			Julian */
			/* In order to avoid this message in case cookies are enabled and page is loaded for the first time, we do this via js... */
			//if(empty($_COOKIE)) {
			//	$content .= '<p class="messagebox info">Cookies must be enabled for login!</p>' . PHP_EOL;
			//}
		} catch (Exception $e) {
			// ... egal
		}

		$auth     = Zend_Auth::getInstance();
		$identity = $auth->getIdentity();

		if ($identity['anonymous']) {
			$content .= '<form method="post" action="' . $this->config->uriBase . 'wiki/login/" class="row-input input-justify-left"><fieldset>' . PHP_EOL;
			
			if (trim($this->config->uriBase, ' /') != '') {
			    $redirect = str_replace($this->config->uriBase, '', $this->request->getServer('REQUEST_URI'));
			} else {
			    $redirect = $this->request->getServer('REQUEST_URI');
			}
			$redirect = '/' . ltrim($redirect, '/');
			
			$content .= '<input type="hidden" name="redirect" value="' . urlencode($redirect) . '" />' . PHP_EOL;
			$content .= '<a name="login" />' . PHP_EOL;
			$content .= '<div><label for="username">' . $this->strings->auth->login->username . '</label> ' .
					'<input class="text" type="text" id="username" name="username" value="' .
					(empty($_REQUEST['username']) ? '' : $_REQUEST['username']) . '" /><br class="clearall" /></div>' . PHP_EOL .
					'<div><label for="password">' . $this->strings->auth->login->password .  '</label> ' .
					'<input class="password" type="password" id="password" name="password" /><br class="clearall" /></div>' . PHP_EOL .
					'<div><input type="checkbox" id="rememberme" name="rememberme" />' .
					$this->strings->auth->login->remember . '<br class="clearall" /></div>' . PHP_EOL .
					'<div class="actionbuttons"><button type="submit">' . $this->strings->auth->login->submit . '</button></div>' . PHP_EOL .
					'<a href="'. $this->config->uriBase . 'wiki/recoverpassword" >' . $this->strings->auth->login->forgotpassword . 
					'</a><br/>' . PHP_EOL .
					'<a href="'. $this->config->uriBase . 'wiki/registernew" >' . $this->strings->auth->login->register . 
					'</a>' . PHP_EOL .
					'</fieldset>' . PHP_EOL .
					'</form>' . PHP_EOL;
			$returnArray = array(
				'cssId'      => 'login',
				'cssClasses' => ($this->session->loginBoxMinimized ? ' is-minimized' : '') .
				                ($this->session->loginBoxDisabled ? ' is-disabled' : ''),
				'title'      => $this->strings->auth->login->login,
				'content'    => $content
			);
		}

		return $returnArray;
	}

	///////////////////////////////////////////////////////////////////////////
	//                                                                       //
	//                            Class Boxes                                //
	//                                                                       //
	///////////////////////////////////////////////////////////////////////////

	/**
	  * Predicates box
	  */
	public function getPredicatesBox() {
		$empty = true;

		if (!$this->session->predicatesBoxDisabled) {
			// find predicates
			$preds = $this->model->findPredicates($this->class, null);
			$numPreds = count($preds);
			$predicates = array();
			$values = array();
			$count = 1;

			foreach ($preds as $p) {
				if ($p->getQualifiedName() != 'rdf:type') {
					$empty = false;
					//$predicatesHtml = OntoWiki_Util::getTitle($p);
					$predicatesHtml = $p->getTitle();

					// find all statements
					foreach ($this->model->findStatements($this->class, $p, null) as $s) {
						// if it's a resource
						if (is_a($s->obj, 'resource') && !($s->obj instanceof Blanknode)) {
							// print its title
							$values[] = '<a about="' . $s->obj->getUri() . '" href="' . OntoWiki_Util::getURL($s->obj) . '">' . $s->obj->getTitle() . '</a>';
							// $predicatesHtml->addString(OntoWiki_Util::getTitle($s->obj));
						}
						// if it's a literal
						elseif (is_a($s->obj, 'literal')) {
							// if it links to an image
							if (in_array($p->getQualifiedName(), $this->config->imgLinkProperties->toArray())) {
								// draw that image
								// :TODO: meaningfull alt tag
								$predicatesHtml .= '<img src="' . $s->obj->getLabel() . '" alt="img" />' . PHP_EOL;
							// if it's an uri
							} elseif (in_array($p->getQualifiedName(), $this->config->webLinkProperties->toArray()) ||
								   							 strpos($s->obj->getLabel(), 'http://') === 0) {
								// print it as link
								$predicatesHtml .= '<a href="' . $s->obj->getLabel() . '">' . $s->obj->getLabel() . '</a>' . PHP_EOL;
							} else {
								// else print it directly
								$values[] = $s->obj->getLabel();
								// $predicatesHtml->addString($s->obj->getLabel() . PHP_EOL);
							}
						}
					}
					if (!empty($values)) {
						$predicatesHtml .= $this->view->makeList($values, 'ul', 'bullets-none separated-horizontal');
						$values = array();
					}

					$predicates[] = $predicatesHtml;
				}
			}

			if (!$empty) {
				$content = $this->view->makeList($predicates, 'ul', 'bullets-none separated-vertical');
			} else {
				$content = '<p class="messagebox info">' . $this->strings->default->noresults . '</p>';
			}
		}

		$returnArray = array(
			'cssId'        => 'predicates',
			'cssClasses'   => ($this->session->predicatesBoxMinimized ? 'is-minimized' : '') .
			                  ($this->session->predicatesBoxDisabled ? ' is-disabled' : ''),
			'title'        => $this->class->getTitle(),
			'headingLevel' => 3,
			'content'      => $content
		);

		return $returnArray;
	}

	/**
	  * Returns the Properties box as a string.
	  *
	  * @return string
	  */
	public function getPropertiesBox() {
		$empty = true;

		if (!$this->session->propertiesBoxDisabled) {
			if ($this->class) {
				$props = $this->class->listPropertiesUsedRecursive();
			} elseif ($this->model) {
				$props = $this->model->listResourcesPredicate();
			} else {
				$props = array();
			}

			if (isset($this->session->show)) {
				$show = $this->session->show[$this->resource->getQualifiedName()];
			}

			if (!is_array($show)) {
				$show = array();
			}

			foreach ($props as $prop) {
				$empty = false;
				$properties[] = '<a href="' . pwlURLParamReplace(array('show[]', 'showRemove[]'), '') .
						(in_array($prop->getQualifiedName(), $show) ? 'showRemove[]=' . urlencode($prop->getQualifiedName()) .
						'" class="selected' : 'show[]=' . urlencode($prop->getQualifiedName())) . '"> ' .
						$prop->getTitle() . '</a>';
			}

			if (!$empty && is_array($properties)) {
				$content = $this->view->makeList($properties, 'ul', 'bullets-none separated-horizontal');
			} else {
				$content = '<p class="messagebox info">' . $this->strings->default->noresults . '</p>';
			}
		}

		$returnArray = array(
			'cssId'        => 'properties',
			'cssClasses'   => ($this->session->propertiesBoxMinimized ? 'is-minimized' : '') .
			                      ($this->session->propertiesBoxDisabled ? ' is-disabled' : ''),
			'title'        => $this->strings->class->properties->title,
			'headingLevel' => 3,
			'content'      => $content
		);

		return $returnArray;
	}

	/*
	 * Filter box helper function
	 */
	public function _showFilterForProperty($property, $class) {
		$args = func_get_args();
		$cache = new stmCache('showFilterForProperty', $args, $property->model);
		
		$cVal = $cache->get();

		if ($cVal !== null) {
			return $cVal;
		}

		if (!($property instanceof Erfurt_Rdfs_Property)) {
			$property = $this->model->propertyF($property);
		}
		
		$valueCount = $property->countDistinctPropertyValues($class, false);
		
		$ratio = $class ?
			$class->countInstancePropertyValues($property, false, 2) :
			(($valueCount > 0) ? ($this->model->countInstances() / $valueCount) : 0);
		
		if ($valueCount && $ratio >= 2 && $valueCount < 100) {
			$values = array();
			
			foreach ($property->listDistinctPropertyValues($class, false) as $key => $val) {
				if (is_string($val)) {
					$values[$val] = $val;
				} else if ($val instanceof Literal) {
				    $literal = $val->getLabel();
				    $values[$literal] = $literal;
				} else {
					$values[$val->getLabel()] = $this->model->resourceF($val, false)->getTitle();
				}
			}
			asort($values);

			// hack to clear empty values
			// TODO: find out why there are sometimes empty values
			if (isset($values[''])) {
				unset($values['']);
				$valueCount--;
			}
			$ret = array_merge(array('' => urldecode($property->getTitle()) . ' (' . $valueCount . ')'), $values);
		} else {
			$ret = '';
		}

		$cache->set($ret, $property);
		return $cache->value;
	}

	/**
	  * Returns the Filter box as a string.
	  *
	  * @return string
	  */
	public function getFilterBox() {
		$empty = true;

		if (!$this->session->filterBoxDisabled) {
			$filterHtml = '<form method="get">' . PHP_EOL;

			// TODO: maybe retain other parameters when filtering
			if ($this->request->getQuery('r')) {
				$filterHtml .= '<input type="hidden" name="r" value="' . $this->request->r . '" />' . PHP_EOL;
			}

			// TODO: do this only once
			if ($this->class) {
				$props = $this->class->listPropertiesUsedRecursive();
			} elseif ($this->model) {
				$props = $this->model->listResourcesPredicate();
			} else {
				$props = array();
			}

			if (!$this->filters) {
				$this->_fetchFilters();
			}
			
			foreach ($props as $prop) {
				$name = 'select[' . $prop->getQualifiedName() . ']';
                $values = $this->_showFilterForProperty($prop, $this->class);
				
				if ($values) {
					$empty = false;
					$filterHtml .= ('<div class="no-min-width">' . new SelectNew(
						$name,
						$this->_filters[$prop->getQualifiedName()],
						$values,
						array(
							'class' => 'width99',
	  				        'cardinality' => 1,
	  				        'onChange' => 'subm(this.form,this.form.elements[\'' . $name . '\']);'
						)
					) . '</div>' . PHP_EOL);
				}
			}
			
			// shall filters be combined?
			$combine = isset($this->request->combine) ? true : false;
			$filterHtml .= '<hr />' . '<div><nobr><input type="checkbox" name="combine" id="combine"' .
				($combine ? 'checked ' : '') .
				' />&nbsp;<label for="combine">' . $this->strings->class->filter->combine . '</label></nobr>' . PHP_EOL . '</div>' .
				'</form>' . PHP_EOL;

			if (!$empty) {
				$content = $filterHtml;
			} else {
				$content = '<p class="messagebox info">' . $this->strings->default->noresults . '</p>';
			}
		}

		$returnArray = array(
			'cssId' => 'filter',
			'cssClasses' => ($this->session->filterBoxMinimized ? 'is-minimized' : '') .
			                      ($this->session->filterBoxDisabled ? ' is-disabled' : ''),
			'title' => $this->strings->class->filter->title,
			'headingLevel' => 3,
			'content' => $content
		);

		return $returnArray;
	}

	/**
	  * Fetches filters from URI
	  */
	public function _fetchFilters() {
		// only old-school OntoWiki filters supported by now
		if (isset($this->request->select)) {
			$this->_filters = array_filter($this->request->select);
		} else {
			$this->_filters = array();
		}

		if (isset($this->request->combine) && $this->request->combine == 'on') {
			$this->_filtersCombined = true;
		} else {
			$this->_filtersCombined = false;
		}
	}


	///////////////////////////////////////////////////////////////////////////
	//                                                                       //
	//                            Instance Boxes                             //
	//                                                                       //
	///////////////////////////////////////////////////////////////////////////

	/**
	  * Returns the Similar Instances box as an array.
	  *
	  * @return string
	  */
	public function getSimilarInstancesBox() {
		$instances = array();
		$similars  = array();
		$empty     = true;

		if (!$this->session->similarsBoxDisabled) {
			// get all types for current instance
			$types = $this->resource->type();
			$left = count($types);
			foreach ($types as $type) {
				// find other nodes of same type
                // foreach ($type->listPropertyValuesObject(EF_RDF_TYPE, 'instance') as $s) {
				foreach ($this->model->findNodes(null, EF_RDF_TYPE, $type, 'instance', 0, 5, &$erg) as $s) {
					// exclude currrent instance
					if ($s->getURI() != $this->resource->getURI()) {
						$instances[] = '<a about="' . $s->getUri() . '" href="' . OntoWiki_Util::getURL($s) . '">' . $s->getTitle($this->session->modelLanguage). '</a>';
					}
				}

				if (count($instances) > 0) {
					$empty = false;
					$sim = $type->getTitle($this->session->modelLanguage) . $this->view->makeList($instances, 'ul', 'bullets-none separated-horizontal');

					// reset array
					$instances = array();

					if ($erg > 5) {
						$sim .= '<hr class="clearall" />';
						$sim .= '<a href="' . $this->config->uriBase . 'resource/list/' . urlencode($type->getQualifiedName()) . '">Show all ' .
							    $erg . ' in Main Tab&thinsp;&raquo;</a>' . PHP_EOL;
					}

					$similars[] = $sim;
				}
			}

			if (!$empty) {
				$content = $this->view->makeList($similars, 'ul', 'bullets-none separated-vertical', 'clearfloat');
			} else {
				$content = '<p class="messagebox info">' . $this->strings->default->noresults . '</p>';
			}
		}

		$returnArray = array(
			'cssId'        => 'similars',
			'cssClasses'   => ($this->session->similarsBoxMinimized ? 'is-minimized' : '') .
			                  ($this->session->similarsBoxDisabled ? ' is-disabled' : ''),
			'title'        => $this->strings->instance->similars->title,
			'headingLevel' => 3,
			'content'      => $content
		);

		return $returnArray;
	}

	/**
	  * Returns the Instances Linking Here box as an array.
	  *
	  * @return string
	  */
	public function getInstancesLinkingHereBox() {
		$instances = array();
		$lists     = array();
		$empty     = true;

		if (!$this->session->linkingBoxDisabled) {
            // foreach ($this->resource->listPropertiesUsedAsObject() as $p) {
            foreach ($this->model->findPredicates(null, $this->resource) as $p) {
                // foreach($this->resource->listPropertyValuesObject($p) as $s) {
				foreach ($this->model->findStatements(null, $p, $this->resource, 0, 5, &$erg) as $s) {
                    if ($s->subj instanceof Erfurt_Rdfs_Resource_Default) {
						$instances[] = '<a href="'
						             . OntoWiki_Util::getURL($s->subj) 
						             . '">' 
						             . $s->subj->getTitle($this->session->modelLanguage)
						             . '</a>';
                    }
				}

				$empty = false;
				$lists[] = $p->getTitle($this->session->modelLanguage) . $this->view->makeList($instances, 'ul', 'bullets-none separated-horizontal') .
						   '<hr class="clearall" />' .
				           // TODO: use common base class instead of owl:Thing
				           '<a href="' . $this->config->uriBase . 'resource/list/owl%3AThing?select%5B' . urlencode($p->getQualifiedName()) . '%5D=' .
				           urlencode($this->resource->getURI()) . '">Show' . ($erg > 1 ? (' ' . $erg) : '') . ' in Main Tab&thinsp;&raquo;</a>';

				$instances = array();
			}

			if (!$empty) {
				$content = $this->view->makeList($lists, 'ul', 'bullets-none separated-vertical');
			} else {
				$content = '<p class="messagebox info">' . $this->strings->default->noresults . '</p>';
			}
		}

		$returnArray = array(
			'cssId'        => 'linking',
			'cssClasses'   => ($this->session->linkingBoxMinimized ? 'is-minimized' : '') .
			                  ($this->session->linkingBoxDisabled ? ' is-disabled' : ''),
			'title'        => $this->strings->instance->linking->title,
			'headingLevel' => 3,
			'content'      => $content
		);

		return $returnArray;
	}

	/**
	  * Returns the Usage As Property box as a string.
	  *
	  * @return string
	  */
	public function getUsageAsPropertyBox() {
		$values = array();
		$uses   = array();
		$empty  = true;

		if (!$this->session->usageBoxDisabled) {
			foreach ($this->model->findSubjects($this->resource, 'Instance', 0, 5, &$erg) as $s) {
				$values[] = '<a about="' . $s->getUri() . '" href="' . OntoWiki_Util::getURL($s) . '">' . $s->getTitle() . '</a>';
				$empty = false;
			}
			
			if (count($values)) {
			    $uses[] = $this->strings->instance->usage->instances . $this->view->makeList($values, 'ul', 'bullets-none separated-horizontal');
			}

			// if ($erg > 5) {
			// 	$usage_box->addString('<br /><a href="index.php?class=' .
			// 	                      urlencode($s->getType()->getQualifiedName()) . '">Show&nbsp;all&nbsp;' .
			// 	                      $erg . '&nbsp;&gt;&gt;</a>');
			// }

			$values = array();

			foreach ($this->model->findObjects($this->resource, 'Instance', 0, 5, &$erg) as $s) {
				if ($s instanceof Resource) {
					$values[] = '<a about="' . $s->getUri() . '" href="' . OntoWiki_Util::getURL($s) . '">' .
					            $s->getTitle() . '</a>';
				} // else {
				//                     $values[] = '<a href="' . $this->config->uriBase . 'resource/view/owl%3AThing?select[' .
				//                                 urlencode($this->resource->getQualifiedName()) . ']=' . urlencode($s->getLabel()) . '">' .
				//                                 $s->getLabel() . '</a>';
				//              }
			}
			
			if (count($values)) {
			    $uses[] = $this->strings->instance->usage->values . $this->view->makeList($values, 'ul', 'bullets-none separated-horizontal');
			}

			if (!$empty) {
				$content = $this->view->makeList($uses, 'ul', 'bullets-none separated-vertical', 'clearfloat');
			} else {
				$content = '<p class="messagebox info">' . $this->strings->default->noresults . '</p>';
			}
		}

		$returnArray = array(
			'cssId'        => 'usage',
			'cssClasses'   => ($this->session->usageBoxMinimized ? 'is-minimized' : '') .
			                  ($this->session->usageBoxDisabled ? ' is-disabled' : ''),
			'title'        => $this->strings->instance->usage->title,
			'headingLevel' => 3,
			'content'      => $content
		);

		return $returnArray;
	}



	///////////////////////////////////////////////////////////////////////////
	//                                                                       //
	//                            Editors Boxes                             //
	//                                                                       //
	///////////////////////////////////////////////////////////////////////////

	/**
	  * Returns the Editors Infos box as an array.
	  *
	  * @return string
	  */
	public function getEditorsBox() {
		$instances = array();
		$empty     = true;
		$editors ="";

		if (!$this->session->EditorsBoxDisabled) {

			$versEngine = new Erfurt_Versioning($this->model);
			// needed to recognize localname from user-uris because
			// an Object loaded with $this->model->resourceF($someuseruri) has no information
			// in requirements- (current-)model
			$sysontModel = Zend_Registry::get('sysontModel');
			
			// This is used to get the Creator from the OntoWiki-versioning Engine
			$this->view->firstMod   = $versEngine->getFirstModifiedForResource($this->resource);
		

			// Get the most recent Author
			if ($this->view->firstMod !== false) {


				if ($this->view->firstMod['user']=="") $this->view->firstMod['user']="System";
				else $this->view->firstMod['user'] = $sysontModel->resourceF($this->view->firstMod['user'],false)->getTitle($lang);
	
				// nice up the date format
				$this->view->firstMod['date'] = Erfurt_Util::dateDifference($this->view->firstMod['date'], false, 3); 
				

				// This is used to get the latest Modifification from the OntoWiki-versioning Engine
				$this->view->lastMod    = $versEngine->getLastModifiedForResource($this->resource);
				if ($this->view->lastMod['user']=="") $this->view->lastMod['user']="System";
				else $this->view->lastMod['user'] = $sysontModel->resourceF($this->view->lastMod['user'], false)->getTitle($lang);

				// nice up the date format
				$this->view->lastMod['date'] = Erfurt_Util::dateDifference($this->view->lastMod['date'], false, 3);
				

				// check whether first author = last author
				if (($this->view->lastMod['user'] === $this->view->firstMod['user'])
				&& ($this->view->lastMod['date'] === $this->view->firstMod['date'])) $this->view->lastMod = false;

			}


			// This was used to get all Mofications from the Ontowiki-Store
			$allMods = $versEngine->getFullHistoryForResource($this->resource);

			$editors="";
			$firstlast=$this->view->firstMod['user'].$this->view->lastMod['user'];
			foreach ($allMods as $mod) {
				if ($mod['user']=="") $m="System"; else
					$m = $sysontModel->resourceF($mod['user'], false)->getTitle($lang);
				if ((strpos($editors,$m)===false)and(strpos($firstlast,$m)===false)) {
					$editors= $editors."<li>".$m."</li>";
				}
			}


			//$all = $this->view->makeList($editors, 'ul', 'bullets-none separated-vertical', 'clearfloat');

			$content="Erstellt ".$this->view->firstMod['date']." durch ".$this->view->firstMod['user']."<br/>";
			//if ($content=="Erstellt vor ca. 374 Tagen durch Admin<br/>") $content="Wurde vor ca. 374 Tagen importiert.<br/>";
			if ($this->view->lastMod) {
				$content = $content. $this->strings->instance->editors->lastchange." ".$this->view->lastMod['date']." ".$this->strings->instance->editors->by." ".$this->view->lastMod['user']."<br/>";
			}
			if ($editors!="") $content=$content."<hr/>".$this->strings->instance->editors->more.":<ul>".$editors."</ul>";

			$returnArray = array(
				'cssId'        => 'editors',
				'cssClasses'   => ($this->session->EditorsBoxMinimized ? 'is-minimized' : '') .
					          ($this->session->EditorsBoxDisabled ? ' is-disabled' : ''),
				'title'        => $this->strings->instance->editors->title,
				'headingLevel' => 3,
				'content'      => $content
			);

		};
		return $returnArray;
	}



}

?>
