<?php

/**
  * Static class providing OntoWiki support methods.
  *
  * @version $Id: Util.php 919 2007-04-01 22:48:13Z nheino $
  */
class OntoWiki_Util {
	
	public function getPagerHtml($start, $itemsPerPage, $itemsTotal) {
		$session = Zend_Registry::get('session');
		
		$prevHtml = '&laquo;&nbsp;Previous';
		$nextHtml = 'Next&nbsp;&raquo;';
		
		$firstHtml = '&lsaquo;&nbsp;First';
		$lastHtml  = 'Last&nbsp;&rsaquo;';
		
		$ret = array();
		
		if ($itemsTotal > $itemsPerPage) {
			if ($start > 0) {
				$ret[] = '<a href="' . Erfurt_Util::replaceUrlParam('start', 0) . '">' . $firstHtml . '</a>';
				$ret[] = '<a href="' . Erfurt_Util::replaceUrlParam('start', ($start - $itemsPerPage)) . '">' . $prevHtml . '</a>';
			// } else {
			// 	$ret[] = $firstHtml;
			// 	$ret[] = $prevHtml;
			}
		}
		
		for ($i = max($start / $itemsPerPage - 2, 0); $i < min($start / $itemsPerPage + 3, $itemsTotal / $itemsPerPage); $i++) {
			if ($start == $i * $itemsPerPage) {
				$ret[] = '<a class="selected" href="">' . ($i + 1) . '</a> ';
			} else {
				$ret[] = '<a href="' . Erfurt_Util::replaceUrlParam('start', $i * $itemsPerPage) . '">' . ($i + 1) . '</a>';
			}
		}
		
		if ($start + $itemsPerPage < $itemsTotal) {
			$ret[] = '<a href="' . Erfurt_Util::replaceUrlParam('start', $start + $itemsPerPage) . '">' . $nextHtml . '</a>';
			$ret[] = '<a href="' . Erfurt_Util::replaceUrlParam('start', ($itemsTotal - $itemsTotal % $itemsPerPage)) . '">' . $lastHtml . '</a>';
		// } else {
		// 	$ret[] = $nextHtml;
		// 	$ret[] = $lastHtml;
		}
		
		return $ret;
	}
	
	public function isValidEmail($string) {
		if (!is_string($string)) {
			throw new Erfurt_Exception('E-mail to be checked needs to be a string!');
		} else {
			if (preg_match('/^([a-zA-Z0-9\._\+-]+)\@((\[?)[a-zA-Z0-9\-\.]+\.([a-zA-Z]{2,7}|[0-9]{1,3})(\]?))$/', $string) > 0) {
				return true;
			} else {
				return false;
			}
		}
	}

	/**
	  * Returns a title string.
	  *
	  * @param $node
	  * @param $includeComment
	  * @param $maxWordLength
	  * @deprecated
	  * @return
	  */
// TODO remove
	public function getTitle($node, $includeComment = true, $maxWordLength = 0) {
		
		$session = Zend_Registry::get('session');
		$config = Zend_Registry::get('config');
		$model = $config->activeModel;
		
		$args = func_get_args();
		
		if (isset($session->modelLanguage)) {
			if ($session->modelLanguage == 'none') {
				unset($session->modelLanguage);
			}
			
			$args['lang'] = $session->modelLanguage;
		}
		
		// intransparent shit cache
		$c = new stmCache('OntoWiki_Util::getTitle', $args, $model);
		
		$cVal = $c->get();
		
		if (null !== $cVal) {
			return $cVal;
		}
			
		// check for non-resource nodes
		if ($node instanceof Literal) {
			return $maxWordLength ? OntoWiki_Util::shorten($node->getLabel(), $maxWordLength) : $node->getLabel();
		} else if ($node instanceof BlankNode) {
			return;
		} else if (!($node instanceof Resource)) {
			return;
		}
		
		// include comments according to language
		if ($includeComment) {
			$comment = isset($session->modelLanguage) ? 
					$node->getComment($session->modelLanguage) :
					$node->getComment('en');

			$comment = $comment ? $comment : $node->getComment();
			$comment = $comment ? $comment->getLabel() : '';

			if (strlen($comment) > 200)
				$comment = substr($comment, 0, 200) . ' &hellip;';
		}
		
		// if it's a title property display it accordingly
		foreach ($config->titleProperties->toArray() as $title) {
			// if language set in session use it
			if (isset($session->modelLanguage) && ($ret = $node->getLiteralPropertyValue($title, $session->modelLanguage))) {
				$label = $ret->getLabel($session->modelLanguage);
				$step = 1;
			// else use anonymous labels (w/o lang tag)
			} elseif ($ret = $node->getLiteralPropertyValue($title, '')) {
				$label = $ret->getLabel();
				$step = 2;
			// if still nothing found, try english labels
			} elseif ($ret = $node->getLiteralPropertyValue($title, 'en')) {
				$label = $ret->getLabel('en');
				$step = 3;
			}
			if ($label) {
				break;
			}
		}
		
		// echo 'lang: ' . $session->modelLanguage . ' title: ' . $label . ' (' . $step . ')<br />';
		
		if (!$label) {
			$label = $node->getQualifiedName();
		}
		
		// encode as entities
		$label = htmlentities($label, ENT_NOQUOTES, $config->encoding);
		
		// shorten if necessary
		if ($maxWordLength && !strpos($label, ' ') && strlen($label) > $maxWordLength) {
			$label = OntoWiki_Util::shorten($label, $maxWordLength);
		}

		$ret = $comment ? ('<span title="' . htmlentities($comment, ENT_NOQUOTES, $config->encoding) . '" class="help">' . $label . '</span>') : $label;
		$c->set($ret, $node);
		
		return $ret;
	}
	
	/**
	  * Shortens a string by splitting it in the middle and returning
	  * beginning and end joined by an ellipse (&helip;).
	  *
	  * The total length of the returned string does not exceed $maxLength.
	  *
	  * @param $label the string to be shortened
	  * @param $maxLength the maximum number of chars the returned string may contain
	  * @return a string that constists of the beginning and end of $label but less than $maxLength in length
	  */
	public function shorten($label, $maxLength) {
		if($maxLength == 0 || strlen($label) < $maxLength)
			return $label;
		else {
			$offset = floor($maxLength/2);
			return rtrim(substr($label, 0, $offset), '.').'&hellip;'.ltrim(substr($label, -$offset, $offset), '.');
		}
	}
	
	/**
	  * Highlights search expressions within text.
	  *
	  * @param $text the text selection to be highlighted.
	  * @param $search the search expression to be highlighted.
	  * @return a &lt;span&gt;-tag of class 'searchHighlight' containing the text to be highlighted.
	  */
	function highlightText($text, $search) {
		
		$arText = array_merge(
			array($search),
			array(str_replace(' ','',$search)),
			explode (' ', $search)
		);
		
		$i = 0;
		$match = array();
		$notFound = true;
		
		while ($notFound && $i <= sizeof($arText) ) {
			$notFound  = !preg_match('/'.$arText[$i].'/i',$text,$match);
			$i++;
		}
		
		foreach ( $match as $strText ) {
			$text =  str_replace($strText,'<span class="highlighted">' . $strText . '</span>',$text);
		}
		
		if (strlen($text)>150) {
			$intSpanStart = strpos($text,'<span');
			$intSpanStop = strrpos($text,'span>');
			if ($intSpanStart > 10)
				$text = '... ' . substr($text,$intSpanStart - 5,$intSpanStop - $intSpanStart + 10) . ' ...';
			else
				$text = substr($text,$intSpanStart,$intSpanStop - $intSpanStart + 10) . ' ...';
		}
				
		return $text;
		
	}
	
	/**
	  * Lists comments loaded from statements table.
	  *
	  * @param $subject
	  * @param $predicate
	  * @param $start
	  * @param $count
	  * @param $erg
	  * @return
	  */
	function listComments($subject = '', $predicate = '', $start = 0, $count = 10, $erg = 0) {
		$session = Zend::Registry('session');
		$config = Zend::Registry('config');
		$model = $config->activeModel;
		$ret = array();
		
		// TODO: fix commenting
		return array();
		
		$sql = 'SELECT * FROM statements s
				INNER JOIN log_statements ls USING(subject,predicate,object,l_language,l_datatype,subject_is,object_is)
				INNER JOIN log_actions la ON(ls.action_id=la.id AND la.model_id=s.modelID)'.
                ($subject || $predicate ?
					'INNER JOIN statements ss ON(1 '.
					($subject ?
						'AND ss.subject=\''.$model->_dbId($subject).'\'' :
						'').
						($predicate ?
							'AND ss.predicate=\''.$model->_dbId($predicate).'\'' :
						'').' AND CONCAT(\'reified://\',ss.id)=s.subject)' :
					'').'WHERE 1 '.
					($model ?
					'AND s.modelID IN ('.$model->getModelIds().')' :
					'').'AND ls.ar=\'a\' AND s.subject like \'reified://%\' AND s.predicate=\''.
							$GLOBALS['RDFS_comment']->getURI().'\' ORDER BY date DESC';

		$rs = $config->erfurt->dbConn->PageExecute($sql, $count, $start/$count+1);
		$erg = $rs->_maxRecordCount ? $rs->_maxRecordCount : $rs->_numOfRows;

		if ($rs) foreach($rs->getArray() as $row) {
			$stm = $model->_convertRowToStatement(array_slice($model->store->dbConn->getRow('
					SELECT * FROM statements WHERE id=\''.str_replace('reified://', '', $row[28]).'\''), 0, 8));

			$stm->reified = $model->_convertRowToStatement(array_slice($row, 0, 8));
			$stm->user = $row[15];
			$stm->added = $row[16];
			$ret[] = $stm;
		}
		return $ret;
	}
	
	/**
	  * Renders comment related html
	  *
	  * @param $resource The resource for which to render
	  */
	public function resourceComment($resource) {
		$config = Zend_Registry::get('config');
		
		if ($s = $resource->getModel()->findStatement($resource, EF_RDF_TYPE, NULL)) {
			if ($r = $resource->getModel()->findStatement('reified://' . $s->dbId, 'rdfs:comment', NULL)) {
				$ret .= '<a href="discussion.php?model=' . 
					urlencode($s->subj->model->modelURI) . 
					'&amp;instance=' . urlencode($s->subj->getQualifiedName()) . 
					'&amp;predicate='.urlencode($s->pred->getQualifiedName()) . 
					'" title="' . $r->obj->getLabel() . '"><img src="'.$config->themeBase.'images/comment.png" alt="comment" /></a>&nbsp;';
			}
		}
		// $ret .= '<a href="javascript:powl.winopen(\'' . 
		// 	urlencode(OntoWiki_Util::nodeEditLink('reified://' . 
		// 	$s->dbId,'rdfs:comment')) . '\',\'comment_edit\',\'width=350,height=400\');" 
		// 		title="Add comment" class="comment" name="comment" 
		// 		style="display:none"><img src="'.$config->themeBase.'images/comment-add.png" alt="comment-add" /></a>';
		
		return $ret;
	}
	
	/**
	  * Returns a URL for the current instance.
	  *
	  * @param $instance the currently selected instance.
	  * @param $view
	  * @param $queryString
	  */
	public function getURL($instance, $model = null, $action = 'view', $queryString = '') {
		$session = Zend_Registry::get('session');
		$config = Zend_Registry::get('config');
		
		if (!$model && $config->activeModel) {
			$model = $config->activeModel;
		}
		
		// no params in uri yet
		$uri_param = false;
		
		// instantiate if necesarry
		if (!($instance instanceof Erfurt_Rdfs_Resource) && $model) {
			$instance = $model->resourceF($instance);
		}
		
		// check if the instance is imported in current model (primarily used for search)
		if ($config->activeModel && in_array($instance->getModel()->modelURI,$config->activeModel->listImports(true))) {
			$inImports = true;
		} else {
			$inImports = false;
		}
		
		$local      = $instance->getLocalNameFast();
		$instBase   = str_replace($local, '', $instance->getURI());
		$namespaces = $model->getParsedNamespaces();
		
		if ($inImports) {
			$url = $instance->getURI();
		} else if ($instBase == $model->getBaseURI()) {
		    // use local named URL
		    $url = $local ? $local : $instance->getURI();
		} else if (array_key_exists($instBase, $namespaces)) {
		    // use prefixed URL
		    $url = $namespaces[$instBase] . ':' . $local;
		} else {
		    // use full URI
		    $url = $instance->getURI();
		}
		
		// use the ?r= parameter syntax if the value contains 
		// characters that confuse apache's rewrite engine
		if (preg_match('/\[|\]|\//', $url) || $inImports) {
			$ret = $config->uriBase . "resource/$action/?r=" . urlencode($url);
			$uri_param = true;
		} else {
			$ret = $config->uriBase . "resource/$action/" . urlencode($url);
		}
		
		// add model parameter if instance's model != currently selected model
		// and there isn't already a model parameter in the queryString
		if ((($config->activeModel->modelURI != $instance->getModel()->modelURI) || !$model ) && !strstr($queryString, 'm=')
			&& !$inImports) {
			$ret .= ($uri_param ? '&amp;m=' : '?m=') . urlencode($instance->getModel()->modelURI);
			$uri_param;
		}
		
		// add query string if any
		if ($queryString != '') {
			$ret .= ($uri_param ? '&amp;' : '?') . $queryString;
		}
		
		return $ret;
	}
	
	/**
	  * Returns a URL for the current instance (absolute with ?r and ?m parameters).
	  *
	  * @param $instance the currently selected instance.
	  * @param $model
	  * @param $action
	  * @param $queryString
	  */
	public function getURLabsolute($instance, $model, $action = 'view', $queryString = '') {
		$session = Zend_Registry::get('session');
		$config = Zend_Registry::get('config');		
		
		// no params in uri yet
		$uri_param = false;
		
		// instantiate if necesarry
		if (!($instance instanceof Erfurt_Rdfs_Resource) && $model) {
			$instance = $model->resourceF($instance);
		}
		
		$local      = $instance->getLocalNameFast();
		$instBase   = str_replace($local, '', $instance->getURI());
		$namespaces = $model->getParsedNamespaces();

		$url = $instance->getURI();
		
		// use the ?r= parameter syntax if the value contains 
		// characters that confuse apache's rewrite engine
		$ret = $config->uriBase . "resource/$action/?r=" . urlencode($url);
		$uri_param = true;
		
		// add model parameter if instance's model != currently selected model
		// and there isn't already a model parameter in the queryString
		$ret .= ($uri_param ? '&amp;m=' : '?m=') . urlencode($instance->getModel()->modelURI);
		
		// add query string if any
		if ($queryString != '') {
			$ret .= ($uri_param ? '&amp;' : '?') . $queryString;
		}
		
		return $ret;
	}
	
	/**
	 * Returns an img-tag with the current resource's average rating obtained from the data base.
	 *
	 * @param $resource the resource whose rating should to be retrieved.
	 */
	public function getRating($resource) {
		$erfurtStore = Zend_Registry::get('erfurt')->getStore();
		$config = Zend_Registry::get('config');
		
		$sql = 'SELECT AVG(rating), COUNT(*) FROM ratings WHERE modelID="'.
			$resource->model->modelID . '" AND ef_resource="' . $resource->getURI() . '" GROUP BY modelID,ef_resource';
		list($avgRating, $numRatings) = $erfurtStore->dbConn->getRow($sql);

		if ($numRatings) {
			return '<img src="' . $config->themeBase . 
				'images/stars/ur_star_' . round($avgRating) . 
				($avgRating - floor($avgRating) > 0.5 && $avgRating - floor($avgRating) < 0.75 ? 'a' : '') . 
				'.png" class="ratingAvg" alt="star" />&nbsp;('.$numRatings.')';
		}
	}
	
	/**
	  * Displays objects.
	  *
	  * @param $s
	  * @param $allowEdit
	  * @param $maxValueLength
	  * @return
	  */
	public function displayObject($s, $allowEdit = true, $maxValueLength = 0) {
		$config = Zend_Registry::get('config');
		$session = Zend_Registry::get('session');
		
		if ($allowEdit) {
			$del = '<a class="inline-edit"><img class="clickicon drop-statement" id="statement-' . $s->dbId . '" src="' . 
			        $config->themeBase . 'images/icon-delete.png" alt="delete" /></a>' . PHP_EOL;
		}
		
		$ret = '<span id="stmId' . $s->dbId . '" class="statement">' . PHP_EOL;
		if (is_a($s->obj,'resource')) {
			// image resources
			if (in_array($s->pred->getURI(), $config->imgLinkProperties->toArray())) {
				$ret .= '<img src="' . $s->obj->getURI() . '" alt="contentIMG" name="contentIMG" />' . $del;
			// weblink resources
			} elseif (in_array($s->pred->getURI(), $config->webLinkProperties->toArray())) {
				$ret .= '<a href="' . $s->obj->getURI() . '">' . $s->obj->getTitle($session->modelLanguage) . '</a>' . $del;
			// other resources
			} elseif (in_array($s->pred->getURI(), $config->mailtoProperties->toArray())) {
				$ret .= '<a about="' . $s->obj->getUri() . '" href="' . $s->obj->getQualifiedName() . '">' . $s->obj->getQualifiedName() . '</a>' . $del;
			} else {
				// $ret .= '<img src="' . $config->themeBase . 'images/plus.gif" alt="closed" title="Expand" onclick="ow.expand(\'inst-' . 
				//         $s->dbId . '\',this,\'' . urlencode($s->obj->getQualifiedName()) . '\')" style="cursor:auto" />' . PHP_EOL;
				// TODO: check if statements exist faster
				if (!($s->obj instanceof BlankNode) && ($s->obj->hasPropertyValue(null, null))) {
					$classAttrib = ' class="expandable"';
				}
				$ret .= '<a about="' . $s->obj->getUri() . '"' . $classAttrib . ' href="' . OntoWiki_Util::getURL($s->obj) . '">' . 
				        $s->obj->getTitle($session->modelLanguage) . '</a>' . $del . '<span id="inst-' . $s->dbId . '"></span>' . PHP_EOL;
			}
		} elseif (is_a($s->obj, 'literal')) {
			// image literals
			if (in_array($s->pred->getURI(), $config->imgLinkProperties->toArray()) /*|| preg_match('/(\.(jpg|gif|jpeg|png)$)/i', $s->obj->getLabel())*/) {
				$ret .= '<img src="' . $s->obj->getLabel() . '" alt="contentIMG" name="contentIMG" />' . PHP_EOL;
			// weblink literals
			} elseif (in_array($s->pred->getURI(), $config->webLinkProperties->toArray()) /*|| strpos($s->obj->getLabel(), 'http://') === 0*/) {
				$ret .= '<a href="'.$s->obj->getLabel().'">'.$s->obj->getLabel().'</a>' . PHP_EOL;
			// other literals
			} else {
				if ($maxValueLength) {
					$ret .= nl2br(OntoWiki_Util::shorten(strip_tags($s->obj->getLabel()), $maxValueLength));
				} else {
					$ret .= nl2br(htmlentities($s->obj->getLabel(), ENT_NOQUOTES, $config->encoding));
					// $ret .= strstr($s->obj->getLabel(), '<') ? $s->obj->getLabel() : nl2br($s->obj->getLabel());
				}
			}
			
			$ret .= $del;
		}

		if ($r=$s->subj->getModel()->findStatement('reified://'.$s->dbId, 'rdfs:comment', NULL)) {
			$ret .= '&nbsp;<a href="discussion.php?model='.
				urlencode($s->subj->model->modelURI).
				'&amp;instance='.
				urlencode($s->subj->getQualifiedName()).
				'&amp;predicate='.
				urlencode($s->pred->getQualifiedName()).
				'" title="'.
				$r->obj->getLabel().
				'"><img src="images/comment.png" alt="comment" /></a>';
		}

		$ret .= '</span><!-- stmId' . $s->dbId . ' -->' . PHP_EOL;

		return $ret;
	}
	
	/**
	  * Renders an instance as shown on the properties page
	  *
	  * @param $instance The instance to be rendered.
	  * @param $allowEdit
	  * @param $maxValueLength
	  * @return A string that can be sent to a web browser for rendering an instance.
	  */
	public function renderInstance($instance, $allowEdit = true, $maxValueLength = 0, $tableId = null, $noCaption = false) {
		$config = Zend_Registry::get('config');
		$model = $config->activeModel;
		$session = Zend_Registry::get('session');
		
		// allow edit
		if ($allowEdit && $model->isEditable()) {
			$cssClass = ' editable';
		} else {
			$cssClass = ' non-editable';
			$allowEdit = false;
		}
		
		// instantiate resource if necessarry
		if (!($instance instanceof Erfurt_Rdfs_Resource)) {
			$instance = $model->resourceF($instance, true);
		}
		
		$ret  = '<table cellspacing="0" class="separated-vertical edittable' . $cssClass . '"' . ($tableId ? (' id="table-' . $tableId . '"') : '') . '>' . PHP_EOL;
		if (!$noCaption) {
			$ret .= '<caption>Properties for ' . $instance->getTitle($session->modelLanguage) . '</caption>' . PHP_EOL;
		}
		
		$ret .= '<tbody>' . PHP_EOL;

		foreach ($instance->getModel()->findPredicates($instance, null) as $p) {
			if ($p->getQualifiedName() != 'rdf:type') {
				// if ($allowEdit) {
				// 	$attributes = ' ondblclick="editRow(this, event, \'' . 
				// 		urlencode($instance->getQualifiedName()) . '\', \'' . 
				// 		urlencode($p->getQualifiedName()) . '\')"';
				// }
				
				$attributes = ' about="' . urlencode($instance->getQualifiedName()) . '" property="' . urlencode($p->getQualifiedName()) . '"';

				$ret .= '<tr>' . PHP_EOL . '<td>'. 
						'<a href="'.OntoWiki_Util::getURL($p).'">'.$p->getTitle($session->modelLanguage).'</a></td>' . PHP_EOL . '<td' . $attributes . '>';

				$pvs = $instance->getModel()->findStatements($instance, $p, null, 0, 10, &$erg);

				$ret .= $erg > 1 ? ('<ul>' . PHP_EOL) : '';
				
				foreach ($pvs as $s) {
					$ret .= ($erg > 1 ? ('<li>' . PHP_EOL) : '') . 
					        OntoWiki_Util::displayObject($s, $allowEdit, $maxValueLength) . 
					        ($erg > 1 ? ('</li>' . PHP_EOL) : '');
				}
				$ret .= $erg > 1 ? ('</ul>' . PHP_EOL) : '';
				
				if ($allowEdit) {
					$ret .= '<a class="inline-edit"><img class="clickicon inline-edit" src="' . $config->themeBase . 'images/icon-edit.png" alt="edit" /></a>' . PHP_EOL;
				}

				if ($erg > 10) {
					$ret .= '<a href="' . $config->uriBase . 'resource/statements/' . urlencode($instance->getQualifiedName()) . 
							'?p=' . urlencode($p->getQualifiedName()) . '">Show all ' . $erg . ' &gt;&gt;</a>' . PHP_EOL;
				}
				
			$ret .= '</td>' . PHP_EOL . '</tr>' . PHP_EOL;
			}
		}
		
		$ret .= '</tbody>' . PHP_EOL . '</table>' . PHP_EOL . OntoWiki_Util::resourceComment($instance);
		
		return $ret;
	}
	
	public function editValues($class, $property, $value = '', $formElemPrefix = 'prop') {		
		$config = Zend_Registry::get('config');
		$model = $config->activeModel;
		
		$p = $model->propertyF($property);
		
		$wf = Zend_Registry::get('widgetFactory');
		$widgetHtml = $wf->getWidgetHtml($model->getClass($class), $p, $value, array('name' => $formElemPrefix));
		
		return $widgetHtml;
	}
	
	public function deleteStatement($id) {
		$config = Zend_Registry::get('config');
		$model = $config->activeModel;
		
		$model->remove($id);
	}

	public function editInstance($instance, $property) {
		Zend_Registry::get('owLog')->debug('editInstance: ' . $instance . ', ' . $property);
		
		$config = Zend_Registry::get('config');
		$model = $config->activeModel;
		
		$instance = $model->instanceF($instance);
		
		return OntoWiki_Util::editValues($instance->getClass(),
						                 $property,
						                 $instance->listPropertyValues($property),
						                 'inst[' . $instance->getUri() . ']');
	}
	
	/**
	  * Attention!--crap ahead!
	  */
	public function nodeEditLink($subject, $predicate, $object = '') {
		$config = Zend_Registry::get('config');

		return $config->erfurtUriBase . 'lib/modules/rdf/node.php?model='.urlencode($config->activeModel->modelURI).
				'&amp;subj='.urlencode(is_a($subject,'resource')?$subject->getQualifiedName():$subject).
				'&amp;pred='.urlencode(is_a($predicate,'resource')?$predicate->getQualifiedName():$predicate).
				'&amp;obj[value]='.urlencode(is_a($object,'node')?$object->getLabel():$object).
				(is_a($object,'literal')?'&amp;obj[type]=literal&amp;obj[lang]='.urlencode($object->getLanguage()).
				'&amp;obj[dtype]='.urlencode($object->getDatatype()):'');
	}
	
	public function getColLinks($cols, $maxCols = 5) {
		$colLinks = array();
		
		if ($maxCols > 5) {
			$maxCols = 5;
		}
		
		if ($maxCols > 1) {
			for ($i = 1; $i <= $maxCols; $i++) {
				if ($i != $cols) {
					$colLinks[] = '<a href="' . Erfurt_Util::replaceUrlParam('cols', $i) . '">' . $i . '</a>';
				} else {
					$colLinks[] = '<a class="selected" href="">' . $i . '</a>';
				}
			}
		}
		
		return $colLinks;
	}
}

?>
