<?php
/**
 * DL-Learner plugin controller
 *
 * @author Vu Duc Minh
 * @version $Id$
 */

class ResourceController extends OntoWiki_Controller_Action {
	
	private $_filters = null;
	
	private $_filtersCombined = false;
	

	private $client;
	
	// ID given to this client by the web service
	private $id;
	private $wsdluri;
	// ID of the DBpedia knowledge source
	private $ksID;
	private $OntowikiUrl;
	/**
	  * Initialisation method.
	  */
	public function init() {
		parent::init();

		// redirect if no model has been selected
		if (!($this->model instanceof RDFSModel)) {
			$this->_redirect('');
			return;
		}
		
		if ($this->model instanceof RDFSModel) {
			try {
				if ($config->activeResource instanceof Erfurt_Rdfs_Resource ||
				isset($this->session->activeResourceUri) ) {
					$this->resource = $this->model->instanceF($this->session->activeResourceUri);
				} else if (isset($this->_request->r)) {
					$this->resource = $this->model->instanceF($this->_request->r);
				}

				// check if resource is a class
				// and forward to list view
				if ($this->resource->isClass()) {
					$this->class = $this->model->classF($this->resource);
					$this->session->activeClassUri = $this->class->getURI();
				}
				
				
			} catch (Exception $e) {
				$this->log->debug('Uncatched Exception: ' . __CLASS__ . ':' .$e->message . PHP_EOL . $e->file . ':' . $e->line);
			}
		} 
	}
    
    function dlLearnerConnectionAction($id = 0, $ksID = 0)
	{
		ini_set('default_socket_timeout',200);
	//	$config = Zend_Registry::get('config');
		
		$this->OntowikiUrl='http://localhost/ontowiki/service/sparql';
	//	echo $this->_getOntowikiBaseUri();
		$this->client=new SoapClient("plugins/Dllearner/main.wsdl",array('features' => SOAP_SINGLE_ELEMENT_ARRAYS));//new SoapClient("plugins/Dllearner/main.wsdl");
		$this->id=$id;
		$this->ksID=$ksID;
	}
	
	function getIDsAction()
	{
		$id=$this->client->generateID();
		$ksID=$this->client->addKnowledgeSource($id,"sparql",$this->OntowikiUrl);
		return array(0 => $id, 1 => $ksID);
	}
	function getConceptFromExamplesDefAction($posExamples,$negExamples)
	{
		
		$this->client->applyConfigEntryInt($this->id, $this->ksID, "recursionDepth",2);
		$this->client->applyConfigEntryStringArray($this->id, $this->ksID, "instances", array_merge($posExamples,$negExamples));
		
		$this->client->setReasoner($this->id, "fastInstanceChecker");
		if(empty($negExamples)){
			$negExamples=$this->client->getNegativeExamples($this->id,$this->ksID,$posExamples,count($posExamples),"http://3ba.se/conferences/");
			$negExamples=$negExamples->item;
		}
		
		$this->client->setReasoner($this->id, "fastInstanceChecker");
		if(empty($negExamples))
			$this->client->setLearningProblem($this->id, "posOnlyDefinition");
		else
			$this->client->setLearningProblem($this->id, "posNegDefinition");
		$this->client->setPositiveExamples($this->id, $posExamples);
	
			$this->client->setNegativeExamples($this->id, $negExamples);
		$la_id=$this->client->setLearningAlgorithm($this->id, "refexamples");
		$this->client->applyConfigEntryStringArray($this->id, $la_id, "ignoredConcepts", array($this->class->getUri(),'http://www.w3.org/2002/07/owl#Thing'));
		
		
		$this->client->initAll($this->id);

		$threaded=true;
	
	    if($threaded == false) {
	
			$concept = $this->client->learn($this->id,'manchester');
			
		} else {
		
			$this->client->learnThreaded($this->id);
			
			$i = 1;
			$sleeptime = 1;
			$learnttl=30000000;
			do {
				// sleep a while
				sleep($sleeptime);
				
				// see what we have learned so far
				
				$running=$this->client->isAlgorithmRunning($this->id);
				
				$seconds = $i * $sleeptime;
				
				$i++;
				
			
			} while($seconds<$this->ttl&&$running);
			
			$this->client->stop($this->id);
		}
		return $this->client->getCurrentlyBestEvaluatedDescriptions($this->id,4);
		
	}
	
	function getConceptFromExamplesIncAction($posExamples,$negExamples)
	{
		
		$this->client->applyConfigEntryInt($this->id, $this->ksID, "recursionDepth",2);
		$this->client->applyConfigEntryStringArray($this->id, $this->ksID, "instances", array_merge($posExamples,$negExamples));
		
		if(empty($negExamples)){
			$negExamples=$this->client->getNegativeExamples($this->id,$this->ksID,$posExamples,count($posExamples),"http://3ba.se/conferences/");
			$negExamples=$negExamples->item;
		}
		
		$this->client->setReasoner($this->id, "owlapi");
		if(empty($negExamples))
			$this->client->setLearningProblem($this->id, "posOnlyInclusion");
		else
			$this->client->setLearningProblem($this->id, "posNegInclusion");
		$this->client->setPositiveExamples($this->id, $posExamples);
	
			$this->client->setNegativeExamples($this->id, $negExamples);
		$la_id=$this->client->setLearningAlgorithm($this->id, "refexamples");
		$this->client->applyConfigEntryStringArray($this->id, $la_id, "ignoredConcepts", array($this->class->getUri(),'http://www.w3.org/2002/07/owl#Thing'));
		
		
		$this->client->initAll($this->id);

		$threaded=true;
	
	    if($threaded == false) {
	
			$concept = $this->client->learn($this->id,'manchester');
			
		} else {
		
			$this->client->learnThreaded($this->id);
			
			$i = 1;
			$sleeptime = 1;
			$learnttl=30000000;
			do {
				// sleep a while
				sleep($sleeptime);
				
				// see what we have learned so far
				//$concepts=$this->client->getCurrentlyBestEvaluatedDescriptions($this->id,4);
				$running=$this->client->isAlgorithmRunning($this->id);
				
				$seconds = $i * $sleeptime;
				
				$i++;
				
			
			} while($seconds<$this->ttl&&$running);
			
			$this->client->stop($this->id);
		}
		return $this->client->getCurrentlyBestEvaluatedDescriptions($this->id,4);
		
	}
	public function learndefAction() {
	    $this->_strings = Zend_Registry::get('strings');
        $identity     = Zend_Auth::getInstance()->getIdentity();
		
		// Add the path of templates needed by this plugin
		$this->view->addScriptPath(REAL_BASE.'plugins/Dllearner/templates/resource/');
		
		// Needed here because of the automatic rendering should be deactivated
		$this->_helper->viewRenderer->setNoRender(true);
         if($identity['anonymous']){
		$_SESSION['idOff'] = '<p class="message error">' . $this->_strings->dl->message->error->id . '</p>';
			header('HTTP/1.0 400 Bad Request');
			echo $this -> view -> render('learndef.php');
			return;
		}
		
		// fetch filters
		$this->fetchFiltersAction();
		
		if ($this->class || $this->resource->getQualifiedName() == 'owl:Thing') {
			if ($this->resource->getQualifiedName() != 'owl:Thing') {
				$instances = $this->class->findInstancesRecursive($this->_filters, 'exact', $start, $this->session->count, &$end);
			} else {
				$instances = $this->model->findInstances($this->_filters, 'exact', $start, $this->session->count, &$end);
		 	}
		}
		$this -> dlLearnerConnectionAction();
		$ids = $this -> getIDsAction();
		$idss = $ids[0];
		$ksIDss = $ids[1];
		$this -> dlLearnerConnectionAction($idss,$ksIDss);
		$posExamples = array();
        $posE = ""; 		 
		 foreach ($instances as $inst){
           $in = $inst->getUri();
           $posE .= $in."<br>" ;
		   array_push($posExamples, $in);
          }  
		//print_r($posE);
		//$negExamples = $this->client->getNegativeExamples($this->id,$this->ksID,$posExamples,count($posExamples),"http://localhost/ontowiki-1/resource");
		$negExamples = array();
		$_SESSION['startat'] = date(H .':'. i .':'. s);
		$concept = $this -> getConceptFromExamplesDefAction($posExamples,$negExamples);
	//	$_SESSION['startat'] = date(H .':'. i .':'. s);
		$this->view->concept        = $concept;
	//	print_r($this->view->concept);
	//	print_r($concept);
		
	    $this->view->posE = $posE;
		$this->view->uri            = $this->resource->getUri();
		$this->view->title   = "Concept for class <b>".$this->getSuperClassPath()."</b>";
	    echo $this -> view -> render('learndef.php');
	}
	
	public function learnincAction() {
	    $this->_strings = Zend_Registry::get('strings');
        $identity     = Zend_Auth::getInstance()->getIdentity();
		// Disable layout (we want only one or more boxes no full ontowiki layout)
		//$this->_helper->layout->disableLayout();
		
		// Add the path of templates needed by this plugin
		$this->view->addScriptPath(REAL_BASE.'plugins/Dllearner/templates/resource/');
		
		// Needed here because of the automatic rendering should be deactivated
		$this->_helper->viewRenderer->setNoRender(true);
        if($identity['anonymous']){
		$_SESSION['idOff'] = '<p class="message error">' . $this->_strings->dl->message->error->id . '</p>';
			header('HTTP/1.0 400 Bad Request');
			echo $this -> view -> render('learninc.php');
			return;
		}
		
		// fetch filters
		$this->fetchFiltersAction();
		
		if ($this->class || $this->resource->getQualifiedName() == 'owl:Thing') {
			if ($this->resource->getQualifiedName() != 'owl:Thing') {
				$instances = $this->class->findInstancesRecursive($this->_filters, 'exact', $start, $this->session->count, &$end);
			} else {
				$instances = $this->model->findInstances($this->_filters, 'exact', $start, $this->session->count, &$end);
		 	}
		}
		$this -> dlLearnerConnectionAction();
		$ids = $this -> getIDsAction();
		$idss = $ids[0];
		$ksIDss = $ids[1];
		$this -> dlLearnerConnectionAction($idss,$ksIDss);
		$posExamples = array();
        $posE = ""; 		 
		 foreach ($instances as $inst){
           $in = $inst->getUri();
           $posE .= $in."<br>" ;
		   array_push($posExamples, $in);
          }  
		//print_r($posE);
		//$negExamples = $this->client->getNegativeExamples($this->id,$this->ksID,$posExamples,count($posExamples),"http://localhost/ontowiki-1/resource");
		$negExamples = array();
		$_SESSION['startat'] = date(H .':'. i .':'. s);
		$concept = $this -> getConceptFromExamplesIncAction($posExamples,$negExamples);
	//	$_SESSION['startat'] = date(H .':'. i .':'. s);
		$this->view->concept        = $concept;
	//	print_r($this->view->concept);
	//	print_r($concept);
		
	    $this->view->posE = $posE;
		$this->view->uri            = $this->resource->getUri();
		$this->view->title   = "Concept for class <b>".$this->getSuperClassPath()."</b>";
	    echo $this -> view -> render('learninc.php');
	}
	
	
	
	
	/**
	  * Fetches filters from URI
	  */
	public function fetchFiltersAction() {
		// only old-school OntoWiki filters supported by now
		if (isset($this->_request->select)) {
			$this->_filters = array_filter($this->_request->select);
		} else {
			$this->_filters = array();
		}
		
		if (isset($this->_request->combine) && $this->_request->combine == 'on') {
			$this->_filtersCombined = true;
		} else {
			$this->_filtersCombined = false;
		}
	}
	
	
	
	
}

?>
